/*
 * $Id: Mp3Data.java,v 1.1 2003/04/14 20:03:51 stranger Exp $
 * Copyright (c) 2003-2003 business solutions AG, Hauerstrasse 12,
 * D-66299 Friedrichsthal, Germany. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of infor business solutions AG ("Confidential Information").  
 * You shall not disclose such Confidential Information and shall 
 * use it only in accordance with the terms of the license 
 * agreement you entered into with infor AG.
 */

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.swing.JOptionPane;

import cdox.data.AudioData;
import cdox.edit.Cover;
import cdox.edit.TextElementGenerator;
import de.hampelratte.id3.ID3v1Tag;
import de.hampelratte.id3.ID3v2Tag;
import de.hampelratte.id3.MP3File;

/**
 * Mp3Data
 *
 * @author jung
 * @version $Revision: 1.1 $
 * @since 22.03.2003
 */
public class Mp3Data extends AudioData {

   protected List tracks = new java.util.ArrayList();
   protected List trackLength = new java.util.ArrayList();

   /**
    * generates a prefix for the track at depth
    * @return
    */
   protected String generateTrackPrefix(int depth) {
      String trackName = "";
      for (int count = 0; count < depth; count++) {
         trackName += " ";
      }
      return trackName;
   }

   /**
    * adds a directory to the tracklist
    * @param depth
    * @param dir
    */

   public void addDirectory(int depth, File dir) {
      tracks.add(generateTrackPrefix(depth) + dir.getName()+" (dir)");
      trackLength.add(new Date(0));
   }

   /**
    * adds a file to the tracklist
    * @param realFile
    * @throws Exception
    */

   public void addFile(int depth, File realFile, Mp3FileSourceDialogue dia) {
      try {
         MP3File file = new MP3File(realFile.getCanonicalPath(), "r");
         if (file.hasID3v1Tag()) {
            ID3v1Tag tag = file.readID3v1Tag();
            String track = generateTrackPrefix(depth) + tag.getArtist()+" : " + tag.getTitle();
            tracks.add(track);
            trackLength.add(new Date(0));
            return;
         } else if(file.hasID3v2Tag()) {
            ID3v2Tag tag = file.readID3v2Tag();
            String track = generateTrackPrefix(depth) + tag.getArtist()+" : " + tag.getTitle();
            tracks.add(track);
            trackLength.add(new Date(0));
            return;
         }

      } catch (Exception e) {
         e.printStackTrace();
      }

      if (JOptionPane
         .showConfirmDialog(
            dia,
            "Could not access id3 tags of file "
               + realFile.getName()
               + " Should I use the file name instead?")
         == JOptionPane.YES_OPTION) {
         tracks.add(generateTrackPrefix(depth) + realFile.getName());
         trackLength.add(new Date(0));
      }
   }

   /**
    * make track list consistent with parent
    * representation
    */
   protected void consist() {
      setTracks(
         (java.util.ArrayList) tracks,
         (java.util.ArrayList) trackLength);
   }

   /**
    * This method will insert the title, subtitle, artist, tracks and the description.
    */
   public void fillBooklet(Cover c) throws IllegalArgumentException {
      if (c.getType() != CDBOOKLET)
         throw new IllegalArgumentException("Wrong cover type.");
      int[] size = CDBOOKLET_SIZE;
      SimpleDateFormat format = new SimpleDateFormat("mm:ss");
      String[] strings = new String[tracks.size() / 2];
      for (int i = 0; i < strings.length; i++) {
         StringBuffer sb = new StringBuffer();
         sb.append((i + 1) + " - ");
         sb.append(tracks.get(i));
         Date d = (Date) trackLength.get(i);
         String s = " (" + format.format(d) + ")";
         if (!s.equals(" (00:00)"))
            sb.append(s);
         strings[i] = sb.toString();
      }
      TextElementGenerator.createMultipleStrings(
         c,
         strings,
         null,
         10,
         10,
         true,
         false,
         false,
         false);
      String[] secondStrings = new String[tracks.size() - strings.length];
      for (int i = strings.length; i < tracks.size(); i++) {
         StringBuffer sb = new StringBuffer();
         sb.append((i + 1) + " - ");
         sb.append(tracks.get(i));
         Date d = (Date) trackLength.get(i);
         String s = " (" + format.format(d) + ")";
         if (!s.equals(" (00:00)"))
            sb.append(s);
         secondStrings[i - strings.length] = sb.toString();
      }
      TextElementGenerator.createMultipleStrings(
         c,
         secondStrings,
         null,
         10,
         10,
         false,
         false,
         false,
         false);
   }

   /* (non-Javadoc)
    * @see cdox.data.Data#getDescription()
    */
   public String getDescription() {
      return "MP3 CD";
   }

   /**
    * @return List
    */
   public List getTracks() {
      return tracks;
   }

}
