package cdox.data;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.util.*;
import java.awt.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * Represents the data for an audio CD.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version July 3rd 2002
 */
public class AudioData implements Data,CDCoverStandards{

    // other fields
    private String title="",artist="",subtitle="",description="";
    // track names:
    private ArrayList tracks=new ArrayList();
    // Date objects reside herein:
    private ArrayList trackLength=new ArrayList();

    public boolean userFillData( Frame parent ){
	AudioDataSheet sheet=new AudioDataSheet(parent);
	sheet.setValues(title,subtitle,artist,description,(String[])tracks.toArray(new String[tracks.size()]),
			(Date[])trackLength.toArray(new Date[trackLength.size()]));
	sheet.show();
	if(sheet.whichButtonPressed()!=-1)//user didn't klick the cancel button
	    sheet.getValues(this);
	if(sheet.whichButtonPressed()==0){//user klicked fill covers
	    //if user filled some fields, it's worth filling the cover(s)
	    try{		
		Cover[] c;
		if(((CDoxFrame)parent).isWizardOn())
		    c = ((CDoxFrame)parent).getWizardInstance().getWizardCovers();
		else
		    c=((CDoxFrame)parent).getEditPane().getCovers();
		for(int i=0;i<c.length;i++){
		    switch(c[i].getType()){
		    case CDFRONT:
			fillFront(c[i]);
			break;
		    case CDBACK_SIDE:
			fillBack(c[i]);
			break;
		    case CDBOOKLET:
			fillBooklet(c[i]);
			break;
		    }
		    if(!((CDoxFrame)parent).isWizardOn()){
			((CDoxFrame)parent).setSave(true);
			((CDoxFrame)parent).repaint();
		    }
		}
	    }
	    catch(IllegalArgumentException iae){
		CDoxFrame.handleError(iae,true);
	    }
	}
	return sheet.whichButtonPressed()!=-1;
    }

    public int getSupportedVersion(){
	return 1;
    }

    public void storeData( org.w3c.dom.Element e ){
	Document doc=e.getOwnerDocument();
	// setting the type
	e.setAttribute("type","AudioData");
	org.w3c.dom.Element f=e.getOwnerDocument().createElement("audiocd");
	e.appendChild(f);
	e=f;
	// adding the artist element
	org.w3c.dom.Element n=e.getOwnerDocument().createElement("artist");
	n.appendChild(doc.createTextNode(artist));
	e.appendChild(n);
	// adding the title element
	n=e.getOwnerDocument().createElement("title");
	n.appendChild(doc.createTextNode(title));
	e.appendChild(n);
	// adding the subtitle element
	if(!subtitle.equals("")){
	    n=e.getOwnerDocument().createElement("subtitle");
	    n.appendChild(doc.createTextNode(subtitle));
	    e.appendChild(n);
	}
	// adding the description element
	if(!description.equals("")){
	    n=e.getOwnerDocument().createElement("description");
	    String s="";
	    try{
		s=new String(Base64.encode(description.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    n.appendChild(doc.createTextNode(s));
	    e.appendChild(n);
	}
	// adding the tracks
	if(tracks.size()>0){
	    // gives the XML duration format:
	    SimpleDateFormat durationFormat=new SimpleDateFormat("'PT'm'M's'S'");
	    n=e.getOwnerDocument().createElement("tracks");
	    for(int i=0;i<tracks.size();i++){
		org.w3c.dom.Element t=(org.w3c.dom.Element)n.appendChild(doc.createElement("track"));
		t.setAttribute("number",(i+1)+"");
		t.setAttribute("title",tracks.get(i).toString());
		t.setAttribute("length",durationFormat.format((Date)trackLength.get(i)));
	    }
	    e.appendChild(n);
	}
    }

    public void reconstruct( org.w3c.dom.Element desc ){
	// simple: just checks for all text and inserts the values.
	setTitle(desc.getElementsByTagName("title").item(0).getChildNodes().item(0).getNodeValue());
	setArtist(desc.getElementsByTagName("artist").item(0).getChildNodes().item(0).getNodeValue());
	NodeList nl=desc.getElementsByTagName("subtitle");
	if(nl.getLength()!=0) setSubtitle(nl.item(0).getChildNodes().item(0).getNodeValue());
	nl=desc.getElementsByTagName("description");
	if(nl.getLength()!=0){
	    String text=nl.item(0).getChildNodes().item(0).getNodeValue();
	    try{
		text=new String(Base64.decode(text.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    setDescription(text);
	}
	nl=desc.getElementsByTagName("tracks");
	if(nl.getLength()!=0){ // we have some tracks
	    SimpleDateFormat format=new SimpleDateFormat("m'M's'S'");
	    nl=((org.w3c.dom.Element)nl.item(0)).getElementsByTagName("track");
	    String[] tracks=new String[nl.getLength()];
	    Date[] length=new Date[nl.getLength()];
	    for(int i=0;i<nl.getLength();i++){
		    NamedNodeMap atts=nl.item(i).getAttributes();
		    int ind=Integer.parseInt(atts.getNamedItem("number").getNodeValue())-1;
		    tracks[ind]=atts.getNamedItem("title").getNodeValue();
		    String du=atts.getNamedItem("length").getNodeValue();
		    int pp=du.indexOf("H");
		    if(pp==-1) pp=du.indexOf("T");
		    pp++;
		    length[ind]=format.parse(du.substring(pp),new ParsePosition(0));
	    }
	    ArrayList t=new ArrayList();
	    ArrayList l=new ArrayList();
	    for(int i=0;i<tracks.length;i++){
		t.add(tracks[i]);
		l.add(length[i]);
	    }
	    setTracks(t,l);
	}
    }

    public String getDescription(){
	return CDox.getLocalizer().get("audiocd");
    }

    /**
     * Sets the title.
     *@param s the new title.
     */
    public void setTitle( String s ){
	title=s;
    }

    /**
     * Sets the subtitle.
     *@param s the new subtitle.
     */
    public void setSubtitle( String s ){
	subtitle=s;
    }

    /**
     * Sets the artist.
     *@param s the new artist.
     */
    public void setArtist( String s ){
	artist=s;
    }

    /**
     * Sets the description.
     *@param s the new description.
     */
    public void setDescription( String s ){
	description=s;
    }

    /**
     * Sets the tracks.
     *@param tracks the track names.
     *@param lengths the lengths of the tracks as <code>Date</code>s.
     */
    public void setTracks( ArrayList tracks, ArrayList lengths ){
	this.tracks.clear();
	trackLength.clear();
	this.tracks.addAll(tracks);
	trackLength.addAll(lengths);
    }

    /**
     * This method will insert a (big) title, a (not so big) subtitle and the artist (as
     * big as title).
     */
    public void fillFront( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDFRONT) throw new IllegalArgumentException("Wrong cover type.");
	addFrontElements(c,0,CDFRONT_SIZE[0]);
    }

    /**
     * This method will insert a list of the tracks and sides with title and artist.//check if title is null
     */
    public void fillBack( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDBACK_SIDE) throw new IllegalArgumentException("Wrong cover type.");
	if(tracks.size()!=0) addTrackElement(c);
	StringBuffer sb = new StringBuffer();
	if(!artist.equals("")){
	    sb.append(artist);
	    if(!title.equals(""))
		sb.append(" - " + title);
	}
	else if(!artist.equals(""))
	    sb.append(artist);
	TextElementGenerator.createSideString(c,sb.toString());
    }

    /**
     * This method will insert the title, subtitle, artist, tracks and the description.
     */
    public void fillBooklet( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDBOOKLET) throw new IllegalArgumentException("Wrong cover type.");
	addTrackElement(c);
	addFrontElements(c,CDBOOKLET_SIZE[0]/2,CDBOOKLET_SIZE[0]/2);
    }

    // Fills the front side for a, well, front side or a booklet. The "left" parameter is
    // the offset to the left, the xsize is the xsize that is used to calculate the
    // elements to the middle.
    private void addFrontElements( Cover c, int left, int xsize ){
	int pos=50; // this is the cumulative position where a new element may start
		    // (y-axis)
	if(!artist.equals(""))//check if artist is null
	    pos+=TextElementGenerator.createSingleString
		(c,artist,null,24,pos,false,false,false,false).getSize().getHeight()+5;
	if(!title.equals(""))//check if title is null
	    pos+=TextElementGenerator.createSingleString
		(c,title,null,18,pos,false,false,false,false).getSize().getHeight()+5;
	if(!subtitle.equals(""))//check if subtitle is null
	    TextElementGenerator.createSingleString
		(c,subtitle,null,14,pos,false,false,true,false);
    }

    // Adds the tracks to the cover. The cover's type should be a back side or a booklet.
    private void addTrackElement( Cover c ){
	int[] size;
	String[] strings=new String[tracks.size()];
	if(c.getType()==CDBACK_SIDE) size=CDBACK_SIDE_SIZE;
	else size=CDBOOKLET_SIZE;
	SimpleDateFormat format=new SimpleDateFormat("mm:ss");
	for(int i=0;i<tracks.size();i++){
	    StringBuffer sb=new StringBuffer();
	    sb.append((i+1)+" - ");
	    sb.append(tracks.get(i));
	    Date d=(Date)trackLength.get(i);
	    String s=" ("+format.format(d)+")";
	    if(!s.equals(" (00:00)")) sb.append(s);
	    strings[i]=sb.toString();
	}
	TextElementGenerator.createMultipleStrings
	    (c,strings,null,14,20,true,false,false,false);
    }

}
