package cdox.data;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This class is used by AudioData to display the audio CD data.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 */
public class AudioDataSheet extends JDialog implements FocusListener{

    private JTextField title=new JTextField(),artist=new JTextField(),subtitle=new JTextField();
    private JTextArea description=new JTextArea();
    private Localizer lang=CDox.getLocalizer();
    private ArrayList tracknames=new ArrayList(),tracklength=new ArrayList();
    private Vector listData=new Vector();
    private SimpleDateFormat format=new SimpleDateFormat("mm:ss");
    private JButton addButton,okButton;
    private int buttonPressed=-1;
    private JList tracks=new JList();

    /**
     * Constructs one.
     * @param parent the parent frame.
     */
    public AudioDataSheet( Frame parent ){
	super(parent,CDox.getLocalizer().get("properties"),true);
	// setting some properties of the components
	title.setColumns(20);
	subtitle.setColumns(20);
	artist.setColumns(20);
	description.setRows(5);
	description.setLineWrap(true);
	description.setWrapStyleWord(true);
	// adding the components
	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	gb.anchor=gb.WEST;
	gb.insets=new Insets(2,2,2,2);
	getContentPane().add(new JLabel(lang.get("adartist")+":"),gb);
	gb.gridx=1;
	gb.anchor=gb.EAST;
	getContentPane().add(artist,gb);
	gb.gridy=1;
	getContentPane().add(title,gb);
	gb.gridx=0;
	gb.anchor=gb.WEST;
	getContentPane().add(new JLabel(lang.get("adtitle")+":"),gb);
	gb.gridy=2;
	getContentPane().add(new JLabel(lang.get("adsubtitle")+":"),gb);
	gb.gridx=1;
	gb.anchor=gb.EAST;
	getContentPane().add(subtitle,gb);
	gb.anchor=gb.CENTER;
	gb.fill=gb.BOTH;
	gb.gridwidth=2;
	gb.gridx=0;
	gb.gridy=3;
	// sub panel for track adding follows:
	JPanel pnl=new JPanel();
	pnl.setBorder(new TitledBorder(new EtchedBorder(),lang.get("adtracks")));
	pnl.setLayout(new GridBagLayout());
	// creating the needed components, they are final to be referred in the action
	// listeners
	final JTextField name=new JTextField(lang.get("adtrackname"));
	name.addFocusListener(this);
	final JIntField min=new JIntField(0,0,99);
	min.addFocusListener(this);
	final JIntField sec=new JIntField(0,0,59);
	sec.addFocusListener(this);
	// for the time
	GridBagConstraints g=new GridBagConstraints();
	g.fill=g.BOTH;
	g.insets=new Insets(1,1,1,1);
	min.setColumns(2);
	pnl.add(min,g);
	g.gridx=1;
	pnl.add(new JLabel(":"),g);
	g.gridx=2;
	sec.setColumns(2);
	pnl.add(sec,g);
	// for the track name
	g.gridwidth=2;
	g.weightx=1;
	g.gridx=3;
	pnl.add(name,g);
	// the buttons/list
	g.gridy=1;
	g.gridwidth=4;
	g.gridheight=5;
	g.gridx=0;
	tracks.addListSelectionListener(new ListSelectionListener(){
		public void valueChanged( ListSelectionEvent evt ){
		    int index=tracks.getSelectedIndex();
		    if(index!=-1){
			name.setText(tracknames.get(index).toString());
			Date d=(Date)tracklength.get(index);
			SimpleDateFormat f=new SimpleDateFormat("m");
			min.setValue(Integer.parseInt(f.format(d)));
			f=new SimpleDateFormat("s");
			sec.setValue(Integer.parseInt(f.format(d)));
		    }
		}
	    });
	tracks.setValueIsAdjusting(false);
	JScrollPane panel=new JScrollPane(tracks);
	pnl.add(panel,g);
	panel.setPreferredSize(panel.getPreferredSize());
	panel.setMinimumSize(panel.getPreferredSize());
	panel.setMaximumSize(panel.getPreferredSize());
	panel.setSize(panel.getPreferredSize());
	g.gridheight=1;
	g.gridx=4;
	g.gridwidth=1;
	g.fill = g.HORIZONTAL;
	g.anchor = g.NORTH;
	JButton button=new JButton(lang.get("change"));
	button.setPreferredSize(new Dimension(100, 23));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=tracks.getSelectedIndex();
		    if(pos!=-1){
			tracknames.remove(pos);
			tracklength.remove(pos);
			tracknames.add(pos,name.getText());
			tracklength.add(pos,new Date(min.getValue()*1000*60+sec.getValue()*1000));
			constructVector();
		    }
		}
	    });
	pnl.add(button,g);
	g.gridy=2;
	addButton=new JButton(lang.get("add"));
	addButton.setPreferredSize(new Dimension(100, 23));
	addButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    tracknames.add(name.getText());
		    tracklength.add(new Date(min.getValue()*1000*60+sec.getValue()*1000));
		    constructVector();
		    min.setValue(0);
		    sec.setValue(0);		    
		    if(name.hasFocus())
			name.selectAll();
		    else 
			name.requestFocus();
		}
	    });
	pnl.add(addButton,g);
	g.gridy=3;
	button=new JButton(lang.get("remove"));
	button.setPreferredSize(new Dimension(100, 23));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=tracks.getSelectedIndex();
		    if(pos!=-1){
			tracknames.remove(pos);
			tracklength.remove(pos);
			constructVector();
		    }
		}
	    });
	pnl.add(button,g);
	g.gridy=4;
	button=new JButton(lang.get("adupbutton"));
	button.setPreferredSize(new Dimension(100, 23));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=tracks.getSelectedIndex();
		    if(pos==0) return;
		    if(pos!=-1){
			Object o1=tracknames.remove(pos);
			Object o2=tracklength.remove(pos);
			tracknames.add(pos-1,o1);
			tracklength.add(pos-1,o2);
			constructVector();
			tracks.setSelectedIndex(pos-1);
		    }
		}
	    });
	pnl.add(button,g);
	g.gridy=5;
	button=new JButton(lang.get("addownbutton"));
	button.setPreferredSize(new Dimension(100, 23));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=tracks.getSelectedIndex();
		    if((pos+1)==tracknames.size()) return;
		    if(pos!=-1){
			Object o1=tracknames.remove(pos);
			Object o2=tracklength.remove(pos);
			tracknames.add(pos+1,o1);
			tracklength.add(pos+1,o2);
			constructVector();
			tracks.setSelectedIndex(pos+1);
		    }
		}
	    });
	pnl.add(button,g);
	getContentPane().add(pnl,gb);
	// description
	gb.gridy=4;
	panel=new JScrollPane(description);
	panel.setBorder(new TitledBorder(new EtchedBorder(),lang.get("addescription")));
	getContentPane().add(panel,gb);
	JPanel buttonPanel = new JPanel(new GridBagLayout());
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridx=0;
	gbc.weightx = 1f;
	gbc.fill=gb.NONE;
	gbc.anchor=gb.CENTER;
	gbc.insets = new Insets(0,3,0,3);
	int maxWidth=0;
	okButton=new JButton(lang.get("okbutton"));
	if(maxWidth<okButton.getPreferredSize().width)
	    maxWidth=okButton.getPreferredSize().width;
	okButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=1;
		    setVisible(false);
		    dispose();
		}
	    });
	getRootPane().setDefaultButton(okButton);
	buttonPanel.add(okButton,gbc);
	gbc.gridx=1;
	button=new JButton(lang.get("fillbutton"));
	button.setToolTipText(lang.get("fillbuttontip"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=0;
		    setVisible(false);
		    dispose();
		}
	    });
	buttonPanel.add(button, gbc);
	gbc.gridx=2;
	button=new JButton(lang.get("cancelbutton"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=-1;
		    setVisible(false); 
		    dispose();
		}
	    });
	for(int i=0; i<buttonPanel.getComponentCount();i++){
	    ((JButton)buttonPanel.getComponent(i)).setPreferredSize
		(new Dimension(maxWidth, button.getPreferredSize().height));
	}
	new EscapeListener(button,getRootPane());
	buttonPanel.add(button,gbc);
	gb.gridy=5;
	gb.weightx=0f;
	gb.fill=gb.BOTH;
	gb.gridwidth=3;
	gb.gridx=0;
	getContentPane().add(buttonPanel,gb);
	// other
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
    }

    /**
     * Returns 1 if ok is pressed -1 for Cancel, 0 for fill.
     * @return 1 for ok, -1 for cancel, 0 for fill.
     */
    public int whichButtonPressed(){
	return buttonPressed;
    }

    // if the user gets into the track name/minute/second fields, enter adds the new
    // track, otherwise, the dialog is closed by the ok button
    public void focusGained( FocusEvent evt ){
	getRootPane().setDefaultButton(addButton);
	((JTextComponent)evt.getSource()).selectAll();
    }

    public void focusLost( FocusEvent evt ){
	getRootPane().setDefaultButton(okButton);
	((JTextComponent)evt.getSource()).setCaretPosition(0);
	
    }

    public void show(){
	// the following should not be necessary if SUN WOULD CODE CORRECTLY.
	setResizable(true);
	constructVector();
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
	super.show();
    }

    /**
     * Fills in the values.
     */
    public void setValues( String title, String subtitle, String artist, String description,
			   String[] tracks, Date[] lengths ){
	this.artist.setText(artist);
	this.title.setText(title);
	this.subtitle.setText(subtitle);
	this.description.setText(description);
	for(int i=0;i<tracks.length;i++){
	    tracknames.add(tracks[i]);
	    tracklength.add(lengths[i]);
	}
	// the following should not be necessary if SUN WOULD CODE CORRECTLY.
	setResizable(true);
	constructVector();
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
    }

    /**
     * Fills the current values in this dialog into the data. If the user pressed cancel
     * the last time, no values are touched in the data object.
     *@param data where to fill the data.
     */
    public void getValues( AudioData data ){
	data.setTitle(title.getText());
	data.setArtist(artist.getText());
	data.setSubtitle(subtitle.getText());
	data.setDescription(description.getText());
	data.setTracks(tracknames,tracklength);
    }

    // reconstructs the vector with the list data from the array lists.
    private void constructVector(){
	listData.clear();
	Iterator i=tracknames.iterator();
	Iterator j=tracklength.iterator();
	int cnt=1;
	while(i.hasNext()){
	    Date d=(Date)j.next();
	    String ds=" ("+format.format(d)+")";
	    if(ds.equals(" (00:00)")) ds="";
	    listData.add(cnt+" - "+i.next()+ds);
	    cnt++;
	}
	tracks.setListData(listData);
    }

}
