package cdox.data;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;

public class FileSourceDialog extends JDialog{

    private JFileChooser chooser;
    private GenericData data=null;
    protected Localizer lang=CDox.getLocalizer();
    protected JCheckBox onlyDirs, recursive, limit, sizes, fullPath;
    protected JTextField dir;
    private JIntField limitNumber;
    private final char treeBuilder= '\u00b9';

    /**
     * Constructs a modal one with the specified frame as parent.
     *@param parent the parent.
     */
    public FileSourceDialog( Frame parent ){
	super(parent,true);
	setTitle(lang.get("fetchproperties"));
	Preferences prefs=Preferences.userNodeForPackage(FileSourceDialog.class);
	chooser=new JFileChooser(prefs.get("dir",System.getProperty("user.home")));
	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	gb.gridx=0;
	gb.fill=gb.HORIZONTAL;
	gb.gridy=0;
	gb.insets=new Insets(2,2,2,2);
	gb.anchor=gb.WEST;
	gb.weightx=1;
	gb.gridwidth=3;
	onlyDirs=new JCheckBox(lang.get("gdonlydirs"));
	onlyDirs.setSelected(prefs.getBoolean("onlydirs",true));
	getContentPane().add(onlyDirs,gb);
	gb.gridy++;
	fullPath=new JCheckBox(lang.get("gdfullpath"));
	fullPath.setSelected(prefs.getBoolean("fullpath",false));
	getContentPane().add(fullPath,gb);
	gb.gridy++;
	recursive=new JCheckBox(lang.get("gdrecursive"));
	recursive.setSelected(prefs.getBoolean("recursive",false));
	getContentPane().add(recursive,gb);
	gb.gridy++;
	limit=new JCheckBox(lang.get("gdlimit"));
	limit.setSelected(prefs.getBoolean("limit",false));
	getContentPane().add(limit,gb);
	limitNumber=new JIntField(prefs.getInt("limitnumber",20),1,100);
	limitNumber.setEnabled(limit.isSelected());
	limit.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    limitNumber.setEnabled(limit.isSelected());
		}
	    });
	gb.gridx=3;
	gb.gridwidth=1;
	getContentPane().add(limitNumber,gb);
	gb.gridwidth=3;
	gb.gridx=0;
	gb.gridy++;
	sizes=new JCheckBox(lang.get("gdsizes"));
	sizes.setSelected(prefs.getBoolean("sizes",true));
	getContentPane().add(sizes,gb);
	gb.gridy++;
	getContentPane().add(new JLabel(lang.get("gddir")),gb);
	gb.gridy++;
	gb.gridwidth=3;
	gb.anchor=gb.CENTER;
	dir=new JTextField(prefs.get("dir","/cdrom"));
	getContentPane().add(dir,gb);
	gb.gridwidth=1;
	gb.gridx=3;
 	gb.weightx=0;
	JButton button;
	getContentPane().add(button=new JButton(lang.get("browse")),gb);
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    selectDir();
		}
	    });
	gb.gridx=0;
	gb.gridy++;
	gb.weightx=1;
	getContentPane().add(new JLabel(),gb);
	gb.gridx=1;
	gb.weightx=0;
	getContentPane().add(button=new JButton(lang.get("okbutton")),gb);
	getRootPane().setDefaultButton(button);
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    storePreferences();
		    createData();
		}
	    });
	gb.gridx=2;
	getContentPane().add(button=new JButton(lang.get("cancelbutton")),gb);
	new EscapeListener(button,getRootPane());
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    setVisible(false);
		    dispose();
		}
	    });
	pack();
	setResizable(false);
	setLocationRelativeTo(null);
    }

    // stores everything in the prefs
    private void storePreferences(){
	Preferences prefs=Preferences.userNodeForPackage(FileSourceDialog.class);
	prefs.putBoolean("onlydirs",onlyDirs.isSelected());
	prefs.putBoolean("fullpath",fullPath.isSelected());
	prefs.putBoolean("recursive",recursive.isSelected());
	prefs.putBoolean("limit",limit.isSelected());
	prefs.putInt("limitnumber",limitNumber.getValue());
	prefs.put("dir",dir.getText());
	try{
	    prefs.flush();
	}catch(BackingStoreException bse){ // and again...
	    ErrorHandler.log(bse,true);
	}
    }

    // shows a FileChooser and lets select a dir
    private void selectDir(){
	chooser.setFileSelectionMode(chooser.DIRECTORIES_ONLY);
	if(chooser.showOpenDialog(this)==chooser.APPROVE_OPTION)
	    dir.setText(chooser.getSelectedFile().getPath());
    }

    // does the magic.
    protected void createData(){
	data=new GenericData();
	ArrayList list=new ArrayList();
	getStrings(dir.getText(),list, -1);
	if(list.size()>30){
	    if(JOptionPane.showConfirmDialog(this,lang.get("gdtoomuch",new String[]{list.size()+""}))==
	       JOptionPane.NO_OPTION) return;
	}
	data.setStrings(list);
	data.setTitle(fullPath.isSelected()?dir.getText():new File(dir.getText()).getName());
	setVisible(false);
	dispose();
    }

    // fills the list according to the options
    protected void getStrings( String dir, ArrayList list, int depth){
	depth++;
	File f=new File(dir);
	if(!f.exists()) return; // should possibly not happen
	// if there is a limit and it is exceeded, return:
	if(limit.isSelected()&&(list.size()>=limitNumber.getValue())) return;
	// if we search recursive or this is our first level, check for new strings
	if(recursive.isSelected()||dir.equals(this.dir.getText())){
	    File[] files=f.listFiles();
	    for(int i=0;i<files.length;i++){
		if(limit.isSelected()&&(list.size()>=limitNumber.getValue())) return;
		if((!onlyDirs.isSelected())||(files[i].isDirectory())){
		    processFile(list, depth, files[i]);
		    if(files[i].isDirectory()&&recursive.isSelected()){
			getStrings(files[i].getPath(),list, depth);
		    }
		}
	    }
	}
    }
    
    protected void processFile(ArrayList list, int depth, File file) {
	StringBuffer sb=new StringBuffer();
	for(int j=0;j<depth;j++)
	    sb.append("   ");
	if(depth>0)
	    sb.append("+-");
      
	sb.append(fullPath.isSelected()?file.getPath():file.getName());
	if(sizes.isSelected()&&!file.isDirectory()){
	    double length=file.length();
	    String b=" bytes";
	    if(length>1024){
		length/=1024;
		b="KB";
	    }
	    if(length>1024){
		length/=1024;
		b="MB";
	    }
	    if(length>1024){
		length/=1024;
		b="GB";
	    }
	    DecimalFormat frm=new DecimalFormat("#.#");
	    sb.append(" ("+frm.format(length)+b+")");
	}
	list.add(sb.toString());
    }

    /**
     * Shows the dialog and returns the resulting data.
     *@return the data or null, if the user cancelled the dialog.
     */
    public Data showAndGetData(){
	show();
	return data;
    }

}
