package cdox.data;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.util.*;
import java.awt.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * Represents the data for an audio CD.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 28th 2002
 */
public class GenericData implements Data,CDCoverStandards{

    private String title="";
    // the list with strings
    private ArrayList strings=new ArrayList();

    public boolean userFillData( Frame parent ){
	GenericDataSheet sheet=new GenericDataSheet(parent);
	sheet.setValues(title,(String[])strings.toArray(new String[strings.size()]));
	sheet.show();
	if(sheet.whichButtonPressed()!=-1)
	    sheet.getValues(this);
	if(sheet.whichButtonPressed()==0){
	    try{
		Cover[] c;
		if(((CDoxFrame)parent).isWizardOn())
		    c = ((CDoxFrame)parent).getWizardInstance().getWizardCovers();
		else
		    c=((CDoxFrame)parent).getEditPane().getCovers();
		for(int i=0;i<c.length;i++){
		    switch(c[i].getType()){
		    case CDFRONT:
			fillFront(c[i]);
			break;
		    case CDBACK_SIDE:
			fillBack(c[i]);
			break;
		    case CDBOOKLET:
			fillBooklet(c[i]);
			break;
		    }
		    if(!((CDoxFrame)parent).isWizardOn()){
			((CDoxFrame)parent).setSave(true);
			((CDoxFrame)parent).repaint();
		    }
		}
	    }
	    catch(IllegalArgumentException iae){
		CDoxFrame.handleError(iae,true);
	    }
	}
	return sheet.whichButtonPressed()!=-1;
    }

    public int getSupportedVersion(){
	return 1;
    }

    public void storeData( org.w3c.dom.Element e ){
	Document doc=e.getOwnerDocument();
	// setting the type
	e.setAttribute("type","GenericData");
	org.w3c.dom.Element f=e.getOwnerDocument().createElement("genericcd");
	e.appendChild(f);
	e=f;
	// adding the title
	org.w3c.dom.Element n=e.getOwnerDocument().createElement("title");
	n.appendChild(doc.createTextNode(title));
	e.appendChild(n);
	// adding the strings
	if(strings.size()>0){
	    Iterator i=strings.iterator();
	    while(i.hasNext()){
		org.w3c.dom.Element t=(org.w3c.dom.Element)e.appendChild(doc.createElement("string"));
		String s="";
		try{
		    s=new String(Base64.encode(i.next().toString().getBytes("utf8")),"utf8");
		}catch(UnsupportedEncodingException uee){
		    CDoxFrame.handleError(uee,true);
		}
		t.appendChild(doc.createTextNode(s));
	    }
	}
    }

    public void reconstruct( org.w3c.dom.Element desc ){
	NodeList nl=desc.getElementsByTagName("title");
	if(nl.getLength()!=0) setTitle(nl.item(0).getChildNodes().item(0).getNodeValue());
	nl=desc.getElementsByTagName("string");
	for(int i=0;i<nl.getLength();i++){
	    String text=nl.item(i).getChildNodes().item(0).getNodeValue();
	    try{
		text=new String(Base64.decode(text.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    strings.add(text);
	}
    }

    public String getDescription(){
	return CDox.getLocalizer().get("genericcd");
    }

    /**
     * Sets the title.
     *@param s the new title.
     */
    public void setTitle( String s ){
	title=s;
    }

    /**
     * Sets the strings.
     *@param strings an array list with the strings.
     */
    public void setStrings( ArrayList strings ){
	this.strings.clear();
	this.strings.addAll(strings);
    }

    /**
     * This method will insert the title.
     */
    public void fillFront( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDFRONT) throw new IllegalArgumentException("Wrong cover type.");
	if(!title.equals(""))
	    TextElementGenerator.createSingleString(c,title,null,18,50,false,false,false,false);
    }

    /**
     * This method will insert a list of the strings and the title on the side.
     */
    public void fillBack( Cover c ) throws IllegalArgumentException{
	TextElementGenerator.createMultipleStrings
	    (c,(String[])strings.toArray(new String[strings.size()]),
	     null,10,10,true,false,false,false);
	if(!title.equals(""))
	    TextElementGenerator.createSideString(c,title);
    }

    /**
     * This method will insert the title and the strings on the other side.
     */
    public void fillBooklet( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDBOOKLET) throw new IllegalArgumentException("Wrong cover type.");
	TextElementGenerator.createMultipleStrings
	    (c,(String[])strings.toArray(new String[strings.size()]),
	     null,10,10,true,false,false,false);
	if(!title.equals(""))
	    TextElementGenerator.createSingleString(c,title,null,18,50,false,false,false,false);
    }

}
