package cdox.data;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * This class is used by AudioData to display the audio CD data.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 */
public class GenericDataSheet extends JDialog implements FocusListener{

    private JTextField title=new JTextField();
    private Localizer lang=CDox.getLocalizer();
    private ArrayList stringData=new ArrayList();
    private Vector listData=new Vector();
    private JButton addButton,okButton;
    private int buttonPressed=-1;
    private JList strings=new JList();

    /**
     * Constructs one.
     * @param parent the parent frame.
     */
    public GenericDataSheet( Frame parent ){
	super(parent,CDox.getLocalizer().get("properties"),true);
	// setting some properties of the components
	title.setColumns(20);
	// adding the components
	JPanel upperPanel = new JPanel(new GridBagLayout());
	JPanel rightButtonPanel = new JPanel(new GridBagLayout());
	JPanel lowerButtonPanel = new JPanel(new GridBagLayout());
	JPanel middlePanel = new JPanel(new GridBagLayout());

	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	gb.anchor=gb.WEST;
	gb.insets=new Insets(2,10,2,2);
	upperPanel.add(new JLabel(lang.get("adtitle")+":"),gb);
	gb.insets=new Insets(2,2,2,10);
	gb.gridx=1;
	gb.weightx=1;
	gb.fill=gb.HORIZONTAL;
	upperPanel.add(title,gb);
	gb.anchor=gb.CENTER;
	gb.fill=gb.BOTH;
	gb.insets = new Insets(0,0,0,0);
	gb.weightx=0f;
	gb.gridx=0;
	gb.gridy=0;

	getContentPane().add(upperPanel, gb);

	// sub panel for track adding follows:
	middlePanel.setBorder(new TitledBorder(new EtchedBorder(),lang.get("gdtext")));
	// creating the needed components, they are final to be referred in the action
	// listeners
	final JTextField name=new JTextField(lang.get("gdtext"));
	name.addFocusListener(this);
	// for the track name
	gb.anchor=gb.WEST;
	gb.fill=gb.HORIZONTAL;
       	gb.insets=new Insets(1,1,1,1);
	gb.weightx=1f;
	gb.gridx=0;
	gb.gridwidth=2;
	middlePanel.add(name,gb);
	// the tracks list--.----
	gb.gridy=1;
	gb.gridx=0;
	gb.gridwidth=1;
	strings.addListSelectionListener(new ListSelectionListener(){
		public void valueChanged( ListSelectionEvent evt ){
		    int index=strings.getSelectedIndex();
		    if(index!=-1){
			name.setText(stringData.get(index).toString());
		    }
		}
	    });
	strings.setValueIsAdjusting(false);
	JScrollPane panel=new JScrollPane(strings);
	middlePanel.add(panel,gb);
	panel.setPreferredSize(panel.getPreferredSize());
	panel.setMinimumSize(panel.getPreferredSize());
	panel.setMaximumSize(panel.getPreferredSize());

	//The buttons to adjust the tracks
	gb.anchor = gb.NORTH;
	gb.fill = gb.HORIZONTAL;
	gb.gridx=0;
	gb.gridy=0;
	gb.weighty=1f;
	int maxWidth = 0;

	JButton button=new JButton(lang.get("change"));
	//searching for largest button
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=strings.getSelectedIndex();
		    if(pos!=-1){
			stringData.remove(pos);
			stringData.add(pos,name.getText());
			constructVector();
		    }
		}
	    });
	rightButtonPanel.add(button,gb);
	gb.gridy=1;
	addButton=new JButton(lang.get("add"));
	//searching for largest button
	if(maxWidth<addButton.getPreferredSize().width)
	    maxWidth=addButton.getPreferredSize().width;
	addButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    stringData.add(name.getText());
		    constructVector();
		    name.selectAll();
		}
	    });
	rightButtonPanel.add(addButton,gb);
	gb.gridy=2;
	button=new JButton(lang.get("remove"));
	//searching for largest button
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=strings.getSelectedIndex();
		    if(pos!=-1){
			stringData.remove(pos);
			constructVector();
		    }
		}
	    });
	rightButtonPanel.add(button,gb);
	gb.gridy=3;
	button=new JButton(lang.get("adupbutton"));
	//searching for largest button
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=strings.getSelectedIndex();
		    if(pos==0) return;
		    if(pos!=-1){
			Object o=stringData.remove(pos);
			stringData.add(pos-1,o);
			constructVector();
			strings.setSelectedIndex(pos-1);
		    }
		}
	    });
	rightButtonPanel.add(button,gb);
	gb.gridy=4;
	button=new JButton(lang.get("addownbutton"));
	//searching for largest button
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    int pos=strings.getSelectedIndex();
		    if((pos+1)==stringData.size()) return;
		    if(pos!=-1){
			Object o=stringData.remove(pos);
			stringData.add(pos+1,o);
			constructVector();
			strings.setSelectedIndex(pos+1);
		    }
		}
	    });
	rightButtonPanel.add(button,gb);

	//setting all buttons to best size
	for(int i=0; i<rightButtonPanel.getComponentCount();i++)
	    ((JButton)rightButtonPanel.getComponent(i)).setPreferredSize
		(new Dimension(maxWidth, button.getPreferredSize().height));
	
	//adding the buttonpanel to the middlePanel
	gb.anchor = gb.EAST;
	gb.gridx=1;
	gb.gridy=1;
	middlePanel.add(rightButtonPanel,gb);

	//adding middle panel to the contentpane
	gb.anchor = gb.CENTER;
	gb.gridx = 0;
	getContentPane().add(middlePanel, gb);

	//the lowerbuttonpanel
	maxWidth = 0;
	gb.fill=gb.NONE;
	gb.insets = new Insets(0,1,0,1);
	gb.gridx=0;
	//	gb.weightx = 1f;

	okButton=new JButton(lang.get("okbutton"));
	if(maxWidth<okButton.getPreferredSize().width)
	    maxWidth=okButton.getPreferredSize().width;
	okButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed = 1;
		    setVisible(false);
		    dispose();
		}
	    });
	getRootPane().setDefaultButton(okButton);
	lowerButtonPanel.add(okButton,gb);

	gb.gridx=1;
	button=new JButton(lang.get("fillbutton"));
	button.setToolTipText(lang.get("fillbuttontip"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=0;
		    setVisible(false);
		    dispose();
		}
	    });
	lowerButtonPanel.add(button,gb);
		
	gb.gridx=2;
	button=new JButton(lang.get("cancelbutton"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=-1;
		    setVisible(false);
		    dispose();
		}
	    });
	new EscapeListener(button,getRootPane());
	lowerButtonPanel.add(button,gb);

	//setting all buttons to best size
	for(int i=0; i<lowerButtonPanel.getComponentCount();i++)
	    ((JButton)lowerButtonPanel.getComponent(i)).setPreferredSize
		(new Dimension(maxWidth, button.getPreferredSize().height));

	//adding buttonPanel to the contentpane
	gb.fill=gb.HORIZONTAL;
	gb.gridy=2;
	gb.gridx=0;
	getContentPane().add(lowerButtonPanel,gb);

	// other
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
    }

    /**
     * Returns 1 if ok is pressed -1 for Cancel, 0 for fill.
     * @return 1 for ok, -1 for cancel, 0 for fill.
     */
    public int whichButtonPressed(){
	return buttonPressed;
    }

    // if the user gets into the track name/minute/second fields, enter adds the new
    // track, otherwise, the dialog is closed by the ok button
    public void focusGained( FocusEvent evt ){
	getRootPane().setDefaultButton(addButton);
    }

    public void focusLost( FocusEvent evt ){
	getRootPane().setDefaultButton(okButton);
    }

    public void show(){
	// the following should not be necessary if SUN WOULD CODE CORRECTLY.
	setResizable(true);
	constructVector();
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
	super.show();
    }

    /**
     * Fills in the values.
     */
    public void setValues( String title, String[] strings ){
	this.title.setText(title);
	for(int i=0;i<strings.length;i++) stringData.add(strings[i]);
	// the following should not be necessary if SUN WOULD CODE CORRECTLY.
	setResizable(true);
	constructVector();
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
    }

    /**
     * Fills the current values in this dialog into the data. If the user pressed cancel
     * the last time, no values are touched in the data object.
     *@param data where to fill the data.
     */
    public void getValues( GenericData data ){
	data.setTitle(title.getText());
	data.setStrings(stringData);
    }

    // reconstructs the vector with the list data from the array lists.
    private void constructVector(){
	listData.clear();
	Iterator i=stringData.iterator();
	int cnt=1;
	while(i.hasNext()) listData.add(i.next());
	strings.setListData(listData);
    }

}
