package cdox.data;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.util.*;
import java.awt.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * Represents the data for a video cd.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 */
public class VideoData implements Data,CDCoverStandards{

    // data fields
    private String title="",subtitle="",description="",format="";
    private Date length;

    public VideoData(){
	try{
	    length=new SimpleDateFormat("h:m").parse("0:0");
	}catch(ParseException pe){
	    // won't happen
	}
    }

    public boolean userFillData( Frame parent ){
	VideoDataSheet sheet=new VideoDataSheet(parent);
	sheet.setValues(title,subtitle,format,description,length);
	sheet.show();
	if(sheet.whichButtonPressed()!=-1)
	    sheet.getValues(this);
	if(sheet.whichButtonPressed()==0){
	    try{
		Cover[] c;
		if(((CDoxFrame)parent).isWizardOn())
		    c = ((CDoxFrame)parent).getWizardInstance().getWizardCovers();
		else
		    c=((CDoxFrame)parent).getEditPane().getCovers();
		for(int i=0;i<c.length;i++){
		    switch(c[i].getType()){
		    case CDFRONT:
			fillFront(c[i]);
			break;
		    case CDBACK_SIDE:
			fillBack(c[i]);
			break;
		    case CDBOOKLET:
			fillBooklet(c[i]);
			break;
		    }
		    if(!((CDoxFrame)parent).isWizardOn()){
			((CDoxFrame)parent).setSave(true);
			((CDoxFrame)parent).repaint();
		    }
		}
	    }
	    catch(IllegalArgumentException iae){
		CDoxFrame.handleError(iae,true);
	    }
	}
    	return sheet.whichButtonPressed()!=-1;
    }

    public int getSupportedVersion(){
	return 1;
    }

    public void storeData( org.w3c.dom.Element e ){
	Document doc=e.getOwnerDocument();
	// setting the type
	e.setAttribute("type","VideoData");
	org.w3c.dom.Element el=e.getOwnerDocument().createElement("videocd");
	e.appendChild(el);
	e=el;
	// adding the title element
	org.w3c.dom.Element n=e.getOwnerDocument().createElement("title");
	n.appendChild(doc.createTextNode(title));
	e.appendChild(n);
	// adding the subtitle element
	if(!subtitle.equals("")){
	    n=e.getOwnerDocument().createElement("subtitle");
	    n.appendChild(doc.createTextNode(subtitle));
	    e.appendChild(n);
	}
	// adding the format element
	if(!format.equals("")){
	    n=e.getOwnerDocument().createElement("format");
	    n.appendChild(doc.createTextNode(format));
	    e.appendChild(n);
	}
	// adding the description element
	if(!description.equals("")){
	    n=e.getOwnerDocument().createElement("description");
	    String s="";
	    try{
		s=new String(Base64.encode(description.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    n.appendChild(doc.createTextNode(s));
	    e.appendChild(n);
	}
	// gives the XML duration format:
	SimpleDateFormat f=new SimpleDateFormat("'PT'H'H'm'M'");
	n=e.getOwnerDocument().createElement("length");
	n.appendChild(e.getOwnerDocument().createTextNode(f.format(length)));
	e.appendChild(n);
    }

    public void reconstruct( org.w3c.dom.Element desc ){
	// simple: just checks for all text and inserts the values.
	setTitle(desc.getElementsByTagName("title").item(0).getChildNodes().item(0).getNodeValue());
	NodeList nl=desc.getElementsByTagName("format");
	if(nl.getLength()!=0) setFormat(nl.item(0).getChildNodes().item(0).getNodeValue());
	nl=desc.getElementsByTagName("subtitle");
	if(nl.getLength()!=0) setSubtitle(nl.item(0).getChildNodes().item(0).getNodeValue());
	nl=desc.getElementsByTagName("description");
	if(nl.getLength()!=0){
	    String text=nl.item(0).getChildNodes().item(0).getNodeValue();
	    try{
		text=new String(Base64.decode(text.getBytes("utf8")),"utf8");
	    }catch(UnsupportedEncodingException uee){
		CDoxFrame.handleError(uee,true);
	    }
	    setDescription(text);
	}
	nl=desc.getElementsByTagName("length");
	if(nl.getLength()!=0){
	    SimpleDateFormat format=new SimpleDateFormat("'PT'H'H'm'M'");
	    try{
		setLength(format.parse(nl.item(0).getChildNodes().item(0).getNodeValue()));
	    }catch(ParseException pe){
	    }
	}
    }

    public String getDescription(){
	return CDox.getLocalizer().get("videocd");
    }

    /**
     * Sets the title.
     *@param s the new title.
     */
    public void setTitle( String s ){
	title=s;
    }

    /**
     * Sets the subtitle.
     *@param s the new subtitle.
     */
    public void setSubtitle( String s ){
	subtitle=s;
    }

    /**
     * Sets the format.
     *@param s the new format.
     */
    public void setFormat( String s ){
	format=s;
    }

    /**
     * Sets the description.
     *@param s the new description.
     */
    public void setDescription( String s ){
	description=s;
    }

    /**
     * Sets the length.
     *@param d the new length.
     */
    public void setLength( Date d ){
	length=d;
    }

    /**
     * This method will insert a title and a subtitle.
     */
    public void fillFront( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDFRONT) throw new IllegalArgumentException("Wrong cover type.");
	addFrontElements(c,0,CDFRONT_SIZE[0]);
    }

    /**
     * This method will insert a title at the sides and the description, title and
     * subtitle.
     */
    public void fillBack( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDBACK_SIDE) throw new IllegalArgumentException("Wrong cover type.");
	TextElementGenerator.createSideString(c,title);
	addBackElements(c,CDBACK_SIDE_SIZE[0]);
    }

    /**
     * This method will insert the title and a subtitle as well as the description.
     */
    public void fillBooklet( Cover c ) throws IllegalArgumentException{
	if(c.getType()!=CDBOOKLET) throw new IllegalArgumentException("Wrong cover type.");
	addFrontElements(c,CDBOOKLET_SIZE[0]/2,CDBOOKLET_SIZE[0]/2);
	addBackElements(c,CDBOOKLET_SIZE[0]/2);
    }

    // Fills the front side for a, well, front side or a booklet. The "left" parameter is
    // the offset to the left, the xsize is the xsize that is used to calculate the
    // elements to the middle.
    private void addFrontElements( Cover c, int left, int xsize ){
	int pos=50; // this is the cumulative position where a new element may start
		    // (y-axis)
	if(!title.equals(""))
	    pos+=TextElementGenerator.createSingleString
		(c,title,null,18,pos,false,false,false,false).getSize().getHeight()+5;
	if(!subtitle.equals(""))
	    TextElementGenerator.createSingleString
		(c,subtitle,null,14,pos,false,false,true,false);
    }

    // Fills a back side with title, subtitle and description. Does NOT fill the sides for
    // a back side. The xsize is the xsize that should be used for middle calculation.
    private void addBackElements( Cover c, int xsize ){
	int pos=50; // this is the cumulative position where a new element may start (y-axis)
	SimpleDateFormat f=new SimpleDateFormat("H:m");
	Date d=null;
	try{
	    d=new SimpleDateFormat("h:m").parse("0:0");
	}catch(ParseException pe){
	    // won't happen
	}
	String t=null;
	if(length.equals(d)) t=title;
	else t=title+" ("+f.format(length)+")";
	if(!title.equals("") || !length.equals(d))
	    pos+=TextElementGenerator.createSingleString
		(c,t,null,18,pos,true,false,false,false).getSize().getHeight()+5;
	if(!subtitle.equals(""))
	    pos+=TextElementGenerator.createSingleString
		(c,subtitle,null,14,pos,true,false,false,false).getSize().getHeight()+5;
	if(!format.equals("")){
	    format="("+format+")";
	    pos+=TextElementGenerator.createSingleString
		(c,format,null,14,pos,true,false,true,false).getSize().getHeight()+5;
	}
	if(!description.equals(""))
	    TextElementGenerator.createSingleString
		(c,description,null,14,pos,true,false,false,false);
    }

}
