package cdox.data;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

/**
 * This class is used by VideoData to display the video CD data.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 */
public class VideoDataSheet extends JDialog{

    private JTextField title=new JTextField(),format=new JTextField(),subtitle=new JTextField();
    private JTextArea description=new JTextArea();
    private Localizer lang=CDox.getLocalizer();
    private JButton okButton;
    private int buttonPressed=-1;
    private JIntField hrs=new JIntField(0,0,99),min=new JIntField(0,0,59);

    /**
     * Constructs one.
     * @param parent the parent frame.
     */
    public VideoDataSheet( Frame parent ){
	super(parent,CDox.getLocalizer().get("properties"),true);
	// setting some properties of the components
	title.setColumns(20);
	subtitle.setColumns(20);
	format.setColumns(20);
	description.setRows(5);
	description.setLineWrap(true);
	description.setWrapStyleWord(true);
	// adding the components
	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	gb.gridx=0;
	gb.gridy=0;
	gb.anchor=gb.WEST;
	gb.insets=new Insets(1,2,1,2);
	getContentPane().add(new JLabel(lang.get("adtitle")+":"),gb);
	gb.gridx=1;
	gb.anchor=gb.EAST;
	gb.gridwidth=2;
	getContentPane().add(title,gb);
	gb.gridy++;
	getContentPane().add(subtitle,gb);
	gb.gridx=0;
	gb.gridwidth=1;
	gb.anchor=gb.WEST;
	getContentPane().add(new JLabel(lang.get("adsubtitle")+":"),gb);
	gb.gridy++;
	getContentPane().add(new JLabel(lang.get("vdformat")+":"),gb);
	gb.gridx=1;
	gb.gridwidth=2;
	gb.anchor=gb.EAST;
	getContentPane().add(format,gb);
	gb.gridwidth=1;
	gb.gridy++;
	gb.fill=gb.BOTH;
	gb.weightx=1;
	getContentPane().add(hrs,gb);
	gb.gridx=2;
	getContentPane().add(min,gb);
	gb.gridx=0;
	gb.weightx=0;
	getContentPane().add(new JLabel(lang.get("vdlength")+":"),gb);
	gb.gridy++;
	gb.gridwidth=3;
	gb.weightx=1;
	JScrollPane panel=new JScrollPane(description);
	panel.setBorder(new TitledBorder(new EtchedBorder(),lang.get("addescription")));
	getContentPane().add(panel,gb);
	JPanel buttonPanel = new JPanel(new GridBagLayout());
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridx=0;
	gbc.weightx = 1f;
	gbc.fill=gb.NONE;
	gbc.anchor=gb.CENTER;
	gbc.insets = new Insets(0,3,0,3);
	int maxWidth=0;
	okButton=new JButton(lang.get("okbutton"));
	if(maxWidth<okButton.getPreferredSize().width)
	    maxWidth=okButton.getPreferredSize().width;
	okButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=1;
		    setVisible(false);
		    dispose();
		}
	    });
	getRootPane().setDefaultButton(okButton);
	buttonPanel.add(okButton,gbc);
	gbc.gridx=1;
	JButton button=new JButton(lang.get("fillbutton"));
	button.setToolTipText(lang.get("fillbuttontip"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=0;
		    setVisible(false);
		    dispose();
		}
	    });
	buttonPanel.add(button, gbc);
	gbc.gridx=2;
	button=new JButton(lang.get("cancelbutton"));
	if(maxWidth<button.getPreferredSize().width)
	    maxWidth=button.getPreferredSize().width;
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    buttonPressed=-1;
		    setVisible(false);
		    dispose();
		}
	    });
	new EscapeListener(button,getRootPane());
	buttonPanel.add(button,gbc);
	for(int i=0; i<buttonPanel.getComponentCount();i++){
	    ((JButton)buttonPanel.getComponent(i)).setPreferredSize
		(new Dimension(maxWidth, button.getPreferredSize().height));
	}
	gb.fill=gb.HORIZONTAL;
	gb.gridy++;
	gb.gridwidth=3;
	gb.anchor=gb.CENTER;
	getContentPane().add(buttonPanel,gb);
	pack();
	setResizable(false);
	setLocationRelativeTo(null); // centers it...
    }


    /**
     * Returns 1 if ok is pressed -1 for Cancel, 0 for fill.
     * @return 1 for ok, -1 for cancel, 0 for fill.
     */
    public int whichButtonPressed(){
	return buttonPressed;
    }

    /**
     * Fills in the values.
     */
    public void setValues( String title, String subtitle, String format, String description,
			   Date length ){
	this.format.setText(format);
	this.title.setText(title);
	this.subtitle.setText(subtitle);
	this.description.setText(description);
	SimpleDateFormat f=new SimpleDateFormat("H");
	hrs.setValue(Integer.parseInt(f.format(length)));
	f=new SimpleDateFormat("m");
	min.setValue(Integer.parseInt(f.format(length)));
    }

    /**
     * Fills the current values in this dialog into the data. If the user pressed cancel
     * the last time, no values are touched in the data object.
     *@param data where to fill the data.
     */
    public void getValues( VideoData data ){
	data.setTitle(title.getText());
	data.setFormat(format.getText());
	data.setSubtitle(subtitle.getText());
	data.setDescription(description.getText());
	SimpleDateFormat f=new SimpleDateFormat("H:m");
	String s=hrs.getValue()+":"+min.getValue();
	try{
	    data.setLength(f.parse(s));
	}catch(ParseException pe){
	    // won't happen
	}
    }

}
