package cdox.edit;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import cdox.*;
import cdox.gui.*;
import cdox.util.*;

/**
 * This class Creates a Panel in which the actual CDCover Panel will be added.
 *It uses the CDCoverStadards interface to determine the size of each Component
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 * @see EditPane
 * @see CDDrawingPane
 */
public class CDCover extends JPanel implements CDCoverStandards{
    //to get the right language and the constraints object
    private Localizer lang=CDox.getLocalizer();
    private GridBagConstraints gb = new GridBagConstraints();

    private CDoxFrame cdoxFrame;
    private CDDrawingPane drawPane;
    private Color enabledColor = Color.black;
    private Color disabledColor = Color.gray;
    private Color coverBackgroundColor;
    private String title;
    private Dimension drawingDim = new Dimension();
    private boolean active = false;

    /**
     * The constructor is called to do all the work.
     *It Checks which cdCover is to be made, and defines the Size of itself
     *by adding 12 (Pixels-width)and 30(Pixels-height) to the coverSize.
     * @param cdoxFrame the CDoxFrame.
     * @param cdKonst the konst (defined in CDCoverStandards) defining which cover to make,
     * @param f The font standardly used for a textPane in the CDDrawingPane,
     * @param bold true if bold is selected in the cdoxframe false otherwhise,
     * @param italic true if italic is selected in the cdoxframe false otherwhise,
     * @param underline true if underline is selected in the cdoxframe false otherwhise.
     * @param tColor the <code>Color</code>.
     */
    public CDCover(CDoxFrame cdoxFrame,int cdKonst,
		   Font f, boolean bold, boolean italic, boolean underline, Color tColor){
	this(cdoxFrame,null,cdKonst,f,bold,italic,underline,tColor);
    }

    /**
     * Constructs new one with a given cover. If cover is null, the cdKonst will be used
     * to generate a new empty cover.
     * @param cdoxFrame the CDoxFrame.
     * @param cover the cover.
     * @param cdKonst the konst (defined in CDCoverStandards) defining which cover to make,
     * @param f The font standardly used for a textPane in the CDDrawingPane,
     * @param bold true if bold is selected in the cdoxframe false otherwhise,
     * @param italic true if italic is selected in the cdoxframe false otherwhise,
     * @param underline true if underline is selected in the cdoxframe false otherwhise.
     * @param tColor the <code>Color</code>.
     */
    public CDCover(CDoxFrame cdoxFrame,Cover cover,int cdKonst,
		   Font f, boolean bold, boolean italic, boolean underline, Color tColor){
	this.cdoxFrame=cdoxFrame;
	setLayout(new GridBagLayout());
	drawingDim = new Dimension();
	Dimension thisDim = new Dimension();
	int h;
	if(cover==null) h=cdKonst;
	else h=cover.getType();
	switch (h){
	case CDFRONT:
	    title = lang.get("cdfront");
	    drawingDim.setSize(CDFRONT_SIZE[0],CDFRONT_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDFRONT_SIZE[0]+12,CDFRONT_SIZE[1]+30);
	    break;
	case CDBOOKLET:
	    title = lang.get("cdbooklet");
	    drawingDim.setSize(CDBOOKLET_SIZE[0],CDBOOKLET_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDBOOKLET_SIZE[0]+12,CDBOOKLET_SIZE[1]+30);
	    break;
	case CDBACK_SIDE:
	    title = lang.get("cdside");
	    drawingDim.setSize(CDBACK_SIDE_SIZE[0], CDBACK_SIDE_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDBACK_SIDE_SIZE[0]+12, CDBACK_SIDE_SIZE[1]+30);
	    break;
	default:
	    break;
	}
	//This is for the CDDrawingPane-Component
	gb.anchor = gb.CENTER;
	gb.fill = gb.NONE;
	if(cover==null)
	    drawPane = new CDDrawingPane(cdoxFrame,drawingDim,cdKonst, 
					 f, bold, italic, underline, tColor);
	else drawPane = new CDDrawingPane(cdoxFrame,drawingDim,cover,cdKonst,
					  f, bold, italic, underline, tColor);
	add(drawPane, gb);

	//This is for this Object
	setPreferredSize(thisDim);
	setMinimumSize(thisDim);
	setMaximumSize(thisDim);
	setBorder(BorderFactory.createCompoundBorder
		  (BorderFactory.createTitledBorder
		   (BorderFactory.createLineBorder(Color.black), 
		    title,TitledBorder.LEADING,TitledBorder.ABOVE_TOP),
		   BorderFactory.createLoweredBevelBorder())); 
    }

    public void replaceCover(Cover cover) {
	remove(0);
	int h=cover.getType();
	Dimension thisDim = new Dimension();
	switch (h){
	case CDFRONT:
	    title = lang.get("cdfront");
	    drawingDim.setSize(CDFRONT_SIZE[0],CDFRONT_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDFRONT_SIZE[0]+12,CDFRONT_SIZE[1]+30);
	    break;
	case CDBOOKLET:
	    title = lang.get("cdbooklet");
	    drawingDim.setSize(CDBOOKLET_SIZE[0],CDBOOKLET_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDBOOKLET_SIZE[0]+12,CDBOOKLET_SIZE[1]+30);
	    break;
	case CDBACK_SIDE:
	    title = lang.get("cdside");
	    drawingDim.setSize(CDBACK_SIDE_SIZE[0], CDBACK_SIDE_SIZE[1]);//dimension of Component to add
	    thisDim.setSize(CDBACK_SIDE_SIZE[0]+12, CDBACK_SIDE_SIZE[1]+30);
	    break;
	default:
	    break;
	}
	gb.anchor = gb.CENTER;
	gb.fill = gb.NONE;
	drawPane = new CDDrawingPane(cdoxFrame,drawingDim,cover,cover.getType(),
				     cdoxFrame.getActualFont(), 
				     cdoxFrame.getTextBold(), 
				     cdoxFrame.getTextItalic(), 
				     cdoxFrame.getTextUnderline(), 
				     cdoxFrame.getTextColor());
	add(drawPane, gb);

	//This is for this Object
	setPreferredSize(thisDim);
	setMinimumSize(thisDim);
	setMaximumSize(thisDim);
	setBorder(BorderFactory.createCompoundBorder
		  (BorderFactory.createTitledBorder
		   (BorderFactory.createLineBorder(Color.black), 
		    title,TitledBorder.LEADING,TitledBorder.ABOVE_TOP),
		   BorderFactory.createLoweredBevelBorder())); 
    }
    
    /**
     *This Method sets the selected component active-deactivated.
     *@param flag the activation flag, true = activated.
     */
    public void setActive(boolean flag){
	active = flag;
	setBorder(BorderFactory.createCompoundBorder 
		  (BorderFactory.createTitledBorder
		   (BorderFactory.createLineBorder((flag)? enabledColor: disabledColor), 
		    title, TitledBorder.LEADING, TitledBorder.ABOVE_TOP, null,
		    (flag)? enabledColor : disabledColor),
		   BorderFactory.createLoweredBevelBorder())); 
	drawPane.setActive(active);
    }

    /**
     * Returns the drawing pane of this CDCover.
     *
     * @return the CDDrawingPane.
     */
    public CDDrawingPane getDrawingPane(){
	return drawPane;
    }

    /**
     * Returns whether the CDCover is active.
     *
     * @return true, if it is.
     */
    public boolean isActive(){
	return active;
    }

    /**
     * Returns the drawing size.
     *
     * @return a <code>Dimension</code> value.
     */
    public Dimension getDrawingDimension(){
	return drawingDim;
    }

    /**
     * Returns whether it has a background.
     *
     * @return true, if yes.
     */
    public boolean hasBackground(){
	return drawPane.getCover().hasBackgroundImage();
    }
    
    /**
     * Here you set the backgroundColor flag.
     * @param bc the <code>Color</code>.
     */
    public void setBackgroundColor(Color bc){
	coverBackgroundColor = bc;
    }
}
    
    
