package cdox.edit;

import cdox.*;
import cdox.gui.*;
import cdox.gui.action.*;
import cdox.gui.undo.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.util.zip.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 *This class represents the Actual drawing Panel. 
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 */
public class CDDrawingPane extends JPanel{

    private Element markedElement=null;
    private Cover cover;
    private Dimension dim;
    private CDDrawingPane itsme = this;
    private int coverHeight, coverWidth, scrollPaneWidth, scrollPaneHeight;
    private boolean isActive = false;

    //Fr text editieren
    private GridBagConstraints gb ;
    private boolean textEnabled = false;
    private TextPane tPane = null;
    private JScrollPane sPane = null;
    private int fontSize;
    private Font textFont = getFont().deriveFont(fontSize);
    private boolean textBold = false, textItalic = false, textUnderline = false;
    private Color textColor;
    private static final boolean DEBUG=true;
    private CDoxFrame cdoxFrame;
    private Localizer lang=CDox.getLocalizer();
    private JPopupMenu popupMenu;

    private Point bPoint;

    

    /**
     * The Constructor sets the size, and the background Color.
     * @param c the CDoxFrame.
     * @param dim the Dimension of this CDDrawingPane,
     * @param template the sort of Cover it is,
     * @param theFont the Font currently selected for the TextPane,
     * @param bold true if bold is selected in the cdoxframe false otherwhise
     * @param italic true if italic is selected in the cdoxframe false otherwhise
     * @param underline true if underline is selected in the cdoxframe false otherwhise
     * @param tColor the <code>Color</code> value.
     */
    public CDDrawingPane(CDoxFrame c,Dimension dim, int template,
			 Font theFont, boolean bold, boolean italic,
			 boolean underline, Color tColor){
	this(c,dim,null,template,theFont,bold,
	     italic,underline,tColor);
    }    

    /**
     * Constructs new one with the given cover. If cover is null, a new, empty Cover of
     * the template type will be created.
     * @param c the CDoxFrame.
     * @param dim the Dimension of this CDDrawingPane,
     * @param cover the cover.
     * @param template the sort of Cover it is,
     * @param theFont the Font currently selected for the TextPane,
     * @param bold true if bold is selected in the cdoxframe false otherwhise
     * @param italic true if italic is selected in the cdoxframe false otherwhise
     * @param underline true if underline is selected in the cdoxframe false otherwhise
     * @param tColor the <code>Color</code>.
     */
    public CDDrawingPane(CDoxFrame c,Dimension dim, Cover cover, int template,
			 Font theFont, boolean bold, boolean italic,
			 boolean underline, Color tColor){
	cdoxFrame=c;
	this.dim = dim;
	if(cover==null) this.cover=new Cover(template);
	else this.cover=cover;
	this.cover.setBackground(Color.white);
 	setPreferredSize(dim);
 	setMinimumSize(dim);
 	setMaximumSize(dim);

	coverHeight = (int)dim.getHeight();
	coverWidth = (int)dim.getWidth();
	
	setLayout(new GridBagLayout());
	gb = new GridBagConstraints();
	gb.fill=gb.BOTH;
	gb.weightx = 2000;
	gb.weighty = 2000;
	
	setBackground(new Color(0.0f,0.0f,0.0f,0.0f));
	
	MouseMover mMover = new MouseMover();
	addMouseListener(mMover);
	addMouseMotionListener(mMover);
	
	textFont = theFont;
	fontSize = textFont.getSize();
	textBold = bold;
	textItalic = italic;
	textUnderline = underline;
	textColor = tColor;
    }

    public String toString(){
	return "CDDrawingPane ("+cover+")";
    }

    /**
     *This Method sets this panel active or Inactive, if inactive the backgroundImage
     *is painted in grayscale.
     *@param b true for active (not grayscaled)
     */
    public void setActive(boolean b){
	isActive = b;
	if(!b)
	    setElementSelected(null);
	cover.setGrayscale(!b);
	repaint();
    }
 
    /**
     *This method returns this drawingPane's cover-Object.
     *@return return this pane's Cover-Object
     */
    public Cover getCover(){
	return cover;
    }

    /**
     *This Method is called when the Window is resized, 
     *it can than check if it needs to scroll.
     *@param width the width of the visible part of the ScrollPane
     *@param height the height of the visible part of the ScrollPane
     */
    public void setScrollPaneDimension(int width, int height){
	scrollPaneWidth = width;
	scrollPaneHeight = height;
    }

    /**
     * This method adds a normal Image to this pane.
     * @param id the TempFiles id of the image.
     * @exception IOException if an error occurs.
     * @see CDoxFrame#setImage
     * @see EditAction#actionPerformed
     */
    public void addImage( String id ) throws IOException{
	ImageElement e=new ImageElement(new Point(),null,id,cover);
	cover.addElement(e);
	cdoxFrame.setUndoAction(new InsertUndoAction(e,cover));
	setElementSelected(e);//setSelected
	repaint();
    }
    
    /**
     * This Method sets the specified image as background image to this panel. For the mode
     *parameter see the Cover methods.
     * @param id the TempFiles id.
     * @param mode the mode.
     * @exception IOException if an error occurs.
     * @see Cover#setBackground(String,boolean)
     */
    public void setBackgroundImage( String id, boolean mode ) throws IOException{
	cover.setBackground(id,mode);
	repaint();
    }

    /**
     * This Method sets the specified image as background image to this panel. For the mode
     * parameter see the Cover methods.
     * @param bi the allready loaded BufferedImage
     * @param id the TempFiles id.
     * @param mode the mode.
     * @exception IOException if an error occurs.
     * @see Cover#setBackground(String,boolean)
     */
    public void setBackgroundImage( BufferedImage bi, String id, boolean mode ){
	cover.setBackground(bi, id,mode);
	repaint();
    }

    /**
     *This Method sets the specified background color to this Panel.
     *@param bgc the Background color
     */
    public void setBackgroundColor(Color bgc){
	cover.setBackground(bgc);
	repaint();
	updateUI();
    }

    /**
     *This Method returns the current background color.
     *@return the current background Color.
     */
    public Color getBackgroundColor(){
	return cover.getBackgroundColor();
    }
    
    /**
     *This Method sets whether we're in textMode or not.
     *@param b true if in TextMode
     */
    public void setTextEnabled(boolean b){
	textEnabled = b;
	defaultCursor();
	if((sPane != null) && (!textEnabled)){//wenn auf ja button von textdialog gedrcht
					      //wurde.
	    remove(sPane);
	    sPane = null;
	    tPane = null;
	    updateUI();
	}
	else if((sPane != null)&&(textEnabled)){//wenn auf no button von textdialog gedrckt wurde,
	    tPane.requestFocus();// mu tpane wieder den Focus kriegen.
	}
    }

    /**
     *Returns true if this CDDrawingPane is in textMode.
     *@return true if in textMode false otherwhise.
     */
    public boolean isTextEnabled(){
	return textEnabled;
    }

    /**
     *This Method returns the current TextPane.
     *@return the current TextPane, or null if no tPane is on.
     */
    public TextPane getTextPane(){
	return tPane;
    }
	    
    /**
     *This Method sets the font to write with.
     *@param f the Font to write with.
     */
    public void setTextFont(Font f){
	textFont = f;
	if(tPane != null)
	    setTPane();
    }
    
    /**
     *This Method sets the Size of the font in the TextPane
     *@param fSize the float value representing the size of the font.
     */
    public void setFontSize(int fSize){
	fontSize = fSize;
	if(tPane != null)
	    setTPane();
    }
    
    /**
     *This Method sets whether to write in bold or not
     *@param b true to write in bold false otherwhise.
     */
    public void setBold(boolean b){
	textBold = b;
	if(tPane != null)
	    setTPane();
    }
    
    /**
     *This Method sets whether to write in italic or not
     *@param i true to write in italic false otherwhise.
     */
    public void setItalic(boolean i){
	textItalic = i;
	if(tPane != null)
	    setTPane();
    }
    
    /**
     *This Method sets whether to write in underline or not
     *@param u true to write in underline false otherwhise.
     */
    public void setUnderline(boolean u){
	textUnderline = u;
	if(tPane != null)
	    setTPane();
    }
    
    /**
     *This method sets the Color of the text.
     *@param tc the new textColor.
     */
    public void setTextColor(Color tc){
	textColor = tc;
	if(tPane != null)
	    setTPane();
    }
	
    /**
     *This Method is called when we need to (re)draw on this element.
     *@param g the Graphics object to draw with
     */    
    public void paint(Graphics g){
	cover.drawMyself(g);
	if(textEnabled) super.paint(g);

    }

    /**
     *Set the default cursor.
     */
    private void defaultCursor(){
	if(!cdoxFrame.isWorking()){
	    if(textEnabled){
		setCursor(new Cursor(Cursor.TEXT_CURSOR));
	    }
	    else
		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
	else{
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));
	}
    }

    /**
     *This Method sets the changes made to the TextPane, and than requests the 
     *tPanes focus.
     */
    private void setTPane(){
	tPane.setAttribute(textFont, fontSize, textColor, textBold, textItalic, textUnderline);
	tPane.requestFocus();
    }

    /**
     *This method is called when a text element mus be re-edited.
     *@param e the textelement that must be re-edited
     */
    public void reEditTextElement(Element e){
	if(e!= null){
	    int result = JOptionPane.OK_OPTION;
	    result = CDoxFrame.showQuestion(lang.get("reedittextquest"),JOptionPane.YES_NO_OPTION);
	    if(result == JOptionPane.OK_OPTION){
		setElementSelected(null);
		cdoxFrame.updateTextBar(null);
		cdoxFrame.setActionBorder("text");
		cover.removeElement(e);
		((TextElement)e).resetFontSizes();
		tPane = ((TextElement)e).getTextPane();
		CaretListener[] cL = tPane.getCaretListeners();
		boolean caretL = false;
		for(int i = 0; i<cL.length;i++){
		    if(cL[i] instanceof CaretUpdater){
			caretL = true;
		    }
		}
		if(!caretL)
		    tPane.addCaretListener(new CaretUpdater(cdoxFrame, tPane));
		tPane.setPoint(e.getPosition());
		tPane.setBackground(new Color(220, 220, 220, 255));
		tPane.setCaretColor(new Color(0, 0, 0, 255));
		sPane = new JScrollPane(tPane);
		gb.insets= new Insets(tPane.getPoint().y, tPane.getPoint().x, 10,10 );
		add(sPane, gb);
		updateUI();
		tPane.requestFocus();	    
	    }
	}
    }
    /**
     *This method keeps track of which Element is selected, should be selected, or
     *should be not selected.
     *@param e the element to be selected, if e == null the currently selected element,
     *will be deselected.
     */
    public void setElementSelected(Element e){
	if(e == null){
	    if(markedElement != null){
		markedElement.setSelected(false);
		markedElement.stopRotating();
		markedElement = null;
	    }
	    cover.setDragging(false);
	    cdoxFrame.getStatusPane().setTextLabel(lang.get("statuspanenoselectedelement"));
	}
	else{
	    cdoxFrame.setCutNWasteable(true);
	    if(markedElement != null){
		markedElement.setSelected(false);
		markedElement = e;
		cover.setDragging(false);
		repaint();
		markedElement.setSelected(true);
		cover.setDragging(true);
	    }
	    else{
		markedElement = e;
		markedElement.setSelected(true);
		cover.setDragging(true);
		markedElement.startRotating(this, markedElement);
	    }
	    cdoxFrame.getStatusPane().setTextLabel(lang.get("statuspaneselectedelement")+": " +markedElement.toString());
	}
    }


    /**
     *Returns the current selected element.
     *@return the current selected element.
     */
    public Element getSelectedElement(){
	if(markedElement != null){
	    return markedElement;
	}
	return null;
    }

    /**
     * Returns the point clicked by the user.
     *
     */
    public Point getCurrentClickPoint(){
	return bPoint;
    }
    
    /**
     *This inner class handles all the movements on the Objects in this Panel.	
     */

    private class MouseMover extends MouseInputAdapter {
	
	public void mousePressed(MouseEvent e){
	    bPoint = e.getPoint();
	    currentElement = cover.getElementAtPosition(bPoint);
	    if(e.getButton() == e.BUTTON1){
		if(cdoxFrame.getEditPane().getActiveCover()!= (CDCover)itsme.getParent())
		    cdoxFrame.getEditPane().getDrawingPane().getCover().setDragging(false);
		    cdoxFrame.getEditPane().setActive((CDCover)itsme.getParent());
		if(!textEnabled){
		    if (currentElement != null){
			currentElement.moveToTop();
			// the next line is for undo
			if(startDragPoint==null)
			    startDragPoint=((Point)currentElement.getPosition().clone());
			if(startDragSize==null)
			    startDragSize=((Dimension)currentElement.getSize().clone());
		    }
		}
	    }
	    if(e.isPopupTrigger()&& (!(textEnabled))) {
		triggerPopup(e); // seems logical, doesn't it?
	    }
	    itsme.setElementSelected(currentElement);
	}

	/**
	 * Clicking does mark the element.
	 *@param e the mouse event.
	 */
	public void mouseClicked( MouseEvent e ){
	    bPoint = e.getPoint();
    
	    if((textEnabled) && (e.getButton() == e.BUTTON1)){//IN TEXTMODE
		if(sPane == null){
		    itsme.setElementSelected(null);
		    tPane = new TextPane( bPoint, textFont, fontSize, textColor, textBold, textItalic, textUnderline);
		    tPane.addCaretListener(new CaretUpdater(cdoxFrame, tPane));
		    sPane = new JScrollPane(tPane);
		    gb.insets= new Insets(bPoint.y, bPoint.x, 10,10 );
		    itsme.add(sPane, gb);
		    itsme.updateUI();
		    tPane.requestFocus();
		}
		else{
		    if(!((sPane.getBounds()).contains(bPoint.x, bPoint.y))){
			if((tPane.getStyledDocument()).getLength() != 0){
			    TextElement tElement = new TextElement(tPane.getPoint(), tPane);
			    cover.addElement(tElement);
			    cdoxFrame.setUndoAction(new InsertUndoAction(tElement,cover));
			    itsme.setElementSelected(tElement);
			    // work needed here?
			}
			itsme.remove(sPane);
			tPane = null;
			sPane = null;
		    }
		}
	    }
	    else{
		currentElement=cover.getElementAtPosition(bPoint);
		if(e.getButton() == e.BUTTON1){
		    if(currentElement!=null){
			if((currentElement instanceof TextElement) && (e.getClickCount() == 2)){
			itsme.reEditTextElement(currentElement);
			}
			else{
			    cdoxFrame.setCutNWasteable(true);
			    itsme.setElementSelected(currentElement);
			}
			cdoxFrame.setSave(true);
		    }else{//es wurde auf kein element geklickt
			itsme.setElementSelected(null);
			cdoxFrame.setCutNWasteable(false);
		    }
		}
		else if((e.isPopupTrigger())&& (!(textEnabled))) {
		    triggerPopup(e);
		}
	    }
	    repaint();
	}

	public void mouseReleased(MouseEvent e){
	    if((startDragPoint!=null)&&(currentElement!=null)){
		// we have a new undo action, moving
		DoAction da=new MoveAction(currentElement,startDragPoint,
					   ((Point)currentElement.getPosition().clone()));
		cdoxFrame.setUndoAction(da);
	    }
	    if((startDragSize!=null)&&(currentElement!=null)){
		// we have a new undo action, resizing
		DoAction da=new ResizeAction(currentElement,startDragSize,
					   ((Dimension)currentElement.getSize().clone()));
		cdoxFrame.setUndoAction(da);
	    }
	    isDragging=false;
	    if(!hasMouse) mouseExited(null);
	    if((e.isPopupTrigger()) && (!(textEnabled))) {
		triggerPopup(e);
	    }
	}
		// shows popup menu
	private void triggerPopup( MouseEvent e ){
	    if(currentElement != null){
		if(currentElement instanceof TextElement){
		    cdoxFrame.showPopup(e.getComponent(),e.getX(),e.getY(),
					lang.get("popuptextelementtitle"));
		}
		else if(currentElement instanceof ImageElement){
		    cdoxFrame.showPopup(e.getComponent(),e.getX(),e.getY(),
					lang.get("popupbildelementtitle"));
		}
	    }
	    else{
		if(cover.hasBackgroundImage()){
		    cdoxFrame.showPopup(e.getComponent(),e.getX(), e.getY(),
					lang.get("popupbackgroundtitle"));
		}
		else{
		    cdoxFrame.showPopup(e.getComponent(),e.getX(), e.getY(),
					null);
		}
	    }
	}

	/**
	 *This methods does the Moving of an Object, it calculates if the mouse has been moved
	 *and if it -the Object- has touched a side.
	 */
	public void mouseDragged(MouseEvent e){
	    if(!textEnabled && isActive){
		isDragging=true;
		if(currentElement !=null){
		    curPoint = e.getPoint();
		    nullPoint = currentElement.getPosition();		
		    delx = curPoint.x - bPoint.x;
		    dely = curPoint.y - bPoint.y;
		    if(diagonalResizeRect != null){//wir wollen resizen
			currentElement.setMaintainRatio(true);
			startDragPoint=null; // ""
			delx += bPoint.x - nullPoint.x; //neue width = bewogene.x + ausgang.x - nullPoint
			dely += bPoint.y - nullPoint.y; //neue height
			if(delx < 10 ){
			    if(dely < 10)
				elementDim = new Dimension(10, 10);
			    else 
				elementDim = new Dimension(10, dely);
			}
			else if(dely <10)
			    elementDim = new Dimension(delx, 10);
			else if((delx+nullPoint.x) > coverWidth){//neueWidth + nullPoint>coverwidth
			    delx = coverWidth - nullPoint.x;
			    if((dely+nullPoint.y) > coverHeight)//und grer als height
				dely = coverHeight - nullPoint.y;
			    elementDim = new Dimension(delx, dely);//nur grer als height
			}
			else if((dely+nullPoint.y)>coverHeight){
			    dely = coverHeight - nullPoint.y;
			    elementDim = new Dimension(delx, dely);
			}
			else
			    elementDim = new Dimension(delx, dely);
			currentElement.setSize(elementDim);
			repaint();
			bPoint = curPoint;
		    }
		    else if(horizontalResizeRect != null){
			currentElement.setMaintainRatio(false);
			delx += bPoint.x - nullPoint.x; //neue width = bewogene.x + ausgang.x - nullPoint
			int elementY = currentElement.getSize().height;
			if(delx < 10)
			    elementDim = new Dimension(10, elementY);
			else if((delx+nullPoint.x) > coverWidth){
			    delx = coverWidth - nullPoint.x;
			    elementDim = new Dimension(delx, elementY);
			}
			else
			    elementDim = new Dimension(delx, elementY);
			currentElement.setSize(elementDim);
			repaint();
			bPoint = curPoint;
		    }
		    else if(verticalResizeRect != null){
			currentElement.setMaintainRatio(false);
			dely += bPoint.y - nullPoint.y; //neue height
			int elementX =(currentElement.getSize().width);
			if(dely < 10)
			    elementDim = new Dimension(elementX, 10);
			else if((dely+nullPoint.y) > coverHeight){
			    dely = coverHeight - nullPoint.y;
			    elementDim = new Dimension(elementX, dely);
			}
			else
			    elementDim = new Dimension(elementX, dely);
			currentElement.setSize(elementDim);
			repaint();
			bPoint = curPoint;
		    }
		    else{
			/**
			 *Diese else clause ist fr das verschieben einen Object zuStndig
			 *sie berechnet den neuen oibere linken Position, und guckt ob die
			 * an einem Rand (coverRand) ist, wenn in einer Ecke mu nicht neu
			 *gezeichnet werden darher den boolean repaint
			 */
			startDragSize=null; // we want no ResizeAction
			boolean repaint = true;
			nullPoint.x += delx;
			nullPoint.y += dely;
			//Hier wird die links geguck ob schon kleiner ist als coverwidth
			if(nullPoint.x <0 ){//links 
			    nullPoint.x = 0;
			    if(nullPoint.y<0){//oben in der Ecke
				nullPoint.y =0;
				repaint = false;
			    }
			    else if(nullPoint.y>(coverHeight-(int)currentElement.getSize().height)){//unten in der Ecke
				nullPoint.y = (coverHeight-(int)currentElement.getSize().height);
				repaint = false;
			    }
			    else//Vertikal Bewegen
				bPoint.y = curPoint.y;
			}
			//Hier wird die rechts geguck ob schon grer ist als coverwidth
			else if(nullPoint.x > (coverWidth - (int)currentElement.getSize().width)){//rechts 
			    nullPoint.x = (coverWidth -(int)currentElement.getSize().width);
			    if(nullPoint.y<0){//oben in der Ecke
				nullPoint.y =0;
				repaint = false;
			    }
			    else if(nullPoint.y>(coverHeight-(int)currentElement.getSize().height)){//unten in der Ecke
				nullPoint.y = (coverHeight-(int)currentElement.getSize().height);
				repaint = false;
			    }
			    else//Vertikal bewegen
				bPoint.y = curPoint.y;
			}
			//Hier wird die oben geguck ob schon kleiner ist als coverheight
			else if(nullPoint.y<0){//oben
			    nullPoint.y =0;
			    if(nullPoint.x <0 ){//links in der Ecke
				nullPoint.x = 0;
				repaint = false;
			    }
			    else if(nullPoint.x > (coverWidth - (int)currentElement.getSize().width)){//rechts in der Ecke
				nullPoint.x = (coverWidth -(int)currentElement.getSize().width);
				repaint = false;
			    }
			    else
				bPoint.x = curPoint.x;//Horizontal bewegen
			}
			//Hier wird die unten geguck ob schon grer ist als coverHeight
			else if(nullPoint.y>(coverHeight-(int)currentElement.getSize().height)){//unten
			    nullPoint.y = (coverHeight-(int)currentElement.getSize().height);
			    if(nullPoint.x <0 ){//links in der Ecke
				nullPoint.x = 0;
				repaint = false;
			    }
			    else if(nullPoint.x > (coverWidth - (int)currentElement.getSize().width)){//rechts in der Ecke
				nullPoint.x = (coverWidth -(int)currentElement.getSize().width);
				repaint = false;
			    }
			    else//Horizontal bewegen
				bPoint.x = curPoint.x;
			}
			else{ //Mitten im Bild
			    bPoint = curPoint;
			}
			currentElement.setPosition(nullPoint);
			if(repaint){
			    repaint();
			}
		    }
		    cdoxFrame.setSave(true);
		}
	    }
	}

	/**
	 * Does the highlighting of an element and updates the status bar.
	 */
	public void mouseMoved(MouseEvent e){
	    // update the status pane
	    cdoxFrame.getStatusPane().setPositionLabel(e.getPoint());
	    // if not in text mode and active, check the highlighting
	    if(!textEnabled && isActive){
		currentElement = cover.getElementAtPosition(e.getPoint());
		// first: if an old element exists
		if (currentElement != null){
		    // if the mouse is over a new element, reset the old one and set the
		    // new one highlighted:
		    if(!(currentElement == tempElement)){
			if(tempElement != null){
			    tempElement.setHighlight();
			    tempElement = null;
			}
			if(!currentElement.getHighlight()){
			    currentElement.setHighlight();
			    cover.setDragging(false);
			    repaint();
			    tempElement = currentElement;
			    cover.setDragging(true);
			}
			repaint();
		    } // end resetting tempElement
		    // check for diagonal resize settings
		    //the working cursor of the CDoxFrame must not be set.
		    if(!cdoxFrame.isWorking()){
			if(currentElement.getDiagonalResizeRect().contains(e.getPoint())){
			    diagonalResizeRect=currentElement.getDiagonalResizeRect();
			    horizontalResizeRect = null;
			    verticalResizeRect = null;
			    itsme.setCursor(new Cursor(Cursor.SE_RESIZE_CURSOR));
			}
			// check for horizontal resize settings
			else if(currentElement.getHorizontalResizeRect().contains(e.getPoint())){
			    diagonalResizeRect=null;
			    verticalResizeRect = null;
			    horizontalResizeRect = currentElement.getHorizontalResizeRect();
			    itsme.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
			}
			// check for vertical resize settings
			else if(currentElement.getVerticalResizeRect().contains(e.getPoint())){
			    diagonalResizeRect=null;
			    verticalResizeRect = currentElement.getVerticalResizeRect();
			    horizontalResizeRect = null;
			    itsme.setCursor(new Cursor(Cursor.S_RESIZE_CURSOR));
			}
			// in this case, we have no resizing at all. so we want to move:
			else{
			    itsme.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			    diagonalResizeRect = null;
			    horizontalResizeRect = null;
			    verticalResizeRect = null;
			}
		    }
		    else{
			itsme.defaultCursor();
		    }
		}
		// there is no currentElement but a tempElement:
		else if(tempElement != null){
		    cover.setDragging(false);
		    repaint();
		    tempElement.setHighlight();
		    cover.setDragging(true);
		    tempElement = null;
		    if(!cdoxFrame.isWorking())//check if the CDoxFrame's working icon is set.
			itsme.defaultCursor();
		    repaint();
		}
		//set the cover to not dragging
		else{
		    cover.setDragging(false);
		}
	    }
	}

	// this one sets the highlighting off, if leaving the cover
	// Attention: this method is also called from mouseReleased, but with null as
	// argument (think about it before using the event...). One can use it, if one
	// wants a npe-blues, though.
	public void mouseExited( MouseEvent evt ){
	    cdoxFrame.getStatusPane().setPositionLabel(new Point());
	    if(!isDragging){
		// if not dragging, de-highlight both current and temp element and set
		// them to null
		if(currentElement!=null){
		    if(currentElement.getHighlight()) currentElement.setHighlight();
		    currentElement=null;
		}
		if(tempElement!=null){
		    if(tempElement.getHighlight()) tempElement.setHighlight();
		    tempElement=null;
		}
		repaint();
	    }
	    // hasMouse will be used to determine, if this method must be called again
	    // when dragging stops (in mouseReleased)
	    hasMouse=false;
	}

	// needed for highlighting
	public void mouseEntered( MouseEvent evt ){
	    hasMouse=true;
	}

	private Element currentElement = null, tempElement = null;
	private boolean isDragging=false;
	private boolean hasMouse=false;
	private Point nullPoint;
	private Point curPoint;
	private Point tempPoint;
	private Point startDragPoint;
	private Dimension startDragSize;
	private Dimension elementDim;
	private Rectangle diagonalResizeRect = null, horizontalResizeRect = null, verticalResizeRect = null;
	private int delx = 0;
	private int dely = 0;
    }

}
