package cdox.edit;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.datatransfer.*;
import cdox.*;
import javax.swing.text.*;
import javax.swing.event.*;
import org.w3c.dom.*;
import java.util.zip.*;

/**
 * This class represents an element that is contained within a document. The
 * GenericElement can contain a java.awt.Component as data.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 */
public class GenericElement extends Element{

    private JComponent component=null;
    /**
     * This is the DataFlavor for cut/pasting a generic element.
     */
    public static DataFlavor COMPONENT_FLAVOR=null;

    /**
     * Creates new element at the specified position.
     *@param pos the position.
     *@param size the size.
     *@param c the component aka the content.
     */
    public GenericElement( Point pos, Dimension size, JComponent c ){
	super(size,pos);
	component=c;
	component.setBorder(null);
	if(COMPONENT_FLAVOR==null){
	    COMPONENT_FLAVOR=new DataFlavor
		(c.getClass(),CDox.getLocalizer().get("genericelement"));
	}
	setResizeRects();
    }

    public DataFlavor[] getTransferDataFlavors(){
	return new DataFlavor[]{COMPONENT_FLAVOR};
    }

    public boolean isDataFlavorSupported( DataFlavor f ){
	return (f.equals(COMPONENT_FLAVOR));
    }

    public Object getTransferData( DataFlavor f ) throws UnsupportedFlavorException{
	if(!f.equals(COMPONENT_FLAVOR)) throw new UnsupportedFlavorException(f);
	return component;
    }

    protected BufferedImage getBufferedImage(){
	BufferedImage img=new BufferedImage(elementSize.width,elementSize.height,BufferedImage.TYPE_INT_ARGB);
	Graphics2D g=(Graphics2D)img.createGraphics();
	SwingUtilities.paintComponent
	    (g,component,new Container(),0,0,
	     elementSize.width,elementSize.height);
	return img;
    }

    /**
     * Attention: there is no support for saving generic elements yet, so this will do
     * nothing!
     */
    protected void saveMyself( org.w3c.dom.Element root, ZipOutputStream out ){
    }

}
