package cdox.edit;

import cdox.util.*;
import cdox.gui.action.*;
import cdox.gui.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This Class represents a TextPane, on which the user can write,
 *it is uses a StyledDocument, and sets all the standards like font,
 *fontSize, position etc...
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @see cdox.edit.CDDrawingPane
 * @see cdox.edit.TextElement
 */
public class TextPane extends JTextPane{
    private javax.swing.text.Element cha;
    private SimpleAttributeSet attributeSet;
    private Point obereEcke;
    private DefaultStyledDocument doc = new DefaultStyledDocument();
    private int prevPoint = 0;
    private int appendIndex = 0;
    
    /**
     * This Constructor is standardly called, and sets the font,
     *fontSize and backgroundColor.
     * @param p the upper left position of the pane,
     * @param f the Font to use,
     * @param size the Fontsize.
     * @param c the Color.
     * @param bold if bold.
     * @param italic if italic.
     * @param underline if underlined.
     */
    public TextPane( Point p, Font f, int size, Color c, boolean bold, boolean italic,
		    boolean underline){
	javax.swing.text.Element root = doc.getParagraphElement(0);
	attributeSet = new SimpleAttributeSet(root.getAttributes());
	StyleConstants.setFontFamily(attributeSet ,f.getFamily());
	doc.setParagraphAttributes(0,1,attributeSet, true);
	root = doc.getCharacterElement(0);
	attributeSet = new SimpleAttributeSet(root.getAttributes());
	StyleConstants.setFontFamily(attributeSet ,f.getFamily());
	doc.setCharacterAttributes(0,1,attributeSet, true);

	setStyledDocument(doc);
	setCaretPosition(0);
	setMargin(new Insets(5,5,5,5));
	setBorder(BorderFactory.createRaisedBevelBorder());
	setBackground(new Color(220, 220, 220, 255));
	setVisible(true);
	setOpaque(false);
	obereEcke = p;
	setAttribute(f, size, c, bold, italic, underline);
    }


    /**
     * Constructor used for loading.
     *@param p the Point. ?
     */
    public TextPane(Point p){
	setStyledDocument(doc);
	setCaretPosition(0);
	setMargin(new Insets(5,5,5,5));
	setBackground(new Color(220, 220, 220, 255));
	setVisible(true);
	setOpaque(false);
	setBorder(BorderFactory.createRaisedBevelBorder());
	obereEcke = p;
    }

    /**
     * Clones the given text pane and makes a new one from its attributes.
     *@param p the text pane to clone.
     */
    public TextPane( final TextPane p ){
	try{
	    PipedOutputStream po=new PipedOutputStream();
	    PipedInputStream pi=new PipedInputStream(po);
	    final ObjectOutputStream out=new ObjectOutputStream(po);
	    new Thread(){
		public void run(){
		    try{
			out.writeObject(p.doc);
		    }catch(IOException ioe){
			CDoxFrame.handleError(ioe,true);
		    }
		}
	    }.start();
	    ObjectInputStream in=new ObjectInputStream(pi);
	    doc=(DefaultStyledDocument)in.readObject();
	    setStyledDocument(doc);
 	    out.close();
 	    in.close();
 	}catch(IOException ioe){
	    CDoxFrame.handleError(ioe,true);
 	}catch(ClassNotFoundException cnfe){
	    CDoxFrame.handleError(cnfe,true);
	}
	setCaretPosition(0);
	setMargin(new Insets(5,5,5,5));
	setVisible(true);
	setOpaque(false);
	obereEcke = (Point)p.obereEcke.clone();
	setCharacterAttributes((AttributeSet)p.attributeSet.clone(),true);
    }

    /**
     *This Method sets the attributes (font, size, color) for the next
     *text-"Element" which is a letter, word or sentence.
     *@param f the Font to write in,
     *@param size the FontSize to write the font in,
     *@param c The color to write the font in,
     *@param bold true if to write bold, false otherwhise,
     *@param italic true if to write italic, false otherwhise,
     *@param underline true if to write underline, false otherwhise.
     */
    public void setAttribute(Font f, int size, Color c, boolean bold, boolean italic, boolean underline){
	attributeSet = new SimpleAttributeSet();
	StyleConstants.setFontFamily(attributeSet ,f.getFamily());
	StyleConstants.setFontSize(attributeSet, size);
	StyleConstants.setForeground(attributeSet, c);
	StyleConstants.setBold(attributeSet, bold);
	StyleConstants.setItalic(attributeSet, italic);
	StyleConstants.setUnderline(attributeSet, underline);
	StyleConstants.setBidiLevel(attributeSet, 10);
	
	setCharacterAttributes(attributeSet, true);
    }
    
    /**
     *This method returns the upperleft corner position, when
     *the panel is created the first time.
     *@return The upperLeft corner of the editable Point 
     */
    public Point getPoint(){
	return obereEcke;
    }

    /**
     *This method sets the upper-left-corner to the new point, which is cloned.
     *@param p the new upper-left-corner.
     *@see CDDrawingPane#reEditTextElement
     */
    public void setPoint(Point p){
	obereEcke = (Point)p.clone();
    }


    /**
     *This method is called when the a saved CDox document with text-elements is loaded.
     *@param b "true" if bold "false" otherwhise,
     *@param i "true" if italic "false" otherwhise,
     *@param u "true" if underline "false" otherwhise,
     *@param fontFamily the name of the fontfamily,
     *@param c the Color in rgb String for example "223330332",
     *@param text the Text to insert/append.
     *@param size the size of the font.
     */
    public void appendText(String b, String i, String u, String fontFamily,
			   String c, String text, String size){
	boolean bold = Boolean.valueOf(b).booleanValue();
	boolean italic = Boolean.valueOf(i).booleanValue();
	boolean underline = Boolean.valueOf(u).booleanValue();
	Color color = new Color(Integer.parseInt(c));
	int fontSize = Integer.parseInt(size);
	attributeSet = new SimpleAttributeSet();
	StyleConstants.setFontFamily(attributeSet ,fontFamily);
	StyleConstants.setFontSize(attributeSet, fontSize);
	StyleConstants.setForeground(attributeSet, color);
	StyleConstants.setBold(attributeSet, bold);
	StyleConstants.setItalic(attributeSet, italic);
	StyleConstants.setUnderline(attributeSet, underline);
	setCharacterAttributes(attributeSet, true);
	setParagraphAttributes(attributeSet, true);
	try{
	    doc.insertString(appendIndex,text, attributeSet);
	}
	catch(BadLocationException ble){
	    CDoxFrame.handleError(ble,false);
	}
	appendIndex += text.length();
    }


    /**
     * Returns the font size at the specified position.
     *
     * @param pos the position.
     * @return the font size.
     */
    public int getFontSizeAtPostion(int pos){
	cha = doc.getCharacterElement(pos);//the element
	attributeSet = new SimpleAttributeSet(cha.getAttributes());
	return StyleConstants.getFontSize(attributeSet);
    }

    /**
     * Returns the font at the specified position.
     *
     * @param pos the position.
     * @return the font name.
     */
    public String getFontAtPosition(int pos){
	cha = doc.getCharacterElement(pos);//the element
	attributeSet = new SimpleAttributeSet(cha.getAttributes());
	return StyleConstants.getFontFamily(attributeSet);
    }

    /**
     *This Method gets the smallest text size used in the textCOmpoent
     *@return the smallest fontsize used in this text.
     */    
    public int getTextSize(){
	int textSize = 1000;
	int tempSize = 0;
	javax.swing.text.Element root = doc.getDefaultRootElement();
	int anfang = 0;
	int ende = root.getEndOffset();
	while(anfang < ende){
	    cha = doc.getCharacterElement(anfang);//the paragraph element
	    attributeSet = new SimpleAttributeSet(cha.getAttributes());
	    tempSize = StyleConstants.getFontSize(attributeSet);
	    if(tempSize<textSize)
		textSize = tempSize;
	    anfang= cha.getEndOffset();
	}
	return textSize;
    }

    /**
     *This Method sets the fontsizes used in this text int the ArrayList, 
     *pro text.Element. (text.Element 0 in arrayList 0 etc).
     *@param fontList the list to put the fontsizes in.
     *@return the filled ArrayList.
     *@see TextElement#TextElement
     */
    public ArrayList getFontSizes(ArrayList fontList){
	javax.swing.text.Element root = doc.getDefaultRootElement();
      	int anfang = 0;
	int ende = root.getEndOffset();
	while(anfang < ende){
	    cha = doc.getCharacterElement(anfang);//the paragraph element
	    attributeSet = new SimpleAttributeSet(cha.getAttributes());
	    fontList.add(new Integer(StyleConstants.getFontSize(attributeSet)));
	    anfang= cha.getEndOffset();
	}
	return fontList;
    }	
    
    /**
     *This Method sets the text.elements with the given fontList sizes.
     *(text.Element0 gets ArrayList[0] etc).
     *@param fontList an ArrayList containing fontsizes.
     *@see TextElement#resetFontSizes
     */
    public void resetFontSizes(ArrayList fontList){
	javax.swing.text.Element root = doc.getDefaultRootElement();
      	int anfang = 0;
	int ende = root.getEndOffset();
	int count = 0;
	while(anfang < ende){
	    cha = doc.getCharacterElement(anfang);//the paragraph element
	    attributeSet = new SimpleAttributeSet(cha.getAttributes());
	    StyleConstants.FontConstants.setFontSize(attributeSet, 
						     ((Integer)fontList.get(count)).intValue());
	    doc.setCharacterAttributes(cha.getStartOffset(), cha.getEndOffset()-cha.getStartOffset(), attributeSet, true);
	    anfang= cha.getEndOffset();
	    count++;
	}
    }

    /**
     *This Method is called when a not editable TextElement is Resized.
     *It iterates over the Elements (e.g. letters, words, or sentences),
     *gets their Font and resizes them with size.
     *@param size is -1 or 1 (smaller, larger) and is adds this to the old Fontsize.
     *@see TextElement#TextElement
     */
    public void resizeAttributeSet(int size){
	javax.swing.text.Element root = doc.getDefaultRootElement();
	int anfang = 0;
	int ende = root.getEndOffset();
	while(anfang < ende){
	     cha = doc.getCharacterElement(anfang);//the paragraph element
	     attributeSet = new SimpleAttributeSet(cha.getAttributes());
	     StyleConstants.FontConstants.setFontSize(attributeSet, 
						      (StyleConstants.FontConstants.getFontSize(attributeSet)+size));
	     doc.setCharacterAttributes(cha.getStartOffset(), cha.getEndOffset()-cha.getStartOffset(), attributeSet, true);
	     anfang= cha.getEndOffset();
	}
    }

    /**
     *This Method is called when a not editable TextElement is Resized.
     *It iterates over the Elements (e.g. letters, words, or sentences),
     *gets their Font and resizes them procentually to their current size.
     *@param procent the relative enlargement of the fontsize.
     *@see TextElement#setSize
     */    
    public synchronized void resizeRelativeAttributeSet(int procent){
	javax.swing.text.Element root = doc.getDefaultRootElement();
	int anfang = 0;
	int ende = root.getEndOffset();
	int count = 0;
	while(anfang < ende){
	     cha = doc.getCharacterElement(anfang);//the paragraph element
	     attributeSet = new SimpleAttributeSet(cha.getAttributes());
	     int fontSize = StyleConstants.FontConstants.getFontSize(attributeSet);
	     if(fontSize <= 1)
		 fontSize = 12;
	     StyleConstants.FontConstants.setFontSize(attributeSet, 
						      (fontSize+
						       (Math.round((float)((float)fontSize/100f)*procent))));
	     doc.setCharacterAttributes(cha.getStartOffset(), cha.getEndOffset()-cha.getStartOffset(), attributeSet, true);
	     anfang= cha.getEndOffset();
	     count++;
	}
    }

    /**
     *This Method sets the text color of a none Editable TextElement.
     *@param c the new Color to draw in.
     */
    public void colorAttributeSet(Color c){
	javax.swing.text.Element root = doc.getDefaultRootElement();

	int anfang = 0;
	int ende = root.getEndOffset();
	while(anfang < ende){
	    cha = doc.getCharacterElement(anfang);//the paragraph element
	    attributeSet = new SimpleAttributeSet(cha.getAttributes());
	    StyleConstants.FontConstants.setForeground(attributeSet, c);
	    doc.setCharacterAttributes(cha.getStartOffset(), cha.getEndOffset()-cha.getStartOffset(), attributeSet, true);
	    anfang= cha.getEndOffset();
	}
    }

    /**
     * This method saves a text element (rather the TextPane).
     *@param cont the content DOM element.
     */
    public void saveMyself( org.w3c.dom.Element cont ){
	javax.swing.text.Element root = doc.getDefaultRootElement();
	int anfang = 0;
	int ende = root.getEndOffset()-1;
	while((anfang) < ende){
	    try{
		cha = doc.getCharacterElement(anfang);//the paragraph element
		attributeSet = new SimpleAttributeSet(cha.getAttributes());
		org.w3c.dom.Element txt=cont.getOwnerDocument().createElementNS
		    ("http://cdox.sf.net/schema/fileformat","text");
		txt.setAttribute("italic",StyleConstants.isItalic(attributeSet)+""); 
		txt.setAttribute("underline",StyleConstants.isUnderline(attributeSet)+""); 
		txt.setAttribute("bold",StyleConstants.isBold(attributeSet)+"");
		txt.setAttribute("font",StyleConstants.getFontFamily(attributeSet));
		txt.setAttribute("color",(StyleConstants.getForeground(attributeSet)).getRGB()+"");
		String s=getText(cha.getStartOffset(), cha.getEndOffset()-cha.getStartOffset());
		try{
		    s=new String(Base64.encode(s.getBytes("utf8")),"utf8");
		}catch(UnsupportedEncodingException uee){
		    CDoxFrame.handleError(uee,true);
		}
		txt.setAttribute("text",s);
		txt.setAttribute("height",StyleConstants.getFontSize(attributeSet)+"");
		cont.appendChild(txt);
		anfang= cha.getEndOffset();
	    }
	    catch(BadLocationException ble){
		CDoxFrame.handleError(ble,true);
	    }
	}
    }
}
