package cdox.gui;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Random;
import java.util.prefs.*;
import javax.naming.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A class that represents the about dialog. It scrolls a Text (saved in an Array of Strings),
 * over the image defined in the constructor. This class uses the Stringplacer class. It also,
 * uses a Timer to scroll the the Text.
 * This dialog has only one ok button.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 22th 2002
 * @see StringPlacer
 */
public class AboutDialog extends JDialog implements ChangeListener, ActionListener{
    private boolean playSound = true;
    private AboutScrollPanel aboutPanel;
    private Localizer lang = CDox.getLocalizer();
    private final String[] scrollString = {lang.get("aboutversion")+" "+CDox.VERSION,
					   lang.get("aboutcdox"),
					   "Rutger Bezema -> cdox@gmx.net",
					   "Andreas Schmitz -> cdox@gmx.net",
					   lang.get("abouttoll"),
					   lang.get("aboutstolz"),
					   lang.get("abouthope"),
					   lang.get("aboutvisit"),
					   lang.get("abouthome"),
					   lang.get("aboutthanx"),
					   "Ien "+ lang.get("abouten")+" Jay,",
					   "Franco Spinelli,",
					   "Nicolas Dubois",
					   "Dr. Christoph Georg Jung",
					   "Marcel Hoffmann."};
    
    private Color currentColor;
    private Random rand;
    private StringPlacer sPlacer;
    private ArrayList placerList;
    private Dimension imageDim;
    private Timer timer;
    private boolean paintString = false;
    private int scrollCount = 0;
    
    /**
     *Constructs a new modal Dialog-frame, with two panes.
     *The two panes are, an image/scroll panel and a button panel with one button.
     *@param owner the owner of this dialog (CDoxFrame),
     *@param title the Title it has.
     */
    public AboutDialog(Frame owner, String title){
	super(owner, title, true);
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);

	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb = new GridBagConstraints();

	//Fr laden vom Image
	Image im = new ImageIcon(getClass().getResource("/cdox/images/cdoxabout.jpg")).getImage();
	imageDim = CDox.getImageDimension();
	im = im.getScaledInstance(imageDim.width, imageDim.height, Image.SCALE_FAST);
	
	//ein neues Panel machen
	aboutPanel = new AboutScrollPanel(im, imageDim);
	aboutPanel.setBorder(BorderFactory.createLoweredBevelBorder());
	gb.insets = new Insets(5,5,5,5);
	getContentPane().add(aboutPanel, gb);	

	//Und ein Button
	JPanel bPanel = new JPanel();
	JButton button = new JButton(lang.get("okbutton"));
	getRootPane().setDefaultButton(button);
	new EscapeListener(button,getRootPane());
	button.addChangeListener(this);
	bPanel.add(button);
	gb.gridy = 1;
	getContentPane().add(bPanel, gb);	
	
	//Die Strings hinzufgen
	Font f = aboutPanel.getFont().deriveFont((float)imageDim.width/24);
	aboutPanel.setFont(f);
	FontMetrics fm = getFontMetrics(aboutPanel.getFont());
	placerList = new ArrayList(scrollString.length);
	int extra = 0;
	for(int i = 0;i<scrollString.length;i++){
	    if(i<6 && i>0)
		extra = 13;
	    else if(i==6)
		extra = 26;
	    else if(i==7 || i==8)
		extra = 39;
	    else if(i>=9)
		extra = 52;
	    sPlacer = new StringPlacer(scrollString[i], fm, imageDim.width, imageDim.height,(13*i+extra));
	    placerList.add(sPlacer);
	}

	//Ein eierkocher setzten
	timer = new Timer(80, this);
	timer.setInitialDelay(10);
	timer.start();
	
	setResizable(false);
	((CDoxFrame)owner).center(this);
	currentColor = Color.black;
	show();
	
    }
    /**
     *Calls the repaint method of the AboutScrollPanel, is called by the Timer.
     *@param e the ActionEvent given by the Timer.
     */
    public void actionPerformed(ActionEvent e){
	paintString = true;
	if(Preferences.userNodeForPackage(CDoxFrame.class).getBoolean("playsounds", true)){
	    PlaySounds.play("about");
	    playSound = false;
	}
	aboutPanel.repaint();
    }	

    /**
     *This method is called when the ok-button is clicked. It stops the Timer, and 
     *disposes the aboutdialog.
     *@param ce the ChangeEvent called from the button click.
     */
    public void stateChanged(ChangeEvent ce){
	timer.stop();
	dispose();
	
    }

    /**
     *A class representing the ScrollPanel.
     */
    private class AboutScrollPanel extends JPanel{
	/**
	 *Construct a new JPanel with the backgroundImage and the dimensions of the
	 *image.
	 *@param im the background image,
	 *@param dim the dimension of the backgroundImage.
	 */
	public AboutScrollPanel(Image im, Dimension dim){
	    setPreferredSize(dim);
	    setMinimumSize(dim);
	    setMaximumSize(dim);
	    image = im;
	}
	
	/**
	 *This method scroll the the text on the background image.
	 *@param g The graphics Object to draw on.
	 */
	public void paint(Graphics g){
	    Graphics2D g2d = (Graphics2D) g;
	    g2d.drawImage(image, 0, 0, this);

	    if(paintString){//nur dann scrollen wenn der Timer dafr auftrag gibt
		if(scrollCount == placerList.size()){
		    for(int i = 0; i<placerList.size();i++){
			((StringPlacer)placerList.get(i)).reset();
		    }
		    scrollCount = 0;
		    playSound = true;
		}
		for(int i = scrollCount;i<placerList.size();i++){
		    if(i==0){
			currentColor = (currentColor == Color.black)? Color.white:Color.black;
			g2d.setPaint(currentColor);
		    }
		    else
			g2d.setPaint(Color.cyan);
		    sPlacer = (StringPlacer)placerList.get(i);
		    g2d.setFont(getFont());
		    g2d.drawString(sPlacer.getString(),
				   sPlacer.getCenterPosition(), 
				   sPlacer.getCurrentHeight());
		    if(sPlacer.getCurrentHeight()<0)
			scrollCount++;
		    sPlacer.setCurrentHeight();
		    
		}
		paintString = false;
	    }
	    paintBorder(g);
	}
	private Image image;
    }
    
}


	
