package cdox.gui;

import cdox.*;
import cdox.data.*;
import cdox.edit.*;
import cdox.gui.action.*;
import cdox.gui.wizard.*;
import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.util.prefs.*;
import java.util.zip.*;
import javax.imageio.*;
import javax.imageio.stream.*;
import javax.print.*;
import javax.swing.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;


/**
 * This is the main frame of the CDox graphical application.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version August 21st 2002 
 */
public class CDoxFrame extends JFrame implements ClipboardOwner, CDCoverStandards, Runnable{

    //The variables
    //The actions of this clas
    private FileAction newAction, openAction, closeAction, saveAction, saveAsAction, printAction,
	exitAction,previewAction, exportAction, propertiesAction;

    private EditAction cutAction, copyAction, pasteAction,optionsAction, undoAction,redoAction,
	rotateAction, reEditTextAction, flipHorizontalAction, flipVerticalAction, moveToBackAction;
    private InsertAction bildAction, backGroundBildAction, backColorAction,removeAction,removeAllAction;
    private HelpAction helpAction, aboutAction;
    private TextAction underlineAction, italicAction, boldAction,colorAction, textAction, fontBoxAction, 
	fontSizeBoxAction;
    private Action popupHelpAction;
    private JMenuItem textMenuItem;
    private EditBildAction editBildBCGAction, editBildSharpenAction, editBildSoftenAction;

    //The mens and toolbars
    private JMenuBar menuBar = new JMenuBar();
    private JToolBar standardToolBar = new JToolBar();//window Toolbar
    private JToolBar textToolBar = new JToolBar("Text");
    private StatusPane statusPane;
    private JScrollPane scrollPane;
    private StatusDrawer statusDraw;
    //For imageediting popupmenu
    private JMenu editBildMenu;

    //a reference on this class
    private CDoxFrame itsme=this;
    private CDox cdoxinstance=null;

    //a few standards.
    private boolean working = false; //indicates the frame is calculating
    private final String imageDir = "/cdox/images/";
    private final Image icon  = new ImageIcon(getClass().getResource(imageDir+"cdox.gif"))
	.getImage();

    //the Editpane
    private EditPane editPane = new EditPane(this);

    //the dialogs
    private JColorChooser colorChooser=new JColorChooser();
    private MyFileChooser getFiles;
    private CoverDialog newDialog;

    //for saving
    private boolean isSaved=true;
    /**
     * The file which is currently used to save this project in
     */
    public File saveFile=null;

    //for the wizard
    private boolean wizardOn=false;
    private CdoxWizard wizardInstance = null;

    // miscellaneous
    private Localizer lang=CDox.getLocalizer();
    private ConfigurationDialog configurationDialog;
    private HashMap actions=new HashMap(); // all actions mapped correctly for
					   // configuration
    // the data for the properties dialog:
    private Data propertiesData=null;

    //The users working directory
    private File workingDirectory;
    private Preferences prefs;

    //Text etc.
    private Font actualFont;
    private int actualFontSize;
    private JComboBox fontBox, fontSizeBox; 
    private Vector fontVector;
    private boolean textEnabled = false, insertButtonPressed = false,
	textBold = false, textItalic = false, textUnderline = false;
    private Color textColor=Color.black;

    private PrintService[] allPrinters= null;
    private PrintService defaultPrinter=null;
    private boolean lookingForPrinters=false;
    private EscapeListener textEscape = null;

    public CDoxFrame( CDox cdox, String filename ){
	this(cdox);
	setWorking(true);
	// and now, open the file
	File file = new File(filename);
	Cover[] c=null;
	if(file.exists()) c=readDocuments(file);
	else{
	    handleError(new Throwable(lang.get("opencdxwrongformat")),false);
	}
	if(c==null) {
	    setWorking(false);//reset the working icon
	    return;
	}
	if(c.length==0){
	    handleError(new Throwable(lang.get("opencdxwrongformat")),false);
	    setWorking(false);//reset the working icon
	    return;
	}
	saveFile=file;
	if(getEditPane().hasComponents()){
	    //TODO A SAVE ACTION
	    getEditPane().removeCoverTemplates();//remove them
	}
	if(getTextEnabled()){
	    updateTextBar(null);
	}
	addCovers(c,file.getName());
	if(!(getExtendedState()==MAXIMIZED_BOTH))//if maximised do not center
	    center(this);
	paintComponents(getGraphics());
	setActionEnabled("redo",false);
	setActionEnabled("undo",false);
	EditAction.redo=null;
	EditAction.undo=null;
	setWorking(false);
    }

    /**
     * <P>This is the main constructor of the program, if this works everything works. First
     * we add a windowListener to insure that closing is done properly. Then the cdox-icon
     * is set. After that the menubar and its menus are made. There are four menus:
     * <ul type="disk">
     * <li>File</li>
     * <li>Edit</li>
     * <li>Insert</li>
     * <li>Help</li>
     * </ul>
     * The actions are initialized menu and the resulting menu-items are added to the
     * menus. Also there are some Actions <I>(line: 186)</I> innitialized for the
     * popupMenus and the  TextToolbar <I>(line: 245)</I>. The menus are added! </P>
     * <P>After that the configurable Actions are set<I>(line: 309)</I>. Then the
     * fontSizeBox is filled up to 72, and the Fonts are loaded <I>(line: 357)</I>. The
     * fonts'actions are added to the font(Size)Box, and some standard stuff for the
     * comboboxes <I>(line: 410)</I>. </P>
     * <P>The Buttons for the textToolBar are created <I>(line: 430)</I>, and after that
     * the toolbars are added to an extra panel, which itself will be added together with
     * the statusDrawer (ox) to the NorthPane, which on its turn is added to the
     * contentPane <I>(line: 454)</I>.</P>
     * <P> The ToolbarButtons are added for the textToolBar and the genericToolbar.
     * <I>(line: 365)</I>. Actions are set Enabled and some not <I>(line: 501)</I>. The
     * File chooser is initialized (so that it recursively can start to go through all
     * disks etc). The EditPane (Which will contain the CDCover's which will contain the
     * CDDrawingPanes, which interact with Cover) is added to a ScrollPane which is added
     * to the ContentPane in the CENTER. The statusPane is added to the SOUTH. The
     * editPane is filled with a front and back-side <I>(line: 539)</I>.</P>
     * <P> All that needs to be done now is the newDialog to initialize and the frame
     * needs to be centered. After this the start-sound is played.</P>
     * @param cdx a <code>CDox</code> value.
     * @see #addCovers(int[])
     * @see #center
     * @see ConfigurationDialog
     * @see CoverDialog
     * @see EditAction
     * @see EditBildAction
     * @see EditPane
     * @see FileAction
     * @see HelpAction
     * @see InsertAction
     * @see MyFileChooser
     * @see StatusDrawer
     * @see StatusPane
     * @see TextAction
     * @see ToolBarPane
     */
    public CDoxFrame(final CDox cdx){
	cdoxinstance=cdx;
	prefs=Preferences.userNodeForPackage(getClass());

	setIconImage(icon);//fr icon on titleBar
	
	cdx.setProgress(5, lang.get("startcolor"));
	//do some menu creating
	setJMenuBar(menuBar); //add the menu bar to the window
	JMenu fileMenu = new JMenu(lang.get("fileMenu"));
	JMenu editMenu = new JMenu(lang.get("editMenu"));
	JMenu insertMenu = new JMenu(lang.get("insertMenu"));
	JMenu helpMenu = new JMenu(lang.get("helpMenu"));
	editBildMenu = new JMenu(lang.get("editbildMenu"));
	
	fileMenu.setMnemonic(lang.getStroke("fileMenu"));
	editMenu.setMnemonic(lang.getStroke("editMenu"));
	insertMenu.setMnemonic(lang.getStroke("insertMenu"));
	helpMenu.setMnemonic(lang.getStroke("helpMenu"));

	//create Actions for menu and controlbar
	JMenuItem item;
 	item=addMenuItem(fileMenu, newAction = new FileAction
			 (this, "new", lang.get("newTip")));
	newAction.setMenuItem(item);
	item=addMenuItem(fileMenu, openAction = new FileAction
			 (this, "open",lang.get("openTip")));
	openAction.setMenuItem(item);
	fileMenu.addSeparator();//ein seperator
 	item=addMenuItem(fileMenu, saveAction = new FileAction
			 (this, "save", lang.get("saveTip")));
	saveAction.setMenuItem(item);
 	item=addMenuItem(fileMenu, saveAsAction = new FileAction
			 (this, "saveas",lang.get("saveasTip")));
	saveAsAction.setMenuItem(item);
	fileMenu.addSeparator();//ein seperator
	item=addMenuItem(fileMenu, exportAction = new FileAction
			 (this, "export",lang.get("exportTip")));
	exportAction.setMenuItem(item);

	fileMenu.addSeparator();//ein Separator
	item=addMenuItem(fileMenu, previewAction = new FileAction
			 (this, "preview", lang.get("previewTip")));
	previewAction.setMenuItem(item);
	item=addMenuItem(fileMenu, printAction = new FileAction
			 (this, "print", lang.get("printTip")));
	printAction.setMenuItem(item);
	// properties entries
	item=addMenuItem(fileMenu, propertiesAction = new FileAction
			 (this, "properties", lang.get("propertiesTip")));
	propertiesAction.setMenuItem(item);
	// end properties entries
	fileMenu.addSeparator();//ein Separator
 	item=addMenuItem(fileMenu, exitAction = new FileAction
			 (this, "exit", lang.get("exitTip")));
	exitAction.setMenuItem(item);
	//set the progressbar
	cdx.setProgress(10, lang.get("startmenu"));
	//The Edit menu...
	item=addMenuItem(editMenu, undoAction = new EditAction
			 (this, "undo", lang.get("undoTip")));
	undoAction.setMenuItem(item);
	item=addMenuItem(editMenu, redoAction = new EditAction
			 (this, "redo", lang.get("redoTip")));
	redoAction.setMenuItem(item);
	editMenu.addSeparator();//Ein Separator
	item=addMenuItem(editMenu, cutAction = new EditAction
			 (this, "cut", lang.get("cutTip")));
	cutAction.setMenuItem(item);
	item=addMenuItem(editMenu, copyAction = new EditAction
			 (this, "copy", lang.get("copyTip")));
	copyAction.setMenuItem(item);
	item=addMenuItem(editMenu, pasteAction = new EditAction
			 (this, "paste", lang.get("pasteTip")));
	pasteAction.setMenuItem(item);
	editMenu.addSeparator();
	item=addMenuItem(editMenu, optionsAction = new EditAction
			 (this, "options", lang.get("optionsTip")));
	optionsAction.setMenuItem(item);

	rotateAction = new EditAction(this, "rotate", lang.get("rotateTip"));
	flipHorizontalAction = new EditAction(this, "fliphorizontal",
					      lang.get("fliphorizontalTip"));
	flipVerticalAction = new EditAction(this, "flipvertical", lang.get("flipverticalTip"));
	removeAction= new InsertAction(this, "remove", lang.get("removeTip"));
	removeAllAction= new InsertAction(this, "removeall", lang.get("removeallTip"));
	reEditTextAction= new EditAction(this, "reedittext", lang.get("reedittextTip"));

	moveToBackAction = new EditAction(this, "movetoback", lang.get("movetobackTip"));
	
	
	//fr popup menu edit-bild-menu
	item = addMenuItem(editBildMenu, editBildBCGAction = new EditBildAction
			   (this, "editbildbcg", lang.get("editbildbcgTip")));
	editBildBCGAction.setMenuItem(item);
	item = addMenuItem(editBildMenu, editBildSharpenAction = new EditBildAction
			   (this, "editbildsharpen", lang.get("editbildsharpenTip")));
	editBildSharpenAction.setMenuItem(item);
	item = addMenuItem(editBildMenu, editBildSoftenAction = new EditBildAction
			   (this, "editbildsoften", lang.get("editbildsoftenTip")));
	editBildSoftenAction.setMenuItem(item);
	
	//set the progressBar
	cdx.setProgress(15, lang.get("startmenu"));
	//The Insert Menu
	

	item=addMenuItem(insertMenu, backGroundBildAction = new InsertAction
			 (this, "backgroundbild", lang.get("backgroundbildTip")));
	backGroundBildAction.setMenuItem(item);
	item=addMenuItem(insertMenu, backColorAction=new InsertAction
			 (this, "bcolor",lang.get("bcolorTip")));
	backColorAction.setMenuItem(item);
	insertMenu.addSeparator(); 
	item=addMenuItem(insertMenu, bildAction = new InsertAction
			 (this, "bild", lang.get("bildTip")));
	bildAction.setMenuItem(item);
 	textMenuItem = addMenuItem(insertMenu, textAction = new TextAction(this, lang.get("texton"), lang.get("textTip")));
	textAction.setMenuItem(textMenuItem);
	item=addMenuItem(insertMenu, colorAction=new TextAction
			 (this, lang.get("color"),lang.get("colorTip")));
	colorAction.setMenuItem(item);


	//set the progressBar
	cdx.setProgress(20, lang.get("startmenu"));	
	
	//The Help menu
	item=addMenuItem(helpMenu, helpAction = new HelpAction
			 (this,"help", lang.get("helpTip")));
	helpAction.setMenuItem(item);
	item=addMenuItem(helpMenu, aboutAction = new HelpAction
			 (this,"about", lang.get("aboutTip")));
	aboutAction.setMenuItem(item);

	menuBar.add(fileMenu);
	menuBar.add(editMenu);
	menuBar.add(insertMenu);
	menuBar.add(helpMenu);

	// some popup menu help
	popupHelpAction=new HelpAction(this,"help",lang.get("helpTip"));
	
	// map all actions in the actions HashMap that should be configured
	// file menu
	actions.put("new",newAction);
	actions.put("open",openAction);
	//	actions.put("close",closeAction);
	actions.put("save",saveAction);
	actions.put("saveas",saveAsAction);
	actions.put("export",exportAction);
	actions.put("preview",previewAction);
	actions.put("print",printAction);
	actions.put("properties",propertiesAction);
	actions.put("exit",exitAction);
	// edit menu
	actions.put("redo",redoAction);
	actions.put("undo",undoAction);
	actions.put("cut",cutAction);
	actions.put("copy",copyAction);
	actions.put("paste",pasteAction);
	actions.put("options",optionsAction);
	// insert menu
	actions.put("insertbackimage",backGroundBildAction);
	actions.put("setbackcolor",backColorAction);
	actions.put("insertimage",bildAction);
	actions.put("textmode",textAction);
	actions.put("setcolor",colorAction);
	// help menu
	actions.put("help",helpAction);
	actions.put("about",aboutAction);
	// popup menu
	actions.put("remove",removeAction);
	actions.put("removeall",removeAllAction);
	actions.put("rotate",rotateAction);
	actions.put("reedittext",reEditTextAction);
	actions.put("editbildbcg",editBildBCGAction);
	actions.put("editbildsharpen",editBildSharpenAction);
	actions.put("editbildsoften",editBildSoftenAction);
	actions.put("fliphorizontal",flipHorizontalAction);
	actions.put("flipvertical",flipVerticalAction);
	actions.put("movetoback", moveToBackAction);
	
	actions.put("coverchange", editPane.getTab());
	// configure'em
	configure(false); // do not show dialog

	//set the progressBar
	cdx.setProgress(25,lang.get("startmenudone"));

	//Die beiden Boxes for Font eigenschaften
	//gre
	CellFontRenderer cRnd = new CellFontRenderer();
	fontSizeBox = new JComboBox();
	fontSizeBox.setRenderer(cRnd);
	for(int i = 8;i<=72;){
	    if(i<36){
		fontSizeBox.addItem(new Integer(i));
		i+=2;
	    }
	    else{
		fontSizeBox.addItem(new Integer(i));
		i+=12;
	    }
	    
	}
	fontSizeBox.setSelectedIndex(2);
	actualFontSize = 12;

	//set the progressBar
	cdx.setProgress(35, lang.get("startfontsize"));

	//Font
	GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
	Font[] allThemFonts= allThemFonts = env.getAllFonts();
	
	fontVector = new Vector();
	int fontCount = 0;
	fontBox = new JComboBox();
	int fontSelected = 0;

	//set the progressBar
	cdx.setProgress(45, lang.get("startfontcheck"));
	cRnd = new CellFontRenderer();
	fontBox.setRenderer(cRnd);
 	for(int i = 0; i< allThemFonts.length; i++){
	    String family = allThemFonts[i].getFamily();
	    if((allThemFonts[i].isPlain()) 
	       && (allThemFonts[i].canDisplayUpTo(family) == -1)
	       && (!(fontVector.contains(allThemFonts[i].getFamily().toLowerCase())))){
		if((allThemFonts[i].getFamily().toLowerCase()).startsWith("times")){
		    actualFont = allThemFonts[i].deriveFont((float)actualFontSize);
		    fontSelected = fontCount;
		}
		fontCount++;
		fontBox.addItem(allThemFonts[i]);
		fontVector.add(allThemFonts[i].getFamily().toLowerCase());
	    }
	}
	//Check to see if any fonts are loaded, if not the program won't work.
	if(fontBox.getItemCount()<=1){
	    CDoxFrame.handleError(new Throwable("no fonts installed"), false);
	    System.exit(1);
	}
	//set the progressBar	private Point bPoint;
	cdx.setProgress(55, lang.get("startfontsort"));

	//set actions to fonts
	fontSizeBoxAction = new TextAction(this, "sizeBox", null);
	fontBoxAction = new TextAction(this, "fontBox",null);
	fontBox.addActionListener(fontBoxAction);
	fontBox.setBackground(Color.white);
	fontBox.setMaximumSize(fontBox.getPreferredSize());
	if(actualFont == null){
	    fontBox.setSelectedIndex(1);
	    actualFont = (Font)fontBox.getSelectedItem();
	}
	else
	    fontBox.setSelectedIndex(fontSelected);
	fontSizeBox.addActionListener(fontSizeBoxAction);
	fontSizeBox.setBackground(Color.white);
	fontSizeBox.setMaximumSize(fontSizeBox.getPreferredSize());



	//set the progressBar
	cdx.setProgress(60,lang.get("startfontdone"));

	//Hier werden die Font buttons implementiert
	italicAction = new TextAction(this, "italic", lang.get("italicTip"));
	underlineAction = new TextAction(this, "underline", lang.get("underlineTip"));
	boldAction = new TextAction(this, "bold", lang.get("boldTip"));
	
	//For the Toolbar
	JPanel northPane = new JPanel(new BorderLayout());
	ToolBarPane toolBarPane = new ToolBarPane(this,new GridBagLayout());
	toolBarPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), 
								 BorderFactory.createEmptyBorder(2,1,2,1)));
	GridBagConstraints tbc = new GridBagConstraints();
	standardToolBar.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), 
								     BorderFactory.createEmptyBorder(2,1,2,1)));
	textToolBar.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(), 
								 BorderFactory.createEmptyBorder(2,1,2,1)));
	textToolBar.setCursor(new Cursor(Cursor.MOVE_CURSOR));
	fontSizeBox.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	fontBox.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	tbc.insets = new Insets(0,2,0,2);
	tbc.gridx = 0;
	tbc.weightx = 2000;
	tbc.weighty = 2000;
	tbc.anchor = tbc.WEST;
	
	//the mouse handler for the draggin' and moving are added.
	textToolBar.addMouseMotionListener(toolBarPane.getListener());
	textToolBar.addMouseListener( toolBarPane.getListener());
	toolBarPane.add(standardToolBar, tbc);

	toolBarPane.add(textToolBar, tbc);

	toolBarPane.setConstraints(tbc);
	northPane.add(toolBarPane, BorderLayout.CENTER);
	statusDraw = new StatusDrawer(this);
	northPane.add(statusDraw, BorderLayout.EAST);
	getContentPane().add(northPane, BorderLayout.NORTH);

	//load buttons in toolbars
	standardToolBar.addSeparator();
	addToolBarButton(newAction);
	addToolBarButton(openAction);
	addToolBarButton(saveAction);
	standardToolBar.addSeparator();
	addToolBarButton(printAction);
	standardToolBar.addSeparator();
	addToolBarButton(undoAction);
	addToolBarButton(redoAction);
	standardToolBar.addSeparator();
	addToolBarButton(cutAction);
	addToolBarButton(copyAction);
	addToolBarButton(pasteAction);
	standardToolBar.addSeparator();
	addToolBarButton(backGroundBildAction);
	addToolBarButton(bildAction);

	//TextToolBar
	textToolBar.addSeparator();
	addToolBarButton(textAction);
	textToolBar.addSeparator();
	textToolBar.add(fontBox);
	textToolBar.add(fontSizeBox);
	textToolBar.addSeparator();
	addToolBarButton(boldAction);
	addToolBarButton(italicAction);
	addToolBarButton(underlineAction);
	textToolBar.addSeparator();
	addToolBarButton(colorAction);
	//set the progressBar
	cdx.setProgress(65, lang.get("starttoolbar"));

	standardToolBar.setFloatable(false);
	textToolBar.setFloatable(true);

	//turning these action off
	bildAction.setEnabled(true);
	boldAction.setEnabled(false);
	colorAction.setEnabled(false);
	copyAction.setEnabled(false);
	cutAction.setEnabled(false);
	fontBox.setEnabled(false);
	fontSizeBox.setEnabled(false);
	italicAction.setEnabled(false);
	newAction.setEnabled(true);
	openAction.setEnabled(true);
	pasteAction.setEnabled(true);
	redoAction.setEnabled(false);
	saveAction.setEnabled(true);
	saveAsAction.setEnabled(true);
	textAction.setEnabled(true);
	underlineAction.setEnabled(false);
	undoAction.setEnabled(false);


	//set the progressBar
	cdx.setProgress(70, lang.get("startactions"));

	//for the JFileChooser Object
	getFiles = new MyFileChooser();

	//set the progressBar
	cdx.setProgress(80, lang.get("startfilechooser"));
	
        //fge ein neues EditPane ein
	scrollPane = new JScrollPane(editPane);
	scrollPane.addComponentListener(editPane.addNewComponentHandler());
	getContentPane().add(scrollPane);
	//editPane.setTabListener();
	
	//Fge ein neus statuspane ein
	statusPane = new StatusPane();
	getContentPane().add(statusPane, BorderLayout.SOUTH);

	//forder und rckseite
	int[] cov={0,-1,2};
	addCovers(cov);
	//set the progressBar
	cdx.setProgress(90, lang.get("startcolorchooser"));

	//mache ein neues Dialog
	newDialog = new CoverDialog(this, lang.get("dialogtitel"));
	new Thread(this, "findPrinters").start();

	center(this);
	//set the progressBar
	cdx.setProgress(100, lang.get("startengage"));
	if(Preferences.userNodeForPackage(CDoxFrame.class).getBoolean("playsounds",true))
	    PlaySounds.play("start");
    }


    /**
     * In this run method the printers are searched for at start up. This can enhance the
     * speed, especially for network printers, of the printing process significantly.
     */
    public void run(){
	lookingForPrinters=true;
	allPrinters=PrintServiceLookup.lookupPrintServices(DocFlavor.SERVICE_FORMATTED.PRINTABLE,null);
	defaultPrinter= PrintServiceLookup.lookupDefaultPrintService();
	lookingForPrinters=false;
    }

    /**
     * Returns all printers on the System, capable of printing this document.
     * @return the PrintServices (speak:"printers"), installed on this system.
     */
    public PrintService[] getAllPrinters(){
	while(lookingForPrinters){
	    try{
		Thread.currentThread().sleep(100);
	    }
	    catch(InterruptedException ie){CDoxFrame.handleError(ie, true);}
	}
	return allPrinters;
    }

    /**
     * Returns the default printer installed on this system.
     * @return the default_PrintService (speak:"default_printer"), installed on this system.
     */
    public PrintService getDefaultPrinter(){
	while(lookingForPrinters){
	    try{
		Thread.currentThread().sleep(100);
	    }
	    catch(InterruptedException ie){CDoxFrame.handleError(ie, true);}
	}
    	return defaultPrinter;
    }

    /**
     *This method recursivly calls all the components containted in the contentPane, this
     *is used to set the mouse to its wait value in the statusDrawer... Note that it also
     *contains things like the Scollbars arrow button and stuff like that.
     *@param cont the current Container
     *@param ar the ArrayList to save the containers
     *@return the filled arraylist.
     */
    private ArrayList getAllComponents(Container cont, ArrayList ar){
	ar.add(cont);
	int count = 0;
	while(cont.getComponentCount() != 0 && count < cont.getComponentCount()){
	    getAllComponents((Container)cont.getComponent(count),ar);
	    count++;
	}
	return ar;
    }	     

    /**
     *The Constructor calls this Method for adding Actions to the FileMenu.
     *And than returns a JMenuItem for the FileMenu. 
     *@param menu the menu to add the item to
     *@param action the Action to add to the item
     *@param keystroke the litteral for this item
     */
    private JMenuItem addMenuItem(JMenu menu, Action action){
	JMenuItem item = menu.add(action);
	item.setIcon(null);
	return item;
    }

    /**
     *The Constructor calls this Method for adding JButtons for the ToolBar.
     *this Method returns a JButton.
     *In this mehtod each button creates it's own MouseListener, which let the button being "pushed".
     *@param action is the action to add to the Toolbar
     */
    private JButton addToolBarButton(Action action){
	JButton button = null;
	if(action instanceof TextAction){
	    String name = (String)action.getValue(action.NAME);
	    if(name.equals(textAction.getValue(Action.NAME))){
		button = new JButton(textAction);
		button.setText(lang.get("textactionbuttondisplay"));
		button.setFont(button.getFont().deriveFont(16f));
		button.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	    else if(name.equals(underlineAction.getValue(action.NAME))){
		button = new JButton(underlineAction);
		button.setText(lang.get("underlineactionbuttondisplay"));
		button.setFont(button.getFont().deriveFont(Font.ROMAN_BASELINE,16f));
		button.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	    else if(name.equals(boldAction.getValue(action.NAME))){
		button = new JButton(boldAction);
		button.setText(lang.get("boldactionbuttondisplay"));
		button.setFont(button.getFont().deriveFont(Font.BOLD,16f));
		button.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	    else if(name.equals(italicAction.getValue(action.NAME))){
		button = new JButton(italicAction);
		button.setText(lang.get("italicactionbuttondisplay"));
		button.setFont(button.getFont().deriveFont(Font.ITALIC, 16f));
		button.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		
	    }
	    if(name.equals(colorAction.getValue(Action.NAME))){
		button = new JButton(colorAction);
		button.setText("ABC");
		button.setForeground(Color.red);
		button.setFont(button.getFont().deriveFont(8f));
		button.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	    button.setPreferredSize(new Dimension(21,21));
	    button.setMinimumSize(new Dimension(21, 21));
	    button.setMaximumSize(new Dimension(21, 21));
	    textToolBar.add(button);
	}
		
	else{
	    button = standardToolBar.add(action);
	}
	button.setToolTipText((String)action.getValue(action.SHORT_DESCRIPTION));//fr tooltip
	button.setBorder(BorderFactory.createRaisedBevelBorder());//gib button ein rand
	if(!(action instanceof TextAction)){
	    button.addMouseListener(new MouseAdapter()   {
		    public void mousePressed(MouseEvent e){
			((JButton)e.getSource()).setBorder(BorderFactory.createLoweredBevelBorder());
		    }
		    public void mouseReleased(MouseEvent e){
			((JButton)e.getSource()).setBorder(BorderFactory.createRaisedBevelBorder());
		    }
		});
	    button.setText(null);
	}
	else if(action == colorAction){
	    button.addMouseListener(new MouseAdapter()   {
		    public void mousePressed(MouseEvent e){
			((JButton)e.getSource()).setBorder(BorderFactory.createLoweredBevelBorder());
		    }
		    public void mouseReleased(MouseEvent e){
			((JButton)e.getSource()).setBorder(BorderFactory.createRaisedBevelBorder());
		    }
		});
	}
	return button;
    }


    /**
     * Sets the given windows to the center of the screen and packs it.
     * @param win the window to center on the screen.
     */
    public void center(Window win){
	win.pack();
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension fs = win.getSize();
	if((ss.width<fs.width)||(ss.height<fs.height)){
	    setExtendedState(MAXIMIZED_BOTH);
	    return;
	}
        win.setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
    }

    /**
     *This method should be called from all exceptions, it is able to play a sound and
     *write in the log-file of CDox-errors.
     *@param e the Trowable that caused the problem
     *@param writeLog true to write the problem in the log, false for a normal error
     *message.
     */
    public static void handleError(Throwable e, boolean writeLog){
	if(Preferences.userNodeForPackage(CDoxFrame.class).getBoolean("playsounds", true)){
	    PlaySounds.play("alert");
	}
	if(writeLog){
	    cdox.util.ErrorHandler.log(e, true);
	}
	else{
	     JOptionPane.showMessageDialog
		 (null,
		  e.getMessage(),
		  CDox.getLocalizer().get("error"),
		  JOptionPane.ERROR_MESSAGE);
	}
    }

    /**
     * This method shows a question dialog to the user, and returns the value the user
     *chosen. It also plays a sound.
     * @param question the question to be displayed
     * @param option the JOptionPane options for example JOptionPane.YES_NO_OPTION.
     * @return an the value the user has clicked (YES NO CANCEL)
     */
    public static int showQuestion(String question, int option){
	if(Preferences.userNodeForPackage(CDoxFrame.class).getBoolean("playsounds", true)){
	    PlaySounds.play("question");
	}
	int value = JOptionPane.showConfirmDialog(null, 
						  question,
						  CDox.getLocalizer().get("question"),
						  option);
	return value;
    }
    

    /**
     * Reads an image out of the stream. This one creates a BufferedImage WITHOUT a
     * IndexedColorModel.
     *@param in where to read the image.
     *@return the image.
     */
    public static BufferedImage loadImage( InputStream in ){
	try{
	    return TempFiles.loadImage(in);
	}catch(IOException ioe){
	    CDoxFrame.handleError
		(new Throwable
		 (CDox.getLocalizer().get
		  ("loadimageio",new String[]{ioe.getLocalizedMessage()}),ioe),false);
	}
	return null;
    }
	
    /**
     * This method is inherited from ClipBoardOwner and does nothing.
     * @param c the Clipboard which is this class contains
     * @param contents the content of the Clipboard
     */
    public void lostOwnership( Clipboard c, Transferable contents ){
    }

    /**
     * Sets whether we can cut'n'waste objects. Causes the cut/copy actions to be
     * available, if true.
     * @param b whether we can be cut'n'wasted.
     */
    public void setCutNWasteable( boolean b ){
	cutAction.setEnabled(b);
	copyAction.setEnabled(b);
    }

    /**
     * This Method returns the ScrollPane for use in other classes.
     * @return the ScrollPane.
     */
    public JScrollPane getScrollPane(){
	return scrollPane;
    }  

    /**
     *This method returns the editPane
     *@return the editpane
     */
    public EditPane getEditPane(){
	return editPane;
    }
    
    /**
     *Returns the popup imageEditMenu.
     *@return the imageEditpopupMenu
     */
    public JMenu getImageEditPopupMenu(){
	return editBildMenu;
    }

    /**
     *This method returns the statusPane.
     *@return the statuspane.
     */
    public StatusPane getStatusPane(){
	return statusPane;
    }

    /**
     *This method returns the statusDrawer.
     *@return the statusDrawer.
     */
    public StatusDrawer getStatusDrawer(){
	return statusDraw;
    }

    /**
     *This method sets the working-icon on or off.
     *@param work true if working false if not.
     */
    public void setWorking(boolean work){
	statusDraw.setWorking(work);
	working = work;
    }

    /**
     *This method returns true if the program is calculating something.
     *@return true if calculating false otherwhise.
     */
    public boolean isWorking(){
	return working;
    }

    /**
     *This method returns the toobar
     *@return the toolbar
     */
    public JToolBar getStandardToolBar(){
	return standardToolBar;
    }
    
    /**
     *This method returns the texttoobar
     *@return the toolbar
     */
    public JToolBar getTextToolBar(){
	return textToolBar;
    }
    
    /**
     *This method returns the fileChooser dialog
     *@return the fileDialog
     */
    public MyFileChooser getFileDialog(){
	return getFiles;
    }

    /**
     *This method returns the Colordialog
     *@return the ColorDialog
     */
    public JColorChooser getColorChooser(){
	return colorChooser;
    }

    /**
     *This method returns the newDialog
     *@return the newDialog
     */    
    public CoverDialog getNewDialog(){
	return newDialog;
    }


    /**
     *Creates a new ActionEvent with a newAction, and calls its actionperformed method.
     */
    public void showNewWizard(){
	ActionEvent ae = new ActionEvent (newAction, 18, null); //eigenes event hat wert 18
	newAction.actionPerformed(ae);
    }

    public void wizardOn(boolean on, CdoxWizard wiz){
	wizardInstance = wiz;
	wizardOn=on;
    }

    public boolean isWizardOn(){
	return wizardOn;
    }

    public CdoxWizard getWizardInstance(){
	return wizardInstance;
    }

    /**
     *This method returns the fontBox (combobox).
     *@return the fontbox.
     */
    public JComboBox getFontBox(){
	return fontBox;
    }

    /**
     *This method returns the fontSizeBox (combobox).
     *@return the fontSizebox.
     */
    public JComboBox getFontSizeBox(){
	return fontSizeBox;
    }
    
    /**
     * Returns the index of the fontfamily, which is coherent with the index of the Font in
     * the fontBox.
     * @param fontFamily the String representing the current used font
     * @return the index of the fontFamily
     * @see CaretUpdater#caretUpdate
     */
    public int getFontIndex(String fontFamily){
	return fontVector.indexOf(fontFamily.toLowerCase());
    }

    /**
     * This method adds the drawingPanes to the EditPane, it calls the same
     * method in editpane. The array of ints are defined as: front, booklet, back -> with
     * following values: 0, 1, 2. The not used values should be -1. If for example a
     * project should have a backside and a front the arra would look like [0, -1, 2].
     * @param whichCovers an array of ints decribing the kind of covers to be displayed
     * in this project.
     */
    public void addCovers(int[] whichCovers){
	editPane.addCoverTemplates(whichCovers, 
				   actualFont, 
				   textBold, 
				   textItalic, 
				   textUnderline,
				   textColor);//Get the choises made by the user, and add
					      //them
	//register all components with their Cursors to the statusDrawer.
	boolean check = working;
	if(check)
	    setWorking(false);
	statusDraw.registerComponents(getAllComponents(getContentPane(),new ArrayList()));
	if(check)
	    setWorking(true);
    }

    /**
     * This method adds the drawingPanes to the EditPane, it calls the same
     *method in editpane. This one gets already created covers as argument.
     * @param whichCovers the covers allready created.
     * @param title the new title of the frame, apart from the CDFront/.../ stuff.
     */
    public void addCovers( Cover[] whichCovers, String title){
	editPane.addCoverTemplates(whichCovers, 
				   actualFont, 
				   textBold, 
				   textItalic, 
				   textUnderline,
				   textColor,title);
	//register all components with their Cursors to the statusDrawer.
	boolean check = working;
	if(check)
	    setWorking(false);
	statusDraw.registerComponents(getAllComponents(getContentPane(),new ArrayList()));
	if(check)
	    setWorking(true);
    }

    /**
     * Sets the saveaction and isSaved variable.
     * @param b true means the cover has to be saved.
     */
    public void setSave( boolean b ){
	isSaved=!b;
	if(b){
	    setActionEnabled("undo",true);
	    setMyTitle("*");
	}
	else{//
	    setMyTitle("");
	}
	
    }

    /**
     * Sets the default saveFile to null.
     */
    public void resetSaveFile(){
	saveFile=null;
    }

    /**
     * This method checks to see whether an opened file have to be saved or not.
     * To do so it uses the boolean isSaved which is standardly put on true,and altered to
     * false when a file has been opened and edited. If false, a dialog is opened, asking
     * for a filename, if one has been chosen, it will be saved, otherwise nothing
     * happens.
     * @see FileAction#actionPerformed
     */
    public void checkSaveable(){
	if(isSaved) {
	    return;
	}
	if(saveFile!=null) saveFile(saveFile);
	else{
	    int[] filters = {CDX};
	    File f=getFiles.showDialog(this, lang.get("filetitlecdoxsave"), 
				   lang.get("fileapprovesavebutton"), null, filters);
	    if(f==null) return;
	    saveFile(f);
	    saveFile = f;
	}
    }
    
    /**
     *Sets the saveFile to f.
     *@param f the file to save_as to.
     */
    public void saveAs(File f){
	saveFile(f);
	saveFile = f;
    }

    /**
     *This Method is concerned whith the saving of a modified or not modified(saveAs)file.
     *@param file is the file to save
     */
    public void saveFile(File file){
	try{
	    ZipOutputStream out=new ZipOutputStream(new FileOutputStream(file));
	    DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	    DocumentBuilder db=dbf.newDocumentBuilder();
	    Document doc=db.newDocument();
	    org.w3c.dom.Element e=doc.createElementNS
		("http://cdox.sf.net/schema/fileformat","cdox");
	    e.setAttribute("xmlns","http://cdox.sf.net/schema/fileformat");
	    e.setAttribute("version","1");
	    doc.appendChild(e);
	    if(propertiesData!=null){
		org.w3c.dom.Element d=(org.w3c.dom.Element)e.appendChild
		    (doc.createElementNS("http://cdox.sf.net/schema/fileformat","description"));
		propertiesData.storeData(d);
	    }
	    Cover[] c=editPane.getCovers();
	    for(int i=0;i<c.length;i++) {
		c[i].saveMyself(doc,out);
	    }
	    editPane.saveMyPreviewSelf(out);
	    out.putNextEntry(new ZipEntry("documents.cdx"));
	    Transformer t=TransformerFactory.newInstance().newTransformer();
	    DOMSource src=new DOMSource(doc);
	    StreamResult res=new StreamResult(out);
	    t.transform(src,res);
	    out.closeEntry();
	    out.finish();
	    out.close();
	}catch(IOException ioe){
	    CDoxFrame.handleError(ioe, false);
	}catch(ParserConfigurationException pce){
	    CDoxFrame.handleError(pce,true);
	}catch(TransformerConfigurationException tce){
	    CDoxFrame.handleError(tce,true);
	}catch(TransformerException te){
	    CDoxFrame.handleError(te,true);
	}
	setMyTitle(file.getName());
	setSave(false);
    }

    /**
     *Sets the title of the frame. Checks if the frameTitle exists, if not the title is
     *set appropriate to s. If frametitle exists s is checked for the "*" and "" which
     *means not saved and saved. If this is methoc is called with those the Title is
     *updated. If the file is saved, automatically the projects name is entered in the
     *frametitlebar.
     *@param s a String which can be a new file, an "*" a "" or the name of the project.
     *@see #setSave
     *@see #saveFile
     *@see EditPane#addCoverTemplates
     */
    public void setMyTitle(String s){
	StringBuffer sb = new StringBuffer(getTitle());
	if(s.equals("*")||s.equals("")){
	    int index = sb.indexOf("*");
	    if(index ==-1){//no asterix 
		sb.append(s);
	    }
	    else sb.replace(index, sb.length(), s);
	}
	else{
	    int index = sb.indexOf(": ");
	    sb.replace(index+2, sb.length(),s);
	}
	setTitle(sb.toString());
    }

    /**
     * This method will read the contents of the file. The file must be in the cdox
     * format, meaning that it's a zipfile containing a xml document named documents.cdx
     * and possibly some files containing information about elements (like
     * images), like the one that is created by the saveFile method. Returns the covers
     * that have been in the file.
     * @param f the file to read.
     * @return the reconstructed covers.
     */
    public Cover[] readDocuments( File f ){
	Cover[] covers=null;
	try{
	    CDoxLoader loader=new CDoxLoader();
	    loader.parse(f);
	    covers=loader.getCovers();
	    propertiesData=loader.getData();
	}catch(IOException ioe){
 	    CDoxFrame.handleError(new Throwable(lang.get("opencdxerror"), ioe),false);
	    return null;
	}catch(SAXException saxe){
	    CDoxFrame.handleError(saxe,true);
	    return new Cover[0];
	}catch(ParserConfigurationException pce){
	    CDoxFrame.handleError(pce,true);
	    return new Cover[0];
	}
	return covers;
    }

    /**
     * This Method is called to toggle the "for files" appropriate Actions on-or-off.
     * @param name the inside-the-program-name for the action
     * @param set is the boolean to set the actions to.
     */
    public void setActionEnabled(String name, boolean set){
	if(name.equals("new")){
	    newAction.setEnabled(set);
	}
	else if(name.equals("open")){
	    openAction.setEnabled(set);
	}
	else if(name.equals("close")){
	    closeAction.setEnabled(set);
	}
	else if(name.equals("save")){
	    saveAction.setEnabled(set);
	}
	else if(name.equals("saveAs")){
	    saveAsAction.setEnabled(set);
	}
	else if(name.equals("print")){
	    printAction.setEnabled(set);
	}
	else if(name.equals("properties")){
	    propertiesAction.setEnabled(set);
	}
	else if(name.equals("exit")){
	    exitAction.setEnabled(set);
	}
	else if(name.equals("preview")){
	    previewAction.setEnabled(set);
	}
	else if(name.equals("cut")){
	    cutAction.setEnabled(set);
	}
	else if(name.equals("copy")){
	    copyAction.setEnabled(set);
	}
	else if(name.equals("redo")){
	    redoAction.setEnabled(set);
	}
	else if(name.equals("undo")){
	    undoAction.setEnabled(set);
	}
	else if(name.equals("paste")){
	    pasteAction.setEnabled(set);
	}
	else if(name.equals("color")){
	    colorAction.setEnabled(set);
	}
	else if(name.equals("backColor")){
	    backColorAction.setEnabled(set);
	}
	else if(name.equals("bild")){
	    bildAction.setEnabled(set);
	}
	else if(name.equals("backGroundBild")){
	    backGroundBildAction.setEnabled(set);
	}
	else if(name.equals("help")){
	    helpAction.setEnabled(set);
	}
	else if(name.equals("about")){
	    aboutAction.setEnabled(set);
	}
	else if(name.equals("underline")){
	    underlineAction.setEnabled(set);	    
	}
	else if(name.equals("italic")){
	    italicAction.setEnabled(set);
	}
	else if(name.equals("bold")){
	    boldAction.setEnabled(set);
	}
	else if(name.equals("text")){
	    textAction.setEnabled(set);
	}
    }

    /**
     *This method returns a String representing this frame.
     *@return a string representing this frame.
     */
    public String toString(){
	return "CDoxFrame";
    }

    /**
     *This method returns the directory of the images.
     *@return a string representing the directory of the images.
     */
    public String getImageDir(){
	return imageDir;
    }

    /**
     * Sets the current undo action.
     *@param undo the action.
     */
    public void setUndoAction( cdox.gui.undo.DoAction undo ){
	setActionEnabled("undo", true);
	setActionEnabled("redo", false);
	EditAction.undo=undo;
    }

    /**
     * This Method passes the Background image id to the drawPane,
     * and sets the background flag from its parent (activeCover). For the mode parameter
     * see the Cover methods.
     * @param id the String which is a valid unique id for the File on disk
     * @param mode the mode.
     * @exception IOException if an error occurs.
     * @see Cover#setBackground(String,boolean)
     */
    public void setBackgroundImage( String id, boolean mode ) throws IOException{
	editPane.getDrawingPane().setBackgroundImage(id,mode);
    }

    
    /**
     * Passes the BufferedImage to the CDrawingPane with its TempFiles-key and the
     * fit-mode
     * @param bi the allready loaded bufferedbackgroundImage
     * @param id the String which is a valid unique id for the File on disk
     * @param mode the mode.
     */
    public void setBackgroundImage( BufferedImage bi, String id, boolean mode){
	editPane.getDrawingPane().setBackgroundImage(bi, id, mode);
    }

    /**
     * Sets the backgroundColor of the current cover to the Color bc.
     * @param bc the new backgroundColor
     */
    public void setBackgroundColor(Color bc){
	editPane.getActiveCover().setBackgroundColor(bc);
	editPane.getDrawingPane().setBackgroundColor(bc);
    }
    /**
     *This method is called when the user wants to change the backgroundimage
     *@return true if activeCover has a backgroundImage false otherwhise
     */
    public boolean hasBackground(){
	return editPane.getActiveCover().hasBackground();
    }

    /**
     * This Method adds a normal Image to the active cover.
     * @param id the String which is a valid unique id for the File on disk
     * @exception IOException if an error occurs.
     */
    public void setImage(String id) throws IOException{
	editPane.getDrawingPane().addImage(id);
    }

    /**
     *This method is called to set the drawingPane in TextMode.
     *@param te true if entering text mode, false otherwhise.
     */
    public void setTextEnabled(boolean te){
	if(te)
	    textEscape = new EscapeListener((JButton)textToolBar.getComponent(1), getRootPane());
	else
	    if(textEscape != null)
		textEscape.invalidateListener();
	textEnabled = te;
	editPane.getDrawingPane().setTextEnabled(te);
    }

    /**
     *This method returns if true if the drawingPane is in TextMode.
     *@return true if in textMode false otherwhise.
     */
    public boolean getTextEnabled(){
	return textEnabled;
    }

    /**
     * This method is called when the user is in textmode, and klicks on the new button,
     * it "fakes" an action event.
     * @param command defining what happens, if on one cover in textmode clicked on
     * another cover command will be "cover" else its null.
     * @see TextAction#actionPerformed
     */
    public void updateTextBar(String command){
	ActionEvent ae = new ActionEvent (textAction, 18, command); //eigenes event hat wert 18
	textAction.actionPerformed(ae);
    }
    
    /**
     *This method arranges the borders on the textToolBar buttons.
     *@param actionName the name of the action to set the Border.
     */
    public void setActionBorder(String actionName){
	JButton button = new JButton();
	if(actionName.equals("text")){
	    button = (JButton)textToolBar.getComponent(1);
	    if(textMenuItem.getText().equals(lang.get("texton"))){
		statusPane.setModeLabel(lang.get("statuspanetexton"));
		textMenuItem.setText(lang.get("textoff"));
	    }
	    else{
		statusPane.setModeLabel(lang.get("statuspanetextoff"));
		textMenuItem.setText(lang.get("texton"));
	    }
	}
	else if(actionName.equals("bold")){
	    button = (JButton)textToolBar.getComponent(6);
	}
	else if(actionName.equals("italic")){
	    button = (JButton)textToolBar.getComponent(7);
	}
	else if(actionName.equals("underline")){
	    button = (JButton)textToolBar.getComponent(8);
	}
	if(button.getBorder() == BorderFactory.createLoweredBevelBorder()){
	    button.setBorder(BorderFactory.createRaisedBevelBorder());
	}
	else	
	    button.setBorder(BorderFactory.createLoweredBevelBorder());
    }
    
    /**
     *This method sets all textstyles like bold, underline and italic off.
     */
    public void setTextStylesOff(){
	textBold = false;
	textUnderline = false;
	textItalic = false;
    }
    
    /**
     *This method returns the actual font.
     *@return the actual font.
     */    
    public Font getActualFont(){
	return actualFont;
    }
    
    /**
     *This method sets the Font to write with.
     *@param f the font to write with.
     */
    public void setActualFont(Font f){
	actualFont = f;
	editPane.getDrawingPane().setTextFont(f);
    }

    /**
     *This method returns the actualfontSize.
     *@return the actual fontsize.
     */    
    public int getActualFontSize(){
	return actualFontSize;
    }

    /**
     *This method sets fontSize to write with, and updates actualFont.
     *@param as the font size to write with.
     */
    public void setActualFontSize(int as){
	actualFontSize = as;
	actualFont = actualFont.deriveFont((float)as);
	editPane.getDrawingPane().setFontSize(as);
    }

    /**
     *This Method sets whether to write in bold or not
     *@param b true to write in bold false otherwhise.
     */
    public void setTextBold(boolean b){
	textBold = b;
	editPane.getDrawingPane().setBold(b);
    }
    
    /**
     *This Method sets whether to write in underline or not
     *@param u true to write in underline false otherwhise.
     */
    public void setTextUnderline(boolean u){
	textUnderline = u;
	editPane.getDrawingPane().setUnderline(u);
    }
    
    /**
     *This Method sets whether to write in italic or not
     *@param i true to write in italic false otherwhise.
     */
    public void setTextItalic(boolean i){
	textItalic = i;
	editPane.getDrawingPane().setItalic(i);
    }

    /**
     *This method returns true if the text is written in bold.
     *@return true if the bold-style is set, false otherwhise.
     */    
    public boolean getTextBold(){
	return textBold;
    }

    /**
     *This method returns true if the text is written with underline.
     *@return true if the underline-style is set, false otherwhise.
     */    
    public boolean getTextUnderline(){
	return textUnderline;
    }

    /**
     *This method returns true if the text is written in italic.
     *@return true if the italic-style is set, false otherwhise.
     */    
    public boolean getTextItalic(){
	return textItalic;
    }
    
    /**
     *This method sets Fontcolor to write with.
     *@param tc the fontcolor to write with.
     */
    public void setTextColor(Color tc){
	textColor = tc;
	editPane.getDrawingPane().setTextColor(tc);
    }

    /**
     *This method returns true if the textcolor.
     *@return the color of the text.
     */    
    public Color getTextColor(){
	return textColor;
    }

    /**
     * Rotates the selected element a by the given degrees. If no element is selected,
     * nothing happens.
     *@param deg the degrees by which to rotate, for example 90.
     */
    public void rotateElement( double deg ){
	cdox.edit.Element e=editPane.getDrawingPane().getCover().getSelectedElement();
	if(e!=null) e.rotateElement(deg);
    }
    
    /**
     * Sets the specified horizontal flipping of the selected element.
     */
    public void flipElementHorizontal(){
	cdox.edit.Element e=editPane.getDrawingPane().getCover().getSelectedElement();
	if(e!=null) e.flipElementHorizontal();
    }

    /**
     * Sets the specified Vertical flipping of the selected element.
     */
    public void flipElementVertical(){
	cdox.edit.Element e=editPane.getDrawingPane().getCover().getSelectedElement();
	if(e!=null) e.flipElementVertical();
    }
    /**
     *This method is called when a text object needs to be re-edited.(popup menu)
     */
    public void reEditTextElement(){
	cdox.edit.Element e = editPane.getDrawingPane().getCover().getSelectedElement();
	if((e!=null) && (e instanceof TextElement))
	   editPane.getDrawingPane().reEditTextElement(e);
    }

    /**
     * Removes all elements from the current Cover.
     */
    public void removeAllElements(){
	editPane.getDrawingPane().setElementSelected(null);
	editPane.getDrawingPane().getCover().removeAllElements();

	editPane.getDrawingPane().repaint();
	paintComponents(getGraphics());
	repaint();
	
    }

    /**
     *Sets the popup to be used on the selected element.
     *@param invoker the active drawing pane,
     *@param x the x-position of the popup menu,
     *@param y the y-position of the popup menu,
     *@param title the title of the popup.
     */
    public void showPopup(Component invoker, int x, int y, String title){
	ContextPopupMenu popup;
	Action[] actions = new Action[16];
	actions[1] = null;
	actions[2] = rotateAction;
	actions[3] = null;
	actions[4] = flipHorizontalAction;
	actions[5] = flipVerticalAction;
	actions[6] = moveToBackAction;
	actions[7] = null;
	actions[8] = removeAction;
	actions[9] = removeAllAction;
	actions[10] = null;
	actions[11] = popupHelpAction;
	actions[12] = null;
	actions[13] = cutAction;
	actions[14] = copyAction;
	actions[15] = pasteAction;
	if(title!= null){
	    if(title.equals(lang.get("popuptextelementtitle"))){
		actions[0] = reEditTextAction;
		popup = new ContextPopupMenu(title, invoker, x, y, actions, null);
	    }
	    else if (title.equals(lang.get("popupbildelementtitle"))){
		actions[0] = null;
		popup = new ContextPopupMenu(title, invoker, x, y, actions, editBildMenu);
	    }
	    else if (title.equals(lang.get("popupbackgroundtitle"))){
		int amount= (editPane.getDrawingPane().getCover().hasElements())? 7:5;
		actions = new Action[amount];
		actions[0] = null;
		actions[1] = null;
		actions[2] = bildAction;
		actions[3] = null;
		actions[4] = removeAction;
		if(editPane.getDrawingPane().getCover().hasElements()){
		    actions[5]= null;
		    actions[6]=removeAllAction;
		}
		popup = new ContextPopupMenu(title, invoker, x, y, actions, editBildMenu);
	    }
	}
	//no background and not clicked on element...
	else{
	    int amount= (editPane.getDrawingPane().getCover().hasElements())? 5:3;
	    actions = new Action[amount];
	    actions[0] = bildAction;
	    actions[1] = null;
	    actions[2] = backGroundBildAction;
	    if(editPane.getDrawingPane().getCover().hasElements()){
		actions[3]= null;
		actions[4]=removeAllAction;
	    }
	    popup = new ContextPopupMenu(title, invoker, x, y, actions, null);
	}
    }
    /**
     * Sets which element is selected.
     *@param e the element, or null, if no element should be selected.
     */
    public void setSelectedElement( cdox.edit.Element e ){
	editPane.getDrawingPane().setElementSelected(e);
	repaint();
    }

    /**
     * If show is true, it shows the configuration dialog, if not, it creates a new
     * one. In either case the actions are configured.
     *@param show whether to show or create a new configuration dialog.
     */
    public void configure( boolean show ){
	if(show){
	    configurationDialog.show();
	    if(configurationDialog.lastOKPressed()&&configurationDialog.mustRestart()){
		int option=JOptionPane.showConfirmDialog
		    (this,lang.get("optionsmessage"),lang.get("options"),
		     JOptionPane.YES_NO_OPTION);
		if(option==JOptionPane.YES_OPTION) CDox.restartCDox();
	    }
	}else configurationDialog=new ConfigurationDialog
		 (this,lang.getLanguageCode(),CDoxFrame.class.getResourceAsStream
		  ("/cdox/util/conf/example.xml"));
	configurationDialog.configure(actions);
    }

    /**
     * Shuts the application down.
     */
    public void stop(){
	int value = cdoxinstance.stop();
	if(value != JOptionPane.CANCEL_OPTION)
	    System.exit(0);
    }

    /**
     * Checks, if the actual document has been saved. If not, it asks the user, if it
     * should be saved.
     * @return the value the user selected as answer (YES NO)
     */
    public int checkCloseable(){
	int value = JOptionPane.NO_OPTION;
	if(!isSaved){
	    value = CDoxFrame.showQuestion(lang.get("notsaved"), JOptionPane.YES_NO_CANCEL_OPTION);
	    if(value==JOptionPane.YES_OPTION) checkSaveable();
	}
	return value;
    }

    /**
     * Is overwritten from JFrame, it enables the user to cancel the accidentially close
     * click.
     * @param we the WindowEvent.
     */
    protected void processWindowEvent(WindowEvent we){
	if(we.getID()==WindowEvent.WINDOW_CLOSING){
	    int value = cdoxinstance.stop();
	    if(value != JOptionPane.CANCEL_OPTION){
		super.processWindowEvent(we);
		System.exit(0);
	    }
	}
    }


    /**
     * Returns the Data for the current project or null, if none has been set.
     * @return the properties of this project.
     */
    public Data getData(){
	return propertiesData;
    }

    /**
     * Sets the data for the current project.
     *@param d the new data.
     */
    public void setData( Data d ){
	propertiesData=d;
	if(d != null)
	    statusPane.setDataLabel(d.getDescription());
	else
	    statusPane.setDataLabel(lang.get("statusnodata"));
    }
}
