package cdox.gui;

import java.awt.*;
import javax.swing.*;


/**
 * This tiny class is used to set the background color of the FontBox and FontSizeBox the
 * size of the fonts in the FontBox are set to 14f. The selected background color is the
 * SystemColor for activeCaption and white if not selected, the foreground is System
 * dependend to an black if not selected.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version may 12th 2002
 */
class CellFontRenderer extends JLabel implements ListCellRenderer {
    /**
     *Construct a new Cell renderer.
     *@see CDoxFrame#CDoxFrame
     */
     public CellFontRenderer() {
         setOpaque(true);
     }

    /**
     *This method sets the desired fontsize and color to the given JList. The size of the
     *fonts in the JList are set to 14f. The selected background color is the SystemColor
     *for activeCaption and white if not selected, the foreground is System dependend to
     *an black if not selected.
     *@param list the JList to render it cells defined here
     *@param value the Object within the cell
     *@param index the index of the cell
     *@param isSelected true if this cell is selected falls otherwhise
     *@param cellHasFocus if the cell has the focus.
     *@return renderer for this JLists cells.
     */
     public Component getListCellRendererComponent(
         JList list,
         Object value,
         int index,
         boolean isSelected,
         boolean cellHasFocus)
     {

	
	 if(value instanceof java.awt.Font){	 
	     setText(((Font)value).getFamily());
	     setFont(((Font)value).deriveFont(14f));
	 }
	 else
	     setText(value.toString());

	 setBackground(isSelected ? SystemColor.activeCaption : Color.white);
         setForeground(isSelected ? SystemColor.activeCaptionText : Color.black);
         return this;
     }
 }
