package cdox.gui;

import javax.swing.*;
import java.awt.*;
import cdox.util.Localizer;
import cdox.gui.*;
import cdox.data.*;
import java.awt.event.*;


/**
 * GUI class to manage the data information.
 *
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version Nov 11th 2002
 */
public class ChooseDataDialog extends JDialog implements ActionListener, ItemListener{
    private Localizer lang;
    private CDoxFrame cFrame;
    private GridBagConstraints gb = new GridBagConstraints();
    private Class[] dataClasses = PluginFactory.getAvailableDataClasses();
    private Data[] data = new Data[dataClasses.length];
    private DataSource[] dataSources;
    private JLabel noSources;
    private JLabel chooseType;
    private JLabel chooseSource;
    private JPanel rightPanel = new JPanel(new GridBagLayout());
    private JPanel leftPanel = new JPanel(new GridBagLayout());
    private int dataSelected = -1;
    private int dataSourceSelected = -1;
    private JButton autoButton,userButton, cancelButton;
    private Data wizardData = null;
    private boolean wizardOn = false;

    
    /**
     * Creates a new <code>ChooseDataDialog</code> instance.
     *
     * @param cF a <code>CDoxFrame</code> value
     * @param l a <code>Localizer</code> value
     */
    public ChooseDataDialog(CDoxFrame cF, Localizer l){
	super(cF, l.get("choosedata"));
	setModal(true);
	cFrame = cF;
	lang = l;
	getContentPane().setLayout(new GridBagLayout());
	
	//The descriptive labels
	noSources = new JLabel(lang.get("nosources"));
	chooseType = new JLabel(lang.get("cdtype"));
	chooseSource = new JLabel(lang.get("datasource"));
	//Setting some font styles to the labels
	chooseType.setFont(chooseType.getFont().deriveFont(Font.ITALIC));
	chooseSource.setFont(chooseSource.getFont().deriveFont(Font.ITALIC));
	noSources.setFont(noSources.getFont().deriveFont(Font.BOLD));

	
	//The panels used in this dialog
	JPanel buttonPanel = new JPanel(new GridBagLayout());

	//containing the left and right panels
	JPanel topPanel = new JPanel(new GridBagLayout());
	topPanel.setPreferredSize(new Dimension(250,150));
	topPanel.setMaximumSize(new Dimension(250,150));
	topPanel.setMinimumSize(new Dimension(250,150));
	topPanel.setBorder(BorderFactory.createEtchedBorder());

	//the dataType panel
	JScrollPane leftScroll = new JScrollPane(leftPanel);
	leftScroll.setPreferredSize(new Dimension(125, leftPanel.getPreferredSize().height));
	leftScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	leftScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	//the dataSource panel
	JScrollPane rightScroll = new JScrollPane(rightPanel);
	rightScroll.setPreferredSize(new Dimension(125, rightPanel.getPreferredSize().height));
	rightScroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	rightScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

	//adding the left panel to the topPanel
	gb.fill = gb.BOTH;
	gb.anchor= gb.WEST;
	gb.weighty=1f;
	gb.weightx=1f;
	topPanel.add(leftScroll,gb);
	
	//adding the right panel to the topPanel
	gb.gridx = 1;
	gb.anchor= gb.EAST;
	topPanel.add(rightScroll,gb);

	//adding the topPanel to the contentpane
	gb.gridx = 0;
	gb.weighty=1f;
	gb.anchor = gb.NORTH;
	gb.insets = new Insets(5,5,0,5);
	getContentPane().add(topPanel, gb);

	try{
	    for(int i=0;i<dataClasses.length;i++)
		data[i]=(Data)dataClasses[i].newInstance();
	}
	catch(InstantiationException ie){
	    CDoxFrame.handleError(ie, true);
	}
	catch(IllegalAccessException iae){
	    CDoxFrame.handleError(iae, true);
	}
	
	//adding the descriptive labels to the panels
	gb.weightx=1f;
	gb.weighty=1f;
	gb.anchor = gb.NORTHWEST;
	gb.fill = gb.NONE;
	gb.insets = new Insets(0,3,0,0);
	gb.gridy=0;
	leftPanel.add(chooseType, gb);
	rightPanel.add(chooseSource, gb);
	
	//Adding dataTypes to leftPanel
	ButtonGroup bg = new ButtonGroup();
	gb.gridy=1;
	for(int i=0; i<dataClasses.length;i++){
	    JRadioButton rButton = new JRadioButton
		(data[i].getDescription());
	    if(cFrame.getData()!= null){//checking for older versions of data's
		if(data[i].getDescription().equals(cFrame.getData().getDescription())){
		    rButton.setSelected(true);
		    dataSelected = i;
		}
	    }
	    else{
		if(i==0){
		    rButton.setSelected(true);
		    dataSelected=0;
		}
	    }
	    rButton.addItemListener(this);
	    leftPanel.add(rButton,gb);
	    gb.gridy++;
	    bg.add(rButton);
	}
	//Checking for sources
	gb.gridy=1;
	Class[] temp=PluginFactory.getDataSourcesFor(dataClasses[dataSelected]);
	if(temp.length!=0){
	    dataSources = new DataSource[temp.length];
	    try{
		for(int i=0;i<temp.length;i++)
		    dataSources[i]=(DataSource)temp[i].newInstance();
	    }
	    catch(InstantiationException ie){
		CDoxFrame.handleError(ie, true);
	    }
	    catch(IllegalAccessException iae){
		CDoxFrame.handleError(iae, true);
	    }
	    ButtonGroup bgr = new ButtonGroup();
	    for(int j=0; j<temp.length;j++){
		// this fixed the bug with "test" label on data sources
		JRadioButton rButton = new JRadioButton(dataSources[j].getDescription());
		if(j==0){
		    rButton.setSelected(true);
		    dataSourceSelected=0;
		}
		rightPanel.add(rButton, gb);
		bgr.add(rButton);
		gb.gridy++;
	    }
	}
	else {//No sources for first dataType
	    rightPanel.add(noSources, gb);
	    dataSources=null;
	    dataSourceSelected = -1;
	}


	cancelButton = new JButton(lang.get("cancelbutton"));
	cancelButton.addActionListener(this);
	userButton = new JButton(lang.get("userbutton"));
	userButton.addActionListener(this);
	autoButton = new JButton(lang.get("autobutton"));
	autoButton.addActionListener(this);
	//if user button is larger as autobutton
	if(userButton.getPreferredSize().width>autoButton.getPreferredSize().width){
	    if(userButton.getPreferredSize().width>cancelButton.getPreferredSize().width){//user is largest
		cancelButton.setPreferredSize(userButton.getPreferredSize());
		autoButton.setPreferredSize(userButton.getPreferredSize());
	    }
	    else{//cancel is largest
		userButton.setPreferredSize(cancelButton.getPreferredSize());
		autoButton.setPreferredSize(cancelButton.getPreferredSize());
	    }
	}
	//else if auto button is larger as cancelbutton, auto is largest
	else if(autoButton.getPreferredSize().width>cancelButton.getPreferredSize().width){
	    cancelButton.setPreferredSize(autoButton.getPreferredSize());
	    userButton.setPreferredSize(autoButton.getPreferredSize());
	}
	//else cancel is largest
	else{
	    userButton.setPreferredSize(cancelButton.getPreferredSize());
	    autoButton.setPreferredSize(cancelButton.getPreferredSize());
	}
	    
	if(dataSourceSelected == -1)
	    autoButton.setEnabled(false);

	//reseting the constraints and adding the buttons to the buttonpanel
	gb.weightx=0f;
	gb.weighty=0f;
	gb.gridy=0;
	gb.anchor = gb.CENTER;
	gb.insets = new Insets(0,3,0,3);
	buttonPanel.add(userButton, gb);
	gb.gridx = 1;
	buttonPanel.add(autoButton, gb);
	gb.gridx=2;
	buttonPanel.add(cancelButton, gb);
	
	//adding the buttonpanel to the contentPane
	gb.gridx= 0;
	gb.gridy=1;
	gb.anchor=gb.SOUTH;
	gb.insets=new Insets(5,0,5,0);
	getContentPane().add(buttonPanel,gb);
	setResizable(false);
	cFrame.center(this);
    }

    /**
     * Creates a new <code>ChooseDataDialog</code> instance.
     *
     * @param cF a <code>CDoxFrame</code> value
     * @param l a <code>Localizer</code> value
     * @param dat a <code>Data</code> value
     */
    public ChooseDataDialog(CDoxFrame cF, Localizer l, Data dat){
	this(cF, l);
	wizardData=dat;
	wizardOn = true;
    }

    public void showIt(){
	show();
    }



    public void actionPerformed(ActionEvent ae){
	String name=((JButton)ae.getSource()).getText();
	if(name.equals(lang.get("autobutton"))){
	    if(dataSourceSelected != -1){
		hide();
		if(!wizardOn){
		    if(cFrame.getData() != null){
			if(!cFrame.getData().getClass().equals(data[dataSelected].getClass())){//other than now
			    int answer=CDoxFrame.showQuestion(lang.get("reusesource"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){//user wanted it
				if(dataSources[dataSourceSelected].hasConfigurationDialog())
				    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
				cFrame.setData(dataSources[dataSourceSelected].retrieveData());
				if(cFrame.getData() != null)
				    cFrame.getData().userFillData(cFrame);
				dispose();
				cFrame.setWorking(false);
				cFrame.setSave(true);
			    }
			    else{
				cFrame.getData().userFillData(cFrame);
				dispose();
				cFrame.setWorking(false);
				cFrame.setSave(true);
			    }
			}
			else{//Data is same as before, but user pushed source button, ask
			     //if he wants to delete old data
			    int answer=CDoxFrame.showQuestion(lang.get("reusesource"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){//user wanted it
				if(dataSources[dataSourceSelected].hasConfigurationDialog())
				    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
				cFrame.setData(dataSources[dataSourceSelected].retrieveData());
				if(cFrame.getData() != null)
				    cFrame.getData().userFillData(cFrame);
				dispose();
				cFrame.setWorking(false);
				cFrame.setSave(true);
			    }
			    else{
				cFrame.getData().userFillData(cFrame);
				dispose();
				cFrame.setWorking(false);
				cFrame.setSave(true);
			    }
			}
		    }
		    else{//this is the first entered data...
			if(dataSources[dataSourceSelected].hasConfigurationDialog())
			    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
			cFrame.setData(dataSources[dataSourceSelected].retrieveData());
			if(cFrame.getData() != null)
			    cFrame.getData().userFillData(cFrame);
			dispose();
			cFrame.setWorking(false);
			cFrame.setSave(true);    
		    }
		}
		else{
		    if(wizardData != null){
			if(!wizardData.getClass().equals(data[dataSelected].getClass())){
			    int answer=CDoxFrame.showQuestion(lang.get("reusesource"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){
				if(dataSources[dataSourceSelected].hasConfigurationDialog())
				    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
				wizardData=dataSources[dataSourceSelected].retrieveData();
				if(wizardData != null)
				    wizardData.userFillData(cFrame);
				dispose();
			    }
			    else{
				wizardData.userFillData(cFrame);
				dispose();
			    }
			}
			else{//Data is same as before, but user pushed source button, ask
			     //if he wants to delete old data
			    int answer=CDoxFrame.showQuestion(lang.get("reusesource"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){
				if(dataSources[dataSourceSelected].hasConfigurationDialog())
				    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
				wizardData=dataSources[dataSourceSelected].retrieveData();
				if(wizardData != null)
				    wizardData.userFillData(cFrame);
				dispose();
			    }
			    else{
				wizardData.userFillData(cFrame);
				dispose();
			    }
			}
		    }
		    else{
			if(dataSources[dataSourceSelected].hasConfigurationDialog())
			    dataSources[dataSourceSelected].showConfigurationDialog(cFrame);
			wizardData=dataSources[dataSourceSelected].retrieveData();
			if(wizardData != null)
			    wizardData.userFillData(cFrame);
			dispose();
		    }
		}
	    }
	}
	else if(name.equals(lang.get("userbutton"))){
	    hide();
	    if(dataSelected != -1){
		if(!wizardOn){
		    if(cFrame.getData() != null){
			if(!cFrame.getData().getClass().equals(data[dataSelected].getClass())){
			    int answer=CDoxFrame.showQuestion(lang.get("typechangequestion"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){
				cFrame.setData(data[dataSelected]);
				if(cFrame.getData()!= null)
				    cFrame.getData().userFillData(cFrame);
				dispose();
				cFrame.setWorking(false);
				cFrame.setSave(true);
			    }
			}
			else{
			    cFrame.getData().userFillData(cFrame);
			    dispose();
			    cFrame.setWorking(false);
			    cFrame.setSave(true);
			}
		    }
		    else{
			cFrame.setData(data[dataSelected]);
			if(cFrame.getData()!= null)
			    cFrame.getData().userFillData(cFrame);
			dispose();
			cFrame.setWorking(false);
			cFrame.setSave(true);    
		    }
		}
		else{
		    if(wizardData != null){
			if(!wizardData.getClass().equals(data[dataSelected].getClass())){
			    int answer=CDoxFrame.showQuestion(lang.get("typechangequestion"), JOptionPane.YES_NO_OPTION);
			    if(answer == JOptionPane.OK_OPTION){
				wizardData=data[dataSelected];
				if(wizardData!= null)
				    wizardData.userFillData(cFrame);
				dispose();
			    }
			}
			else{
			    wizardData.userFillData(cFrame);
			    dispose();
			}
		    }
		    else{
			wizardData=data[dataSelected];
			if(wizardData!= null)
			    wizardData.userFillData(cFrame);
			dispose();
		    }
		}
	    }
	}
	else if(name.equals(lang.get("cancelbutton"))){
	    hide();
	    dispose();    
	    if(!wizardOn){
		cFrame.setWorking(false);
		cFrame.setSave(true);    
	    }
	}
    }

    /**
     * Describe <code>getWizardData</code> method here.
     *
     * @return a <code>Data</code> value
     */
    public Data getWizardData(){
	return wizardData;
    }
    
    public void itemStateChanged(ItemEvent ie){
	if(ie.getStateChange() == ie.SELECTED){
	    JRadioButton source = (JRadioButton)ie.getSource();
	    String rString = source.getText();
	    if(((JPanel)source.getParent())==leftPanel){
		dataSources = null;
		dataSourceSelected = -1;
		autoButton.setEnabled(false);
		rightPanel.removeAll();
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weightx=1f;
		gbc.weighty=1f;
		gbc.anchor = gb.NORTHWEST;
		gbc.fill = gb.NONE;
		gbc.insets = new Insets(0,3,0,0);
		gbc.gridy=0;
		rightPanel.add(chooseSource, gbc);	    
		gbc.gridy=1;
		
		for(int i=0; i<dataClasses.length;i++){
		    if(rString.equals(data[i].getDescription())){
			dataSelected=i;
			Class[] temp=PluginFactory.getDataSourcesFor(dataClasses[i]);
			if(temp.length!=0){
			    dataSources = new DataSource[temp.length];
			    try{
				for(int k=0;k<temp.length;k++)
				    dataSources[k]=(DataSource)temp[k].newInstance();
			    }
			    catch(InstantiationException iec){
				CDoxFrame.handleError(iec, true);
			    }
			    catch(IllegalAccessException iae){
				CDoxFrame.handleError(iae, true);
			    }
			    ButtonGroup bg = new ButtonGroup();
			    for(int j=0; j<dataSources.length;j++){
				JRadioButton rButton = new JRadioButton(dataSources[j].getDescription());
				if(j==0)rButton.setSelected(true);
				rButton.addItemListener(this);
				autoButton.setEnabled(true);
				rightPanel.add(rButton, gbc);
				bg.add(rButton);
				gbc.gridy++;
				
			    }
			    dataSourceSelected = 0;
			}
			else {
			    rightPanel.add(noSources, gbc);
			}
		    }
		}
	    }
	    else{
		if(dataSources!= null){//cannot be
		    for(int i = 0; i< dataSources.length;i++){
			if(rString.equals(dataSources[i].getDescription())){
			    dataSourceSelected = i;
			}
		    }
		}
	    }
	    paintComponents(getGraphics());
	    repaint();
	}
    }
}
