package cdox.gui;

import cdox.*;
import cdox.gui.action.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.util.prefs.*;
import java.util.Vector;
import javax.swing.*;
/**
 * This class represents a dialog, in which the user is able to chooce which covers he wants to make.
 *The user is able to select front, back, and side on the same time, the booklet is a standalone.
 *The Dialog is an always present one, it is constructed in the CDoxFrame-Constructor, and is hidden,
 *or shown at will, with the showNewDialog() method
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 8th 2002
 * @see FileAction
 */
public class CoverDialog extends JDialog implements ActionListener{
    //The language selector, and its keys
    private Localizer lang = CDox.getLocalizer();
    private String[] theNames = {lang.get("cdfront"),
				 lang.get("cdbooklet"),
				 lang.get("cdside")};

    private String[] buttonText = {lang.get("okbutton"),
				   lang.get("cancelbutton"),
				   lang.get("wizardonbutton")};

    //the components to add to the dialog
    private JCheckBox checkBox;
    private JButton button;
    private Vector buttonGroup;
    private static final boolean DEBUG=false;
    private boolean hasFront=true,hasBack=true;
    
    //the  returned array
    private int[] choises = new int[theNames.length];

    //the ok-button flag
    private boolean okPressed = false;

    //the CDoxFrame
    private CDoxFrame cFrame;

    /**
     *The Constructor first creates a checkbox pane in which the checkboxes will be stored,
     *and after that a buttonpanel in which the OK-Cancel-Button are added.
     *@param owner the calling JFrame(CDoxFrame),
     *@param title the Title of the Dialog.
     */
    public CoverDialog(Frame owner, String title){
	super(owner, title, true);
	cFrame = (CDoxFrame)owner;
	setDefaultCloseOperation(HIDE_ON_CLOSE);

	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb = new GridBagConstraints();
	JPanel checkBoxPanel = new JPanel(new GridBagLayout());
	checkBoxPanel.setBorder
	    (BorderFactory.createTitledBorder
	     (BorderFactory.createEtchedBorder(),lang.get("boxtitel")));
	buttonGroup = new Vector();
	for (int i=0;i<theNames.length;i++){
	    if(i==0)
		checkBox = new JCheckBox(theNames[i],true);
	    else
		checkBox = new JCheckBox(theNames[i]);
	    checkBox.addActionListener(this);
	    buttonGroup.add(checkBox);
	    gb.insets = new Insets(2,2,2,2);
	    gb.anchor = gb.WEST;
	    gb.gridy = i;
	    checkBoxPanel.add(checkBox, gb);
	}
	gb.gridy = 0;
	getContentPane().add(checkBoxPanel, gb);
	JPanel buttonPanel = new JPanel(new GridBagLayout());
	gb.fill = gb.HORIZONTAL;
	gb.insets = new Insets(0,0,5,0);
	button = new JButton(buttonText[0]);
	button.setPreferredSize(new Dimension(button.getPreferredSize().width, 23));
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	getRootPane().setDefaultButton(button);
	buttonPanel.add(button, gb);

	gb.gridy = 1;
	gb.insets = new Insets(5,0,5,0);
	button = new JButton(buttonText[1]);
	button.setPreferredSize(new Dimension(button.getPreferredSize().width, 23));
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	new EscapeListener(button,getRootPane());
	buttonPanel.add(button, gb);

	gb.gridy = 2;
	gb.insets = new Insets(5,0,0,0);
	button = new JButton(buttonText[2]);
	button.setPreferredSize(new Dimension(button.getPreferredSize().width, 23));
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	new EscapeListener(button,getRootPane());
	buttonPanel.add(button, gb);

	gb.gridy= 0;
	gb.gridx= 1;
	gb.anchor = gb.EAST;
	getContentPane().add(buttonPanel, gb);

	setResizable(false);
	((CDoxFrame)owner).center(this);
    }
    /**
     * In this Method the actions are evaluated. It's ensured that one can only choose one
     * front side (normal or booklet).
     *If the OK-Button is pressed the current selection is stored in the choises-array.
     * @param e an <code>ActionEvent</code> value
     * @see #getChoices
     * @see #isOkPressed
     * @see #showNewDialog
     */
    public void actionPerformed(ActionEvent e){
	if(e.getSource() instanceof JCheckBox){
	    checkBox = (JCheckBox)e.getSource();
	    // check the selections
	    if(checkBox.getText().equals(theNames[0])&&checkBox.isSelected()){
		if(hasFront) ((JCheckBox)buttonGroup.get(1)).setSelected(false);
		else hasFront=true;
	    }
	    if(checkBox.getText().equals(theNames[1])&&checkBox.isSelected()){
		if(hasFront) ((JCheckBox)buttonGroup.get(0)).setSelected(false);
		else hasFront=true;
	    }
	    // if none is selected, select the actual.
	    if((!((JCheckBox)buttonGroup.get(0)).isSelected())&&
	       (!((JCheckBox)buttonGroup.get(1)).isSelected())&&
	       (!((JCheckBox)buttonGroup.get(2)).isSelected())) checkBox.setSelected(true);
	}
	else{
	    button = (JButton)e.getSource();
	    if(button.getText().equals(buttonText[0])){//OK-Button
		for(int i=0; i<buttonGroup.size();i++){
		    if(((JCheckBox)buttonGroup.get(i)).isSelected())
			choises[i] = i;
		}
		okPressed = true;
	    }
	    else if(button.getText().equals(buttonText[2])){//wizard-Button
		Preferences prefs=Preferences.userNodeForPackage(CDoxFrame.class);
		prefs.putBoolean("wizard", true);
		hide();
		cFrame.showNewWizard();
		return;	
	    }
	    hide();
	}
	 	
    }

    /**
     *This Method returns the users selected checkboxes in an int-array form.
     *@return the users choices.
     */
    public int[] getChoices(){
	return choises;
    }
    
    /**
     *This method should be called to see whether ok was pressed or not.
     *@return true if ok-button is pressed false otherwhise.
     */
    public boolean isOkPressed(){
	return okPressed;
    }

    /**
     *This method should be called to show the dialog, it resets the choises array.
     */
    public void showNewDialog(){
	for(int i=0;i<choises.length;i++)
	    choises[i]=-1;
	okPressed = false;
	this.show();
    }
    /**
     *Returns the String "CoverDialog".
     *@return the String "CoverDialog".
     */
    public String toString(){
	return "CoverDialog";
    }
}
	
