package cdox.gui;

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * This class arranges the file extensions in the JFileChooser defined in CDoxFrame.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class ExtensionFilter extends FileFilter{
    private String description;
    private String extension;
    private String alternative;
    /**
     *The Constructor accepts two String (extension and description) which will be stored.
     *The extension-String is converted to lowercase.
     *@param ext is the extension
     *@param desc is the file extension's description
     *@param alt is an alternative filter (for example *.jpg && *.jpeg)
     */
    public ExtensionFilter(String ext, String desc, String alt){
	extension = ext.toLowerCase();
	description = desc;
	alternative = alt;
    }

    /**
     * This Method returns true when the file is accepted by the extensionFilter.
     *This Method must be overwritten from the abstract class FileFilter.
     * @param file is the File to check.
     * @return true if the file fits the profile of this ExtensionFilter false otherwhise.
     */
    public boolean accept(File file){
	if(alternative == null)
	    return(file.isDirectory() || 
		   file.getName().toLowerCase().endsWith(extension));
	else
	    return(file.isDirectory() || 
		   file.getName().toLowerCase().endsWith(extension) ||
		   file.getName().toLowerCase().endsWith(alternative));
    }
    /**
     *This Method returns the extension's description.
     *@return this filefilter description
     */
    public String getDescription(){ return description; }

    /**
     *Returns the extension.
     *@return this fileFilter's extension.
     */
    public String getExtension(){
	return extension.toLowerCase();
    }

    /**
     *Returns the alternative extension.
     *@return this fileFilter's alternative extension.
     */
    public String getAlternativeExtension(){
	return alternative;
    }
    
    /**
     * Returns this class as the extensions it represents.
     * @return the extension
     */
    public String toString(){
	return extension;
    }

}
	
