package cdox.gui;

import cdox.*;
import cdox.edit.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

/**
 * Displays a high quality preview of the given Cover.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class PreviewWindow extends JDialog implements CDCoverStandards{

    private BufferedImage image;
    private JDialog itsme=this;
    private boolean gray=false;
    private Cover cover;

    /**
     * Constructs one.
     * @param frame a <code>JFrame</code> value
     * @param c the cover you want to display.
     */
    public PreviewWindow( JFrame frame, Cover c ){
	super(frame,CDox.getLocalizer().get("preview"),false);
	c.setQuality(true);
	Localizer lang=CDox.getLocalizer();
	image=c.getBufferedImage();
	c.setQuality(false);
	cover=c;
	JPanel panel=new JPanel(){
		public void paint( Graphics g ){
		    g.drawImage(image,0,0,itsme);
		}
	    };
	switch(c.getType()){
	case CDFRONT: panel.setPreferredSize
			  (new Dimension(CDFRONT_SIZE[0],CDFRONT_SIZE[1])); break;
	case CDBOOKLET: panel.setPreferredSize
			    (new Dimension(CDBOOKLET_SIZE[0],CDBOOKLET_SIZE[1])); break;
	case CDBACK_SIDE: panel.setPreferredSize
			      (new Dimension(CDBACK_SIDE_SIZE[0],CDBACK_SIDE_SIZE[1])); break;
	}
	getContentPane().add(panel);
	panel=new JPanel();
	panel.setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	gb.gridx=0;
	gb.gridy=0;
	gb.gridwidth=1;
	JButton button=new JButton(lang.get("close"));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    setVisible(false);
		    dispose();
		}
	    });
	new EscapeListener(button,getRootPane());
	getRootPane().setDefaultButton(button);
	panel.add(button,gb);
	gb.gridx=1;
	button=new JButton(lang.get("reload"));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    reload();
		}
	    });
	panel.add(button,gb);
	gb.gridx=2;
	button=new JButton(lang.get("previewbuttongray"));
	button.addActionListener(new GrayColor());		
	panel.add(button,gb);
	getContentPane().add(panel,BorderLayout.SOUTH);
	pack();
	setResizable(false);
	center();
    }

    private class GrayColor implements ActionListener{
	public void actionPerformed( ActionEvent evt ){
	    gray=!gray;
	    if(!gray)
		((JButton)evt.getSource()).setText(CDox.getLocalizer().get("previewbuttongray"));
	    else
		((JButton)evt.getSource()).setText(CDox.getLocalizer().get("previewbuttoncolor"));
	    reload();
	}
    }

    // reloads the previewed image
    private void reload(){
	cover.setGrayscale(gray);
	cover.setQuality(true);
	image=cover.getBufferedImage();
	cover.setQuality(false);
	cover.setGrayscale(false);
	repaint();
    }

    /**
     * Displays this dialog in the center of the screen and packs it.
     */
    public void center(){
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension fs = getSize();
	setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
    }
}
