package cdox.gui;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * This class represents a modal rotate dialog where the user can set the amount of degrees
 * to rotate.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 8th 2002
 */
public class RotateDialog extends JDialog implements ActionListener, AdjustmentListener, CaretListener{
    private JIntField angleField;
    private JScrollBar sBar;
    private JRadioButton linksButton;
    private JRadioButton rechtsButton;
    private int returnValue = 0, scrollCycle = 0;
    private boolean linksRum = true, isWriting = false, okPressed = false, mousePressed = false;
    private Localizer lang = CDox.getLocalizer();
    private Dimension buttonDim=new Dimension(80,23);

    /**
     *The Constructor takes care of most things. It makes four panels(button, option, and on the option
     *the radioPanel, and the inputPanel). The slider is a scrollbar, which is set with a JIntField.
     *It add some listeners with some components, action for the buttons, caret for the textfield, the 
     *scrollbar has two, adjustment and mouselistener.
     *@param owner the owner of this dialog, in this case CDoxFrame
     *@param title the title of this dialog.
     */
    public RotateDialog(Frame owner, String title){
	super(owner, title, true);

	//setLayoutmanagers etc...
	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gbRoot = new GridBagConstraints();
	GridBagConstraints gbPanel = new GridBagConstraints();
	
	//Construct the rotate panel
	JPanel optionPanel = new JPanel(new GridBagLayout());
	optionPanel.setBorder
	    (BorderFactory.createTitledBorder
	     (BorderFactory.createEtchedBorder(),lang.get("rotateborder")));

	//For the radio buttons
	ButtonGroup bGroup = new ButtonGroup();
	JPanel radioPanel = new JPanel();
	linksButton = new JRadioButton(lang.get("rotatelinksbutton"), true);
	linksButton.addActionListener(this);
	bGroup.add(linksButton);
	radioPanel.add(linksButton);
	rechtsButton = new JRadioButton(lang.get("rotaterechtsbutton"));
	rechtsButton.addActionListener(this);
	bGroup.add(rechtsButton);
	radioPanel.add(rechtsButton);
	optionPanel.add(radioPanel, gbPanel);

	//For the rotate input
	JPanel inputPanel = new JPanel();
	angleField = new JIntField(0, 0,360);
	angleField.setPreferredSize(new Dimension(30, angleField.getPreferredSize().height));	
	angleField.addCaretListener(this);
	inputPanel.add(angleField);

	//The scrollbar
	sBar = new JScrollBar(JScrollBar.VERTICAL,0,1,-1,361);
	JComponent[] sBarButtons = new JComponent[sBar.getComponentCount()];
	for(int i=0; i<sBar.getComponentCount();i++){
	    sBarButtons[i]=(JComponent)sBar.getComponent(i);
	    sBarButtons[i].addMouseListener(new ScrollBarMouseListener());
	}	    
	sBar.setMinimumSize(new Dimension(16, angleField.getPreferredSize().height));
	sBar.setPreferredSize(new Dimension(16, angleField.getPreferredSize().height));
	sBar.setMaximumSize(new Dimension(16, angleField.getPreferredSize().height));
	sBar.addAdjustmentListener(this);
	inputPanel.add(sBar);
	gbPanel.gridy = 1;
	optionPanel.add(inputPanel,gbPanel); 

	//Add option panel
	gbRoot.anchor = gbRoot.NORTH;	
	gbRoot.gridwidth = gbRoot.REMAINDER;
	gbRoot.fill=gbRoot.BOTH;
	getContentPane().add(optionPanel, gbRoot);
	gbRoot.fill=gbRoot.NONE;
	
	// The buttons
	GridBagConstraints gb=new GridBagConstraints();
	gb.gridy=1;
	gb.gridx=0;
	gb.weightx=1;


	//OK-button
	JButton button = new JButton(lang.get("okbutton"));
	button.setPreferredSize(buttonDim);
	getRootPane().setDefaultButton(button);
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	gb.insets = new Insets(2,2,2,2);
	getContentPane().add(button,gb);

	//Cancel-button
	gb.gridx=1;
	button = new JButton(lang.get("cancelbutton"));
	button.setPreferredSize(buttonDim);
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	new EscapeListener(button,getRootPane());
	getContentPane().add(button,gb);

	// Help button
	button=new JButton(lang.get("help"));
	button.setPreferredSize(buttonDim);
	button.setBorder(BorderFactory.createRaisedBevelBorder());
	button.addActionListener(this);
	gb.gridx=2;
	getContentPane().add(button,gb);

	//set dialog things
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	setResizable(false);
	((CDoxFrame)owner).center(this);
	angleField.requestFocus();
	angleField.selectAll();
	show();
    }
    
    /**
     *This method is registerd with the buttons, if ok is pressed, the current JInfieldValue
     *is saved in returnValue. If a JRadioButton is pressed it sets the linksRum boolean, true(left),
     *false(right).
     *@param e the ActionEvent send from the button.
     */
    public void actionPerformed(ActionEvent e){
	if(e.getSource() instanceof JButton){
	    if(((JButton)e.getSource()).getText().equals(lang.get("okbutton"))){
		okPressed = true;
		returnValue = angleField.getValue();
	    }
	    if(((JButton)e.getSource()).getText().equals(lang.get("help")))
		CDox.documentation.displayDocumentationFor("genericpopup",lang.getLanguageCode());
	    else dispose();
	}
	else if(e.getSource() instanceof JRadioButton){
	    if(((JRadioButton)e.getSource()).getText().equals(lang.get("rotatelinksbutton")))
		linksRum = true;
	    else
		linksRum = false;
	    angleField.requestFocus();
	}
    }

    /**
     *This method should be called to get the inputted rotateAngle.
     *@return the degrees to rotate the element.
     */
    public double isOkPressed(){
	if(okPressed){
	    if(linksRum){
		returnValue = 360-returnValue;
	    }
	    return (double)returnValue;
	}
	else 
	    return 0;
    }

    /**
    *This Method is registered with the scrollBar it is called when any changes on the 
    *scrollbar have taken place. If the mouse is pressed it starts counting the times,
    *this mehtod is called, and sets the increment value to 10 at the 20-th time.
    *@param ae the AdjusmentEvent sent by the scrollbar.
    */
    public void adjustmentValueChanged(AdjustmentEvent ae){
	if(mousePressed){
	    if(scrollCycle <=20)
		scrollCycle++;
	    if(scrollCycle == 20)
		sBar.setUnitIncrement(10);
	}
	if((sBar.getValue() == -1)){
	    sBar.setValue(359);
	}
	else if(sBar.getValue() == 360)
	    sBar.setValue(0);
	if(!isWriting){
	    angleField.setValue(sBar.getValue());
	    angleField.selectAll();
	}	
    }

    /**
     *This Method is called when the user manually types in the JIntField.
     *It sets the scrollbar to the typed value.
     *@param ce the CaretEven sent by the JIntField.
     */
    public void caretUpdate(CaretEvent ce){
	isWriting = true;
	if(angleField.getValue() != sBar.getValue())
	    sBar.setValue(angleField.getValue());
	isWriting = false;
    }

    /**
     *This inner class is registered with the <arrow> buttons of the scrollBar.
     *It sets the mousePressed boolean on true if pressed, which is a way to 
     *set the amount added to the JIntValue up to 10 instead of 1.
     */
    private class ScrollBarMouseListener extends MouseAdapter{
	public void mousePressed(MouseEvent me){
	    mousePressed = true;
	}
	public void mouseReleased(MouseEvent me){
	    scrollCycle = 0;
	    sBar.setUnitIncrement(1);
	    mousePressed = false;
	}
    }
}
