package cdox.gui;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import javax.swing.*;

/**
 * The statusbar for the CDoxFrame. It has three Labels, the left one telling the user
 * which element is currently selected, the middle one, tells the position of the mouse on
 * the cover, the right one tells in which mode the user is.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class StatusPane extends JPanel{

    private JLabel textLabel=new JLabel(CDox.getLocalizer().get("statuspanenoselectedelement"));
    private JLabel modeLabel=new JLabel(CDox.getLocalizer().get("statuspanetextoff"));
    private JLabel dataLabel=new JLabel(CDox.getLocalizer().get("statusnodata"));
    private JLabel positionLabel=new JLabel("x: 0/y: 0");

    /**
     * Constructs a new statusPane with an etchedBorder and a preferredHeight of 18
     * pixels.
     */
    public StatusPane(){
	setLayout(new GridBagLayout());
	setBorder(BorderFactory.createEtchedBorder());
	components();
	setPreferredSize(new Dimension(getPreferredSize().width,18));
    }

    // adds the components
    private void components(){
	// setting fonts
	textLabel.setFont(textLabel.getFont().deriveFont(10f));
	modeLabel.setFont(modeLabel.getFont().deriveFont(10f));
	positionLabel.setFont(positionLabel.getFont().deriveFont(10f));
	// adding them
	GridBagConstraints gb=new GridBagConstraints();
	gb.weightx=1;
	gb.anchor=gb.WEST;
	add(textLabel,gb);
	JPanel panel=new JPanel(new GridBagLayout());
	GridBagConstraints gb1=new GridBagConstraints();
 	gb1.fill=gb.BOTH;
	gb1.insets= new Insets(1,1,1,1);
	panel.add(positionLabel,gb1);
	panel.setBorder(BorderFactory.createLoweredBevelBorder());
	gb.gridx=1;
	gb.anchor=gb.EAST;
	gb.weightx=0;
	add(panel,gb);
	panel=new JPanel(new GridBagLayout());
	panel.add(modeLabel,gb1);
	panel.setBorder(BorderFactory.createLoweredBevelBorder());
	gb.gridx=2;
	add(panel,gb);
	panel=new JPanel(new GridBagLayout());
	panel.add(dataLabel,gb1);
	panel.setBorder(BorderFactory.createLoweredBevelBorder());
	gb.gridx=3;
	add(panel,gb);
    }

    /**
     * Sets the text on the left (the text label).
     *@param s the text.
     */
    public void setTextLabel( String s ){
	textLabel.setText(s);
    }

    /**
     * Sets the text on the mode panel (on the right side).
     *@param s the text.
     */
    public void setModeLabel( String s ){
	modeLabel.setText(s);
    }

    /**
     * Sets the text on the data panel (on the outmost right side).
     *@param s the text.
     */
    public void setDataLabel( String s ){
	dataLabel.setText(s);
    }

    /**
     * Sets the text on the position panel (on the right side).
     *@param p the position.
     */
    public void setPositionLabel( Point p ){
	if(p!=null) positionLabel.setText("x: "+p.x+"/y: "+p.y);
    }

}
