package cdox.gui;

import java.awt.FontMetrics;

/**
 * A class to do some calculating on a String used for scrolling on the AboutPane, so that
 * the String always is placed in the middle. This class also stores the position of the
 * String. A call to setHeight sets the current positiont to -1, reset, resets the
 * currentHeight to the original height.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 * @see AboutDialog
 */
public class StringPlacer{
    
    private int imageWidth, imageHeight, posToCenter, currentHeight, extraHeight;
    private String scrollString;

    /**
     *Construct a new StringPlacer Object with given parameters. It calculates
     *the place on the backgroundImage to draw the given String in the center.
     *@param na the String to do the calculating on,
     *@param fm the FontMetrics with which the text will be drawn, used to calculate the center of the String,
     *@param w the imageWidth,
     *@param h the imageHeight,
     *@param xL the start offset of the nextLines, 0 for first 13 for second, 26.... etc
     */
    public StringPlacer(String na, FontMetrics fm, int w, int h, int xL){
	scrollString = na;
	imageWidth = w;
	imageHeight = h;
	extraHeight = xL;
	currentHeight = h + xL;
	posToCenter = (imageWidth/2)-((fm.stringWidth(scrollString))/2);
    }

    /**
     *Returns the x position of the String to center it.
     *@return the x poxition of the String.
     */
    public int getCenterPosition(){
	return posToCenter;
    }

    /**
     *Resets all the standard settings.
     */
    public void reset(){
	currentHeight = imageHeight+extraHeight;
    }

    /**
     *returns the current position of the text on the backgroundimage.
     *@return the current position of the text.
     */
    public int getCurrentHeight(){
	return currentHeight;
    }

    /**
     *Sets the current y position -1. For the graphics object to draw this String on has
     *(0,0) in the upper left corner.
     */
    public void setCurrentHeight(){
	currentHeight --;
    }

    /**
     *Returns the String used in this StringPlacer.
     *@return the String to draw.
     */
    public String getString(){
	return scrollString;
    }
}
