package cdox.gui.action;

import cdox.gui.*;
import cdox.util.conf.*;
import javax.swing.*;

/**
 * This is the abstract superclass of all CDox actions. It implements the interface
 * Configurable to provide easy configuration.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 * @see Configurable
 */
public abstract class CDoxAction extends AbstractAction implements Configurable{

    /**
     * The menu Item which can be used to define the KeyStroke.
     */
    protected JMenuItem menuItem=null;
    /**
     * The CDoxFrame.
     */
    protected CDoxFrame cFrame=null;

    /**
     * Constructor.
     * @param f the CDoxFrame this action belongs to
     * @param name a name for this action
     * @param tooltip the possible tooltip on this Action, which can be null.
     */
    protected CDoxAction( CDoxFrame f, String name, String tooltip ){
	super(name);
	cFrame=f;
	if(tooltip != null) putValue(SHORT_DESCRIPTION, tooltip);
    }

    /**
     * Sets the menu item for this command. This is only used to change the key stroke
     * later on.
     * @param menuItem the menu item whose accelerator should be set when configuring this
     * action.
     */
    public void setMenuItem( JMenuItem menuItem ){
	this.menuItem=menuItem;
    }

    /**
     * Inherited from Configurable. Sets the short-cut key to the KeyStroke which is in
     * Object. If this Action has a MenuItem, the keyStroke is also set for this one.
     * @param o the KeyStroke-Object.
     */
    public void configure( Object o ){
	putValue(ACCELERATOR_KEY,o);
	// well, not THAT easy to get some simple keystroke work...
	cFrame.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).
	    put((KeyStroke)o,getValue(NAME));
	cFrame.getRootPane().getActionMap().put(getValue(NAME),this);
	if(menuItem!=null) menuItem.setAccelerator((KeyStroke)o);
    }

}
