package cdox.gui.action;

import cdox.edit.*;
import cdox.gui.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * A simple class to update the fontBox and fontSizeBox of the CDoxFrame if the user moves
 * the caret in a new textElement(that is in a TextPane).
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class CaretUpdater implements CaretListener{

    private CDoxFrame cFrame;
    private TextPane tPane;
    private String font = new JLabel().getFont().getFamily();
    private int fontSize = 0;
    
    /**
     *Constucts a new CaretUpdater with the specified TextPane.
     *@param cF the CDoxFrame
     *@param tP the typing TextPane
     *@see CDDrawingPane
     */
    public CaretUpdater(CDoxFrame cF, TextPane tP){
	cFrame = cF;
	tPane = tP;
    }

    /**
     *This method is inherited from the CaretListener interface. It is called each time a
     *caret-event has taken place. That is the cursor(caret) moved. It first gets the
     *fontsize and font-family of the current position and than checks if it is the same
     *as the last update, if not the fontBox and fontSizeBox of the CFrame are updated.
     *@param ce the caretEvent.
     */
    public void caretUpdate(CaretEvent ce){
	int tempSize = tPane.getFontSizeAtPostion(ce.getDot());
	String tempFont = tPane.getFontAtPosition(ce.getDot());
	if(tempSize != fontSize){
	    cFrame.getFontSizeBox().setSelectedItem(new Integer(tempSize));
	    fontSize = tempSize;
	}
	if(!tempFont.equals(font)){
	    cFrame.getFontBox().setSelectedIndex(cFrame.getFontIndex(tempFont));
	    font = new String(tempFont);
	}
    }
}
