package cdox.gui.action;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.gui.undo.*;
import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.zip.*;
import javax.imageio.*;
import javax.swing.*;

    
/**
 * This is an Class is called from CDoxFrame, which handles the EditActions.
 * EditActions are both in the toolBar and in the editMenu "Edit".
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class EditAction extends CDoxAction{
    
    private Localizer lang=CDox.getLocalizer();
    /** The current redo action is stored here. */
    public static DoAction redo=null;
    /** The current undo action is stored here. */
    public static DoAction undo=null;

    /**
     * The EditAction Constructor stores an Image(if available) and a ToolTip(if
     * available).
     *@param cF the CDoxFrame.
     *@param name A String representing the EditAction's name.
     *@param tooltip A String representing the EditAction's toolTip.
     */
    public EditAction(CDoxFrame cF, String name, String tooltip) {
	super(cF,CDox.getLocalizer().get(name),tooltip); 
	String iconFilename = cFrame.getImageDir()+name+".gif";
	if(getClass().getResource(iconFilename)!=null)
	    putValue(SMALL_ICON, new ImageIcon(getClass().getResource(iconFilename)));
    }
    /**
     *This Method handles all the EditActions. It if's the name of the EditAction.
     *@param e an ActionEvent generated by a pressed toolBar Button or a selected EditmenuItem.
     */
    public void actionPerformed(ActionEvent e){
	cFrame.setWorking(true);
	String name = (String)getValue(NAME);
	Clipboard clip=Toolkit.getDefaultToolkit().getSystemClipboard();
	if(name.equals(lang.get("cut"))){
	    Cover cover=cFrame.getEditPane().getDrawingPane().getCover();
	    Element selected=cover.getSelectedElement();
	    if(selected==null) {
		cFrame.setWorking(false);
		return;
	    }
	    cover.removeElement(selected);
	    undo=new RemoveAction(selected,cover);
	    clip.setContents(selected,(ClipboardOwner)cFrame);
	    cFrame.setActionEnabled("paste", true);
	    cFrame.setActionEnabled("undo",true);
	    cFrame.setCutNWasteable(false);
	    cFrame.setSave(true);
	    cFrame.setSelectedElement(null);
	    cFrame.repaint();
	}
	else if(name.equals(lang.get("copy"))){
	    Cover cover=cFrame.getEditPane().getDrawingPane().getCover();
	    Element selected=cover.getSelectedElement();
	    if(selected==null) {
		cFrame.setWorking(false);
		return;
	    }
	    clip.setContents(selected,(ClipboardOwner)cFrame);
	    cFrame.setActionEnabled("paste", true);
	}
	else if(name.equals(lang.get("paste"))){
	    Cover cover=cFrame.getEditPane().getDrawingPane().getCover();
	    Transferable t=clip.getContents(cFrame);
	    try{ // if it is an image: create a new temp file and store it within it
		BufferedImage i=(BufferedImage)t.getTransferData(DataFlavor.imageFlavor);
		File f=File.createTempFile("cdox.",null);
		f.deleteOnExit();
		String s=f.toString();
		TempFiles.add(s,f);
		ImageIO.write(i,"png",f);
		cFrame.getEditPane().getDrawingPane().addImage(s);
		cFrame.setSave(true);
	    }catch(UnsupportedFlavorException ufe){
		try{ // maybe it's a text
		    TextElement elem=(TextElement)t.getTransferData
			(TextElement.TEXTELEMENT_FLAVOR);
		    cFrame.getEditPane().getDrawingPane().getCover().addElement(elem);
		    cFrame.setSelectedElement(elem);
		    cFrame.setActionEnabled("undo",true);
		}catch(UnsupportedFlavorException ufe2){
		    try{ // so it must be a generic
			Component c=(Component)t.getTransferData
			    (GenericElement.COMPONENT_FLAVOR);
			// has also to be done
		    }catch(UnsupportedFlavorException ufe3){
			// in this case, we don't even support it...
		    }catch(IOException ioe3){
		    }
		}catch(IOException ioe2){
		}
	    }catch(IOException ioe){
	    }
	}
	else if(name.equals(lang.get("options"))){
	    cFrame.configure(true); // shows configuration dialog
	}
	else if(name.equals(lang.get("undo"))){
	    if(undo!=null){
		undo.undo();
		redo=undo;
		cFrame.setSelectedElement(null);
		cFrame.setActionEnabled("undo",false);
		cFrame.setActionEnabled("redo",true);
		cFrame.repaint();
	    }
	}else if(name.equals(lang.get("redo"))){
	    if(redo!=null){
		redo.redo();
		cFrame.setActionEnabled("redo",false);
		cFrame.setActionEnabled("undo",true);
		cFrame.repaint();
	    }
	}
	else if(name.equals(lang.get("rotate"))){
	    RotateDialog rDiag =new RotateDialog(cFrame, lang.get("rotatetitle"));
	    cFrame.rotateElement(rDiag.isOkPressed());
	}
	else if(name.equals(lang.get("fliphorizontal"))){
	    cFrame.flipElementHorizontal();
	}
	else if(name.equals(lang.get("flipvertical"))){
	    cFrame.flipElementVertical();
	}
	else if(name.equals(lang.get("reedittext"))){
	    cFrame.reEditTextElement();
	}
	else if(name.equals(lang.get("movetoback"))){
	    if(cFrame.getEditPane().getDrawingPane().getSelectedElement() != null)
	    cFrame.getEditPane().getDrawingPane().getCover()
		.moveToBack(cFrame.getEditPane().getDrawingPane().getSelectedElement());
	}
	cFrame.setWorking(false);
    }
}
