package cdox.gui.action;

import cdox.*;
import cdox.data.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.gui.wizard.*;
import cdox.print.*;
import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.prefs.*;
import javax.swing.*;

/**
 * This Class handles the FileActions.
 * FileActions are both in the toolBar and in the FileMenu "File".
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version July 17th 2002
 */
public class FileAction extends CDoxAction implements CDCoverStandards{

    private Localizer lang=CDox.getLocalizer();

    /**
     * The FileAction Constructor stores an Image(if available) and a ToolTip(if available).
     * @param cF the CDoxFrame calling this ActionClass
     * @param name A String representing the FileAction's name
     * @param tooltip A String representing the FileAction's toolTip
     */
    public FileAction(CDoxFrame cF, String name, String tooltip ){ 
	super(cF,CDox.getLocalizer().get(name),tooltip);
	String iconFilename = cFrame.getImageDir()+name+".gif";
	if(getClass().getResource(iconFilename)!=null)
	    putValue(SMALL_ICON, new ImageIcon(getClass().getResource(iconFilename)));
    }

    /**
     *This Method handles all the FileActions. It if's the name of the FileAction.
     *@param e an ActionEvent generated by a pressed toolBar Button or a selected FilemenuItem.
     */
    public void actionPerformed(ActionEvent e){
	cFrame.setWorking(true);//set the working icon to work
	String name = (String)getValue(NAME);
	//-----------------ITS A NEW ACTION-----------------
	if(name.equals(lang.get("new"))){
	    cFrame.checkCloseable();
	    //Show the newDialog
	    Preferences prefs=Preferences.userNodeForPackage(CDoxFrame.class);
	    if(prefs.getBoolean("wizard", true)){
		new CdoxWizard(cFrame, "Wizard");
	    }
	    else{
		cFrame.getNewDialog().showNewDialog();
		if(cFrame.getNewDialog().isOkPressed()){		//if the OK-Button is pressed
		    if(cFrame.getEditPane().hasComponents()){//check to see if there were Components in the EditPane
			//TODO A SAFE ACTION
			cFrame.getEditPane().removeCoverTemplates();//remove them
		    }
		    if(cFrame.getTextEnabled()){
			cFrame.updateTextBar(null);
		    }
		    cFrame.resetSaveFile();
		    int[] whichCovers = cFrame.getNewDialog().getChoices();
		    cFrame.addCovers(whichCovers);
		    cFrame.setData(null);
		}
	    }
// 	    if(!(cFrame.getExtendedState()==cFrame.MAXIMIZED_BOTH))//if maximised do not center
// 		cFrame.center(cFrame);
	    cFrame.paintComponents(cFrame.getGraphics());
	    cFrame.setActionEnabled("redo",false);
	    cFrame.setActionEnabled("undo",false);
	    EditAction.redo=null;
	    EditAction.undo=null;
	    cFrame.repaint();
	}

	//-----------------ITS AN OPEN ACTION----------------
	else if(name.equals(lang.get("open"))){
	    cFrame.checkCloseable();
	    int[] filters = {CDX};
	    File file = cFrame.getFileDialog().showDialog
		(cFrame, lang.get("filetitlecdoxopen"),lang.get("fileapproveopenbutton"), null, filters);
	    if(file != null){
		Cover[] c=null;
		if(file.exists()) c=cFrame.readDocuments(file);
		if(c==null) {
		    cFrame.setWorking(false);//reset the working icon
		    return;
		}
		if(c.length==0){
		    CDoxFrame.handleError(new Throwable(lang.get("opencdxwrongformat")),false);
		    cFrame.setWorking(false);//reset the working icon
		    return;
		}
		cFrame.saveFile=file;
		if(cFrame.getEditPane().hasComponents()){
		    //TODO A SAVE ACTION
		    cFrame.getEditPane().removeCoverTemplates();//remove them
		}
		if(cFrame.getTextEnabled()){
		    cFrame.updateTextBar(null);
		}
		cFrame.addCovers(c,file.getName());
		if(!(cFrame.getExtendedState()==cFrame.MAXIMIZED_BOTH))//if maximised do not center
		    cFrame.center(cFrame);
		cFrame.paintComponents(cFrame.getGraphics());
		cFrame.setActionEnabled("redo",false);
		cFrame.setActionEnabled("undo",false);
		EditAction.redo=null;
		EditAction.undo=null;
	    }
	}
	//-----------------ITS A SAVE ACTION-----------------	   
	else if(name.equals(lang.get("save"))){
	    cFrame.checkSaveable();
	}

	//-----------------ITS A SAVEAS ACTION-----------------
	else if(name.equals(lang.get("saveas"))){
	    int[] filters = {CDX};
	    File f=cFrame.getFileDialog().showDialog(cFrame, lang.get("filetitlecdoxsave"), 
				       lang.get("fileapprovesavebutton"), null, filters);
	    if(f==null) {
		cFrame.setWorking(false);//set te working icon to pause
		return;
	    }
	    cFrame.saveAs(f);
	}
	//-----------------ITS An export ACTION-----------------
	else if(name.equals(lang.get("export"))){
	    int[] filters = {JPG, PNG};
	    File file = cFrame.getFileDialog().showDialog(cFrame, lang.get("filetitleexport"),
							  lang.get("fileapprovesavebutton"),
							  null , filters);
	    if(file != null){
		Cover cov = cFrame.getEditPane().getDrawingPane().getCover();
		cov.exportMySelf(file);
	    }
	}

	//------------------ITS A PRINT ACTION-----------------
	else if(name.equals(lang.get("print"))){
	    new PrintCover(cFrame, cFrame.getEditPane().getCovers()).start();
	    return;
	}
	//------------------ PROPERTIES ACTION------------------
	else if(name.equals(lang.get("properties"))){
	    ChooseDataDialog cd=new ChooseDataDialog(cFrame, lang);
	    cd.show();
	}
	//-----------------ITS AN EXIT ACTION-----------------
	else if(name.equals(lang.get("exit"))){
	    cFrame.stop();
	}else if(name.equals(lang.get("preview"))){
	    new PreviewWindow(cFrame,cFrame.getEditPane().getDrawingPane().getCover()).show();
	}
	cFrame.setWorking(false);//set te working icon to pause
    }
}
