package cdox.gui.action;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.gui.undo.*;
import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;

/**
 * This class handles the InsertActions.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version August 21st 2002
 */
public class InsertAction extends CDoxAction implements CDCoverStandards{

    private Localizer lang=CDox.getLocalizer();

    /**
     * The InsertAction Constructor stores an Image(if available) and a ToolTip(if available).
     *@param cF the CDoxFrame.
     *@param name A String representing the EditAction's name.
     *@param tooltip A String representing the EditAction's toolTip.
     */
    public InsertAction(CDoxFrame cF,String name, String tooltip) {
	super(cF,CDox.getLocalizer().get(name),tooltip); 
	String iconFilename = cFrame.getImageDir()+name+".gif";
	if(getClass().getResource(iconFilename)!=null)
	    putValue(SMALL_ICON, new ImageIcon(getClass().getResource(iconFilename)));
    }

    /**
     *This Method handles all the InsertActions. It if's the name of the InsertActions.
     *@param e an ActionEvent generated by a pressed toolBar Button or a selected EditmenuItem.
     */
    public void actionPerformed(ActionEvent e){
	cFrame.setWorking(true);//sets the icon working...
	boolean textEnabled = cFrame.getTextEnabled();
	//let everybody know an insertButton ispressed.
	if(textEnabled){
	    if(cFrame.getEditPane().getDrawingPane().getTextPane() != null){//wenn schon ein textpane voranden ist
		cFrame.updateTextBar(null); 
		cFrame.setActionBorder("text");
	    }
	}
	String name = (String)getValue(NAME);

	//-------------------EIN BILD-----------------------------
	if(name.equals(lang.get("bild"))){
	    int[] filters = {JPG, GIF, PNG, ALL};
	    File file = cFrame.getFileDialog().showDialog(cFrame, lang.get("filetitlebild"),
							  lang.get("fileapproveopenbutton"), 
							  null,
							  filters);
	    if(file != null){
		try{
		    cFrame.setImage(TempFiles.add(file));
		    cFrame.setSave(true);
		}catch(IOException ioe){
		    CDoxFrame.handleError
			(new Throwable(lang.get("loadimageio",new String[]{ioe.getLocalizedMessage()}), ioe),
			 false);
		}
	    }
	}
	    
	//-------------------EIN HINTERGRUND BILD-----------------------------
	else if(name.equals(lang.get("backgroundbild"))){
	    int option = -1;
	    if(cFrame.hasBackground()){//wenn schon ein hintergrundbild voranden ist
		option = CDoxFrame.showQuestion(lang.get("overwritebackgroundquest"), JOptionPane.YES_NO_OPTION);
	    }
	    if(option == JOptionPane.YES_OPTION || option == -1){//wenn es berschrieben oder neu gemacht werden mu
		int[] filters = {JPG, GIF, PNG, ALL};
		File file = cFrame.getFileDialog().showDialog(cFrame, lang.get("filetitlebackground"), 
							      lang.get("fileapproveopenbutton"), 
							      null,
							      filters);
		if(file!=null){
		    try{
			if(file.exists()){
			    boolean mode=true;
			    String s=TempFiles.add(file);
			    if(cFrame.getEditPane().getDrawingPane().getCover().
			       getType()==Cover.CDBACK_SIDE)
				// leaving the sides free for back sides?
				mode=(CDoxFrame.showQuestion(lang.get("freeside"), JOptionPane.YES_NO_OPTION)==
				      JOptionPane.NO_OPTION);
			    if(cFrame.getEditPane().getDrawingPane().getCover().
			       getType()==CDFRONT) {
				BufferedImage bi = TempFiles.getImage(s);
				float percentage = (float)bi.getWidth()/(float)bi.getHeight();
				Cover[] cover = new Cover[1];
				if(percentage <2.3f && percentage > 1.8f){
				    int quest = CDoxFrame.showQuestion(lang.get("bookletquest"), JOptionPane.YES_NO_OPTION);
				    if(quest == JOptionPane.YES_OPTION){
					Cover c = new Cover(CDBOOKLET);
					cFrame.getEditPane().replaceCover
					    (cFrame.getEditPane().getDrawingPane().getCover(), c);
					if(!(cFrame.getExtendedState()==cFrame.MAXIMIZED_BOTH))//if maximised do not center
					    cFrame.center(cFrame);
					cFrame.paintComponents(cFrame.getGraphics());
					cFrame.setActionEnabled("redo",false);
					cFrame.setActionEnabled("undo",false);
					EditAction.redo=null;
					EditAction.undo=null;
					cFrame.setData(null);
				    }				    
				} 
				cFrame.setBackgroundImage( bi, s, mode);
				cFrame.setSave(true);
				bi=null;
				System.gc();
			    }		
			    else if((cFrame.getEditPane().getDrawingPane().getCover().
			       getType()==CDBOOKLET)){
				BufferedImage bi = TempFiles.getImage(s);
				float percentage = (float)bi.getWidth()/(float)bi.getHeight();
				Cover[] cover = new Cover[1];
				if(percentage >0.9f && percentage < 1.1f){
				    int quest = CDoxFrame.showQuestion(lang.get("frontquest"), JOptionPane.YES_NO_OPTION);
				    if(quest == JOptionPane.YES_OPTION){
					Cover c = new Cover(CDFRONT);
					cFrame.getEditPane().replaceCover
					    (cFrame.getEditPane().getDrawingPane().getCover(), c);
					if(!(cFrame.getExtendedState()==cFrame.MAXIMIZED_BOTH))//if maximised do not center
					    cFrame.center(cFrame);
					cFrame.paintComponents(cFrame.getGraphics());
					cFrame.setData(null);
				    }				    
				} 
				cFrame.setBackgroundImage( bi, s, mode);
				cFrame.setSave(true);
				bi=null;
				System.gc();
			    }
			    else{
				cFrame.setBackgroundImage(s,mode);
				cFrame.setSave(true);
			    }
			}
		    }catch(IOException ioe){
			CDoxFrame.handleError
			    (new Throwable(lang.get("loadimageio",new String[]{ioe.getLocalizedMessage()}), ioe),
			     false);
		    }
		}
		
	    }
	}
	else if(name.equals(lang.get("bcolor"))){
	    final CDDrawingPane cdPane=cFrame.getEditPane().getDrawingPane();
	    if(cdPane.getBackgroundColor()!=null)
		cFrame.getColorChooser().setColor(cdPane.getBackgroundColor());
	    else 
		cFrame.getColorChooser().setColor(Color.BLACK);
	    
	    //ask the user which color to use as background
	    JColorChooser.createDialog
		(cFrame, lang.get("ccolor"),true, cFrame.getColorChooser(),
		 new ActionListener(){ // ok
		     public void actionPerformed( ActionEvent evt ){
			 cFrame.setBackgroundColor(cFrame.getColorChooser().getColor());
			 cFrame.setSave(true);
			 cFrame.repaint();
		     }
		 },null).show();
	}
	else if(name.equals(lang.get("remove"))){
	    Cover cover=cFrame.getEditPane().getDrawingPane().getCover();
	    Element selected=cover.getSelectedElement();
	    if(selected==null) {
		cFrame.setActionEnabled("undo", true);
		cFrame.setUndoAction(new RemoveBackgroundAction(cFrame, cover, cover.getKey()));
		cover.deleteBackgroundImage();
	    }else{
		cover.removeElement(selected);
		cFrame.setActionEnabled("undo",true);
		cFrame.setUndoAction(new RemoveAction(selected,cover));
		cFrame.setCutNWasteable(false);
	    }
	    cFrame.setSave(true);
	    cFrame.setSelectedElement(null);
	    cFrame.repaint();
	}else if(name.equals(lang.get("removeall"))){
	    if(cFrame.showQuestion(lang.get("removeallquest"),JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION){
		cFrame.removeAllElements();
	    }
	    
	}
	cFrame.setWorking(false);//sets the icon not working...
    }
}
