package cdox.gui.action;

import cdox.*;
import cdox.edit.*;
import cdox.gui.*;
import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Handles everything that goes with text.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 12th 2002
 */
public class TextAction extends CDoxAction{

    private Localizer lang=CDox.getLocalizer();

    /**
     *The TextAction Constructor stores an Image(if available) and a ToolTip(if available).
     *@param cF the CDoxFrame.
     *@param name A String representing the TextAction's name.
     *@param tooltip A String representing the TextAction's toolTip.
     */
    public TextAction(CDoxFrame cF,
		      String name, String tooltip) {
	super(cF,name,tooltip); 
    }

    /**
     * This method is inherited from Action, it is called when a TextAction has happened
     * in the CDoxFrame. This can be the pressing of a Style button, selecting a new Font
     * or fontSize, or going in or out of textMode. This method checks which Action called
     * this method by checking their name.
     * @param ae The ActionEvent causing this method to be called.
     */
    public void actionPerformed(ActionEvent ae){
	String name = (String)getValue(NAME);
	if(name.equals("fontBox")){
	    if(cFrame.getTextEnabled()){
		cFrame.setActualFont(((Font)cFrame.getFontBox().getSelectedItem())
				     .deriveFont(cFrame.getActualFontSize()));
		Font actualFont = cFrame.getActualFont();
		cFrame.getFontBox().setFont(actualFont.deriveFont(14f));
	    }
	}
	else if(name.equals("sizeBox")){
	    cFrame.setActualFontSize(((Integer)cFrame.getFontSizeBox().getSelectedItem()).intValue());//set the font size
	}
	else if(name.equals("underline")){
	    cFrame.setActionBorder("underline");
	    cFrame.setTextUnderline((cFrame.getTextUnderline())? false : true);
	}
	else if(name.equals("italic")){
	    cFrame.setActionBorder("italic");
	    cFrame.setTextItalic((cFrame.getTextItalic())? false : true);
	}
	else if(name.equals("bold")){
	    cFrame.setActionBorder("bold");
	    cFrame.setTextBold((cFrame.getTextBold())? false : true);
	}
	else if(name.equals(lang.get("color"))){
	    cFrame.getColorChooser().setColor(cFrame.getTextColor());
	    JColorChooser.createDialog
		(cFrame,lang.get("ccolor"),true,cFrame.getColorChooser(),
		 new ActionListener(){ // ok
		     public void actionPerformed( ActionEvent evt ){
			 cFrame.setTextColor(cFrame.getColorChooser().getColor());
		     }
		 },null).show();
	}
	else if((name.equals(lang.get("texton")))||(name.equals(lang.get("textoff")))){
	    if(ae.getID() == 18){//mein eigenes event nur bei neuer cover
		if(cFrame.getTextBold()){//hat button nummer 6
		    cFrame.setActionBorder("bold");
		}
		if(cFrame.getTextItalic()){//hat button nummer 7
		    cFrame.setActionBorder("italic");
		}
		if(cFrame.getTextUnderline()){//hat button nummer 8
		    cFrame.setActionBorder("underline");
		}
		cFrame.setTextStylesOff();
	    }
	    else cFrame.setActionBorder("text");
	
	    boolean text = false;
	    if(cFrame.getTextEnabled()){//text mode is on must go out
		int option = -1;
		//wenn schon ein textpane voranden ist
		if(cFrame.getEditPane().getDrawingPane().getTextPane() != null){
		    if(!cFrame.getEditPane().getDrawingPane().getTextPane().getText().equals("")){
			TextElement te = new TextElement
			    (cFrame.getEditPane().getDrawingPane().getCurrentClickPoint(),
			     cFrame.getEditPane().getDrawingPane().getTextPane());
			cFrame.getEditPane().getDrawingPane().getCover().addElement(te);
			cFrame.getEditPane().getDrawingPane().setElementSelected(te);
			cFrame.setSave(true);
		    }
		}
		if(option == JOptionPane.YES_OPTION || option == -1){//wenn es gelscht werden mu
		    text=false;	
		    cFrame.setTextEnabled(false);
		    if(ae.getActionCommand().equals("cover")){
			cFrame.setActionBorder("text");
		    }
		    if(ae.getID() !=18)
			cFrame.updateTextBar(null); 
		}
		else{
		    if(!ae.getActionCommand().equals("cover"))
			cFrame.setActionBorder("text"); // in this case, we already called
			// this method (and must call it
			// again, because the state should not
			// change).
		    text = true;
		}
	    }
	    else{ //is out must go on
		text=true;
	    }
	    cFrame.setTextEnabled(text);
	    cFrame.getFontBox().setEnabled(text);
	    cFrame.getFontSizeBox().setEnabled(text);
	    cFrame.setActionEnabled("italic",text);
	    cFrame.setActionEnabled("underline",text);
	    cFrame.setActionEnabled("bold",text);
	    cFrame.setActionEnabled("color",text);

	}
	if((cFrame.getTextEnabled()) && 
	   (cFrame.getEditPane().getDrawingPane().getTextPane() != null))
	    cFrame.getEditPane().getDrawingPane().getTextPane().requestFocus();
    }
}
