package cdox.gui.undo;

import cdox.edit.*;
import cdox.gui.*;
import cdox.util.*;

/**
 * This class represents the undo/redo action for an backgroundImage-correction. This can
 * be blur sharpen, alphacorrection, contrast or brightness. It saves the keys to the files
 * where the backgroundImages are Stored. If undo is called the old image is loaded in the
 * Cover and if redo is called the filtered image is loaded into the Cover.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 5th 2002
 * @see Cover#setFilteredBackgroundImage
 */
public class BackgroundEditAction implements DoAction{
    
    private String oldKey, newKey;
    private CDoxFrame cFrame;
    private Cover cover;

    /**
     * Constructs a new backgroundEditAction which stores the keys to the temporary files
     * of the filtered and un-filtered (original) image.
     * @param cf The CDoxFrame
     * @param c the cover which contains the backgroundImage
     * @param oKey the oldKey to the originalImage
     * @param nKey the newKey to the temp-filtered-Image.
     */
    public BackgroundEditAction(CDoxFrame cf, Cover c, String oKey, String nKey){
	cFrame = cf;
	oldKey = oKey;
	newKey = nKey;
	cover = c;
    }

    public void undo(){
	cover.resetBackgroundImage(oldKey);
    }

    public void redo(){
	cover.resetBackgroundImage(newKey);
    }

    public void invalidate(){
	oldKey = null;
	newKey = null;
	cover = null;
    }
}
