package cdox.gui.undo;

/**
 * <p>
 * This interface represents an action that a user can do in our editor. One can invoke the
 * redo/undo methods to redo or undo the action that is represented by the appropriate
 * instance. Please see the various subclasses.
 *</p>
 *<p>
 * Please note that some actions can be an undo and a redo action at the same time, while
 * others can not. It is the task of the subclasses to make sure that no surprises
 * happen.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public interface DoAction{

    /**
     * This method redoes the action.
     */
    public void redo();

    /**
     * This method undoes the action.
     */
    public void undo();

    /**
     * This method invalidates the action, which typically means that the undo/redo
     * methods do nothing.
     */
    public void invalidate();

}
