package cdox.gui.undo;

import cdox.edit.*;
import cdox.gui.*;

/**
 * This class represents the undo/redo action for an Image-correction. This can be blur,
 * sharpen, alphacorrection, contrast or brightness. 
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class FotoEditAction implements DoAction{
    
    private ImageElement theElement;
    private String oldKey, newKey;
    private CDoxFrame cFrame;
    
    /**
     * Creates a new FotoEditAction which stores the key (id) to the oldImage and the key
     * to the newImage. At undo the oldKey is set to the ImageElement, redo sets the new
     * key.
     * @param cf the CDoxFrame
     * @param el the ImageElement which has been corrected,
     * @param oKey the oldKey which represents a unique id to an (temp)Image on disk
     * @param nKey the newKey which represents a unique id to an (temp)Image on disk
     */
    public FotoEditAction(CDoxFrame cf,ImageElement el, String oKey, String nKey){
	cFrame = cf;
	theElement = el;
	oldKey = oKey;
	newKey = nKey;
    }

    /**
     * Resets the oldKey to the ImageElement.
     */
    public void undo(){
	theElement.resetOldImage(oldKey);
    }

    /**
     * Resets the newKey to the ImageElement.
     */
    public void redo(){
	theElement.resetOldImage(newKey);
    }

    public void invalidate(){
	theElement = null;
	oldKey = null;
	newKey = null;
    }
}
	
	
