package cdox.gui.undo;

import cdox.edit.*;
import java.awt.*;

/**
 * This is an action that represents a simple position change. It stores the source and
 * destination points of the move and moves the element to the source position if undone
 * and to the destination position if redone.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class MoveAction implements DoAction{

    private Element element;
    private Point src,dest;

    /**
     * Makes a new MoveAction with the specified move.
     *@param e the element that has been moved.
     *@param src the original position of the element.
     *@param dest the target position of the element.
     */
    public MoveAction( Element e, Point src, Point dest ){
	element=e;
	this.src=src;
	this.dest=dest;
	element.registerDoAction(this);
    }

    /**
     * Sets the position to the source point.
     */
    public void undo(){
	if(element!=null) element.setPosition(src);
    }

    /**
     * Sets the position to the destination point.
     */
    public void redo(){
	if(element!=null) element.setPosition(dest);
    }

    public void invalidate(){
	element.unregisterDoAction(this);
	element=null;
    }

}
