package cdox.gui.undo;

import cdox.edit.*;
import java.awt.*;

/**
 * This class represents a remove action. Redo will remove the element again, undo will
 * insert it.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class RemoveAction implements DoAction{

    private Element element;
    private Cover cover;

    /**
     * Creates a new one.
     *@param e the element that is removed.
     *@param c the cover from which the element is removed.
     */
    public RemoveAction( Element e, Cover c ){
	element=e;
	cover=c;
    }

    /**
     * Inserts the element.
     */
    public void undo(){
	if(cover!=null) cover.addElement(element);
    }

    /**
     * Removes the element.
     */
    public void redo(){
	if(cover!=null) cover.removeElement(element);
    }

    public void invalidate(){
	cover=null;
	element=null;
    }

}
