package cdox.gui.undo;

import cdox.edit.*;
import cdox.gui.*;

/**
 * This class represents a remove action. Redo will remove the backgroundImage again, undo will
 * insert it.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class RemoveBackgroundAction implements DoAction{

    private CDoxFrame cFrame;
    private Cover cover;
    private String oldKey;

    /**
     * Creates a new one.
     * @param cF the CDoxFrame
     * @param c the cover from which the backgroundImage is removed
     * @param oKey The unique String representation to the TempFile on disk.
     */
    public RemoveBackgroundAction( CDoxFrame cF, Cover c, String oKey ){
	cFrame = cF;
	cover=c;
	oldKey = oKey;
    }

    /**
     * Inserts the backgroundimage
     */
    public void undo(){
	cover.resetBackgroundImage(oldKey);
    }

    /**
     * Removes the backgroundimage
     */
    public void redo(){
	cover.resetBackgroundImage(null);
    }

    public void invalidate(){
	cover=null;
	oldKey=null;
    }

}
