package cdox.gui.undo;

import cdox.edit.*;
import java.awt.*;

/**
 * This class represents a resizing action. Redo will set the size to the new size, undo
 * will reset it to the old size.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 16th 2002
 */
public class ResizeAction implements DoAction{

    private Element element;
    private Dimension src,dest;

    /**
     * Creates a new one.
     *@param e the element that is resized.
     *@param src the old size of the element.
     *@param dest the new size of the element.
     */
    public ResizeAction( Element e, Dimension src, Dimension dest ){
	element=e;
	this.src=src;
	this.dest=dest;
	element.registerDoAction(this);
    }

    /**
     * Sets the size to the destination size.
     */
    public void redo(){
	element.setSize(dest);
    }

    /**
     * Sets the size to the old size.
     */
    public void undo(){
	element.setSize(src);
    }

    public void invalidate(){
	element.unregisterDoAction(this);
	element=null;
    }

}
