package cdox.gui.wizard;

import javax.swing.*;
import java.awt.*;
import cdox.gui.*;
import cdox.*;
import cdox.util.*;
import cdox.data.*;
import cdox.edit.*;
import java.util.prefs.*;
import java.awt.event.*;
import javax.swing.border.*;


/**
 * <P>This class represents the CDox wizard. The wizard lets the user easely create his/her
 * own covers, without much effort. The wizard actually exists of three different questions
 * which the user can awnser to, and a finish screen with a preview of the created
 * cover(s). The three questions are, Which kind of project?, Do you have an image to use?,
 * Do you want to insert your own tracks?. After this the user sees his/her project in a
 * preview, and is able to make changes by clicking the back button, or if ready the ready
 * Button. </P>
 * <P>The questions are asked (from the cd-ox) in an embedded pane, the QuestionPanel at
 * the top.</P>
 * <P>The Previews are made in WizardPreview, which are added to the mainpanel.</P>
 *
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 10th 2002
 * @see QuestionPanel
 * @see WizardPreview
 */

public class CdoxWizard extends JDialog implements ActionListener, ItemListener, CDCoverStandards{
    private CDoxFrame cFrame;
    private Localizer lang = CDox.getLocalizer();
    private Graphics2D g;

    //the panels
    private JPanel upperPanel = new JPanel(new GridBagLayout());
    private QuestionPanel qPanel = null;
    private JPanel mainPanel = new JPanel(new GridBagLayout());
    private JPanel buttonPanel = new JPanel(new GridBagLayout());

    //the constraints used in the panels different 
    private GridBagConstraints gb = new GridBagConstraints();
    
    //the dialog count represents the screen number->0=whichcover, 1=image, 2=insert by
    //hand and 3=finish
    private int dialogCount = 0;

    //standard button size for uniform button look and feel.
    private Dimension buttonSize=new Dimension(100,23);

    //The buttons
    private JButton[] buttons = {new JButton(lang.get("prevbutton")),
				new JButton(lang.get("nextbutton")),
				new JButton(lang.get("cancelbutton")),
				new JButton(lang.get("wizardoffbutton"))};
    
    //the which cover to use radio Buttons-> see make radio buttons
    private JRadioButton[] radioButtons = new JRadioButton[5];

    //the questions
    private String[] question = {lang.get("wizardtodo"),
				 lang.get("wizardhaveimage"),
				 lang.get("wizardhaveimages"),
				 lang.get("wizardaddtracks"),
				 lang.get("wizardsayready"),
				 lang.get("wizardsayreadys")};

    //the button group for various button groups
    private ButtonGroup bg = new ButtonGroup();

    //the resulting covers
    private Cover[] covers = new Cover[1];

    //for the insert by hand question, the little "semi"-previews when text will or will
    //not be inserted
    private JPanel imagePanel = new JPanel(new GridBagLayout());

    //For filling with own data
    private Data chosenData=null;   
    private boolean insertByHand = false;

    
    /**
     * Constructs a new CDoxWizard with all standard panels, starting at page one.
     *
     * @param cF the CDoxFrame
     * @param title The title of the Wizard, which is wizard
     */
    public CdoxWizard(CDoxFrame cF, String title){
	super(cF, title, true);
	cFrame = cF;
	cFrame.wizardOn(true, this);
	g=(Graphics2D)cFrame.getGraphics();

	//calculate largest size of Question-Strings
	int maxWidth = (int)(g.getFont()).getStringBounds
	    (question[0], ((Graphics2D)g).getFontRenderContext()).getWidth();
	for(int i = 1; i<question.length;i++) {
	    int temp = (int)(g.getFont()).getStringBounds
	    (question[i], ((Graphics2D)g).getFontRenderContext()).getWidth();
	    if(temp>maxWidth)
		maxWidth = temp;
	}
	getContentPane().setLayout(new GridBagLayout());
	//create new QuestionPanel
	qPanel = new QuestionPanel(maxWidth+20, g.getFont());
	
	//add the question panel to the upper panel
	gb.weightx=1f;
	gb.anchor=gb.WEST;
	upperPanel.add(qPanel, gb);

	//and add the "c		cFrame.setWizardOn(true);lose wizard"-button to the upper panel.
	gb.weightx=0f;
	gb.anchor=gb.EAST;
	gb.gridx = 1; 
	gb.fill= gb.NONE;
	buttons[3].addActionListener(this);
	upperPanel.add(buttons[3], gb);

	//than add the upperpanel to the dialog
	gb.gridx=0;
	gb.gridy=0;
	gb.anchor=gb.WEST;
	gb.fill=gb.BOTH;
	getContentPane().add(upperPanel, gb);
	
	//add the MainPanel (which interacts with the user) to the Dialog.
	gb.gridy=1;
	getContentPane().add(mainPanel, gb);

	//set the size.	
	maxWidth+=qPanel.getImageWidth()+buttons[3].getPreferredSize().width+20;
	mainPanel.setPreferredSize(new Dimension(maxWidth, 220));
	mainPanel.setMinimumSize(new Dimension(maxWidth, 220));
	mainPanel.setMaximumSize(new Dimension(maxWidth, 220));

	//set mainPanel border
	mainPanel.setBorder(BorderFactory.createEtchedBorder());

	//add the buttons to the buttonPanel
	gb.gridy=0;
	gb.weightx=0f;
	gb.anchor=gb.EAST;
	gb.insets=new Insets(0,5,0,0);
	for (int i = 0; i<buttons.length-1;i++){
	    gb.gridx =i;
	    buttonPanel.add(buttons[i],gb);
	    buttons[i].addActionListener(this);
	    buttons[i].setPreferredSize(buttonSize);
	}

	//and add the button panel to the Dialog
	gb.gridx=0;
	gb.gridy=2;
	getContentPane().add(buttonPanel, gb);

	//set the border for the imagePanel (page 2)cdoxdoc=CDox Dokument
	imagePanel.setBorder(BorderFactory.createEtchedBorder());
	
	//create the 1st page radiobuttons
	makeRadioButtons();

	//and update the mainpanel
	updateMainPanel();

	//escape key
	new EscapeListener(buttons[2], getRootPane());
	
	getRootPane().setDefaultButton(buttons[1]);
	setResizable(false);
	cFrame.center(this);
	show();
    }

    /**
     * Fills the radioButton-Array with radioButtons, each having a selected and a
     * not-selected image. Each button has this class as an ItemListener.
     * @see itemStateChanged
     */
    private void makeRadioButtons(){
	ImageIcon icon=null;
	ImageIcon sIcon=null;
	String title=null;;
	for(int i=0; i< radioButtons.length;i++){
	    switch(i){
	    case 0:
		icon = new ImageIcon(getClass().getResource("/cdox/images/front.gif"));
		sIcon = new ImageIcon(getClass().getResource("/cdox/images/pressedfront.gif"));
		title=lang.get("wizardfront");
		break;
	    case 1:
		icon = new ImageIcon(getClass().getResource("/cdox/images/book.gif"));
		sIcon = new ImageIcon(getClass().getResource("/cdox/images/pressedbook.gif"));
		title=lang.get("wizardbooklet");
		break;
	    case 2:
		icon = new ImageIcon(getClass().getResource("/cdox/images/back.gif"));
		sIcon = new ImageIcon(getClass().getResource("/cdox/images/pressedback.gif"));
		title=lang.get("wizardback");
		break;
	    case 3:
		icon = new ImageIcon(getClass().getResource("/cdox/images/frontback.gif"));
		sIcon = new ImageIcon(getClass().getResource("/cdox/images/pressedfrontback.gif"));
		title=lang.get("wizardfrontback");
		break;
	    case 4:
		icon = new ImageIcon(getClass().getResource("/cdox/images/bookback.gif"));
		sIcon = new ImageIcon(getClass().getResource("/cdox/images/pressedbookback.gif"));
		title=lang.get("wizardbookletback");
		break;
	    }
	    radioButtons[i] = new JRadioButton(title, icon);
	    radioButtons[i].addItemListener(this);
	    radioButtons[i].setSelectedIcon(sIcon);
	    if(i==0) radioButtons[i].setSelected(true);
	    bg.add(radioButtons[i]);
	}
    }

    /**
     * Add the text-on /off preview images to the imagePanel, which will be added to the
     * mainPanel
     *
     * @param withText is true when the text-on images should be drawn, false otherwhise.
     */
    private void setTextImages(boolean withText){
	insertByHand = withText;
	ImageIcon icon=null;
	imagePanel.removeAll();
	for(int i=0; i<covers.length;i++){
	    switch(covers[i].getType()){
	    case CDFRONT:
		icon = new ImageIcon(getClass().getResource((withText)?
							    "/cdox/images/wtextonfront.gif":
							    "/cdox/images/wtextofffront.gif"));
		break;
	    case CDBOOKLET:
		icon = new ImageIcon(getClass().getResource((withText)?
							    "/cdox/images/wtextonbook.gif":
							    "/cdox/images/wtextoffbook.gif"));
		break;
	    case CDBACK_SIDE:
		icon = new ImageIcon(getClass().getResource((withText)?
							    "/cdox/images/wtextonback.gif":
							    "/cdox/images/wtextoffback.gif"));
		break;
	    }
	    JLabel label = new JLabel(icon);
	    gb.insets =new Insets(3,3,3,3);
	    gb.gridy=i;
	    imagePanel.add(label, gb);
	}
	resetConstraints();
	imagePanel.updateUI();
	paintComponents(getGraphics());
    }
    
    /**
     * This method is called to build a new "Page" in the dialog. It sets the question to
     * the currentQuestion and updates the mainPanel for this question. Four mainPanels
     * are defined, 0->Which cover, 1-> have an image?, 2->Fill in with hand, 3-> Finish.
     *
     */
    private void updateMainPanel(){
	//The mainPanel is not cleared if switched to the finish panel.
	if(dialogCount!=3)mainPanel.removeAll();
	switch(dialogCount){
	case 0://Ask the user which kind of cover he/she wants to make
	    //set the "previous" button off
	    buttons[0].setEnabled(false);

	    //Ask new question
	    qPanel.setQuestion(question[0], g);
	    gb.anchor = gb.WEST;
	    gb.weightx = 1.0f;
	    //add the radioButtons to the mainPanel
	    for(int i=0; i< radioButtons.length;i++){
		gb.gridy=i;
		gb.insets = new Insets(2,3,0,0);

		//add the changes to the mainPanel
		mainPanel.add(radioButtons[i], gb);
	    }
	    break;
	case 1://Ask the user if he/she has an image he/she wants to use.
	    //set the "previous" button on
	    buttons[0].setEnabled(true);

	    //Ask new question
	    qPanel.setQuestion((covers.length <=1)?
			       question[1]:
			       question[2], 
			       g);

	    //add the selected cover(s) to the mainpanel as a preview.
	    gb.anchor=gb.WEST;
	    gb.weightx=1f;
	    gb.weighty=1f;
	    gb.insets = new Insets(0,3,0,0);
	    for(int i=0; i<covers.length;i++){
		String buttonString = null;
		switch(covers[i].getType()){
		case CDFRONT:
		    buttonString= lang.get("wizardimagefront");
		    break;
		case CDBOOKLET:
		    buttonString= lang.get("wizardimagebooklet");
		    break;
		case CDBACK_SIDE:
		    buttonString= lang.get("wizardimageback");
		    break;
		}
		WizardPreview wp = new WizardPreview(cFrame, 
						     buttonString, 
						     covers[i]);

		//add the changes to the mainPanel
		gb.gridy=i;
		mainPanel.add(wp, gb);
	    }
	    break;
	case 2://Ask the user if he/she wants to insert her/his own tracks by hand.
	    //If coming from finish screen set the "ready" button to "next" button.
	    if(buttons[1].getText().equals(lang.get("wizardready")))
		buttons[1].setText(lang.get("nextbutton"));
	    
	    //clean the covers.	g2d.setPaint(Color.BLACK);
	    for(int i=0;i<covers.length;i++){
		covers[i].removeAllElements();
	    }
	    insertByHand = false;

	    //Ask new question
	    qPanel.setQuestion(question[3], g);

	    //add the radioButtons and the previews for text on/off.
	    bg = new ButtonGroup();
	    gb.insets=new Insets(0,3,0,0);
	    gb.anchor=gb.WEST;
	    for (int i=0; i<2;i++){
		JRadioButton rButton = new JRadioButton((i==0)?lang.get("yes"):lang.get("no"));
		if(i==1)rButton.setSelected(true);
		gb.gridy=i;
		bg.add(rButton);
		rButton.addItemListener(this);
		mainPanel.add(rButton, gb);
	    }

	    //use the "text-off" images for the preview
	    setTextImages(false);

	    //add the changes to the mainPanel
	    gb.gridx=1;
	    gb.gridy=0;
	    gb.weightx=1f;
	    gb.gridheight=2;
	    gb.anchor=gb.CENTER;
	    mainPanel.add(imagePanel, gb);
	    break;
	case 3://ask the user to push the finish button
	    //If the user wanted to insert the tracks by hand, open the Properties dialog.
	    if(insertByHand){
		ChooseDataDialog cd = new ChooseDataDialog(cFrame, lang, chosenData);
		cd.show();
		chosenData = cd.getWizardData();
	    }
	    
	    //Clear the mainPanel
	    mainPanel.removeAll();

	    //set the "next" button to "ready"
	    buttons[1].setText(lang.get("wizardready"));

	    //Ask new question
	    qPanel.setQuestion((covers.length<=1)?
		question[4]:
		question[5], 
		g);
	    
	    //insert the finished covers as a preview
	    gb.weightx=1f;
	    gb.weighty=1f;
	    gb.insets = new Insets(0,0,0,5);
	    for(int i=0; i<covers.length;i++){
		WizardPreview wp = new WizardPreview(cFrame, 
						     null, 
						     covers[i]);
		//add the changes to the mainPanel
		gb.gridx=i;
		mainPanel.add(wp, gb);
	    }
	    break;
	    //TODO
	}
	//set all changes made to the GridBagConstraints to default-values
	resetConstraints();
	
	//paint the dialog's components anew
	paintComponents(getGraphics());
	
	//repaint the dialog
	repaint();
    }
    
    /**
     *Resets all used GridBagConstraints values to there standard values.
     */
    private void resetConstraints(){
	gb.weightx=0f;
	gb.weighty=0f;
	gb.anchor=gb.CENTER;
	gb.gridheight=1;
	gb.fill=gb.NONE;
	gb.gridx=0;
	gb.gridy=0;
    }
    
    
    /**
     * Returns the covers used in this wizard.
     *
     * @return the covers in the wizard.
     */
    public Cover[] getWizardCovers(){
	return covers;
    }

    /**
     * This methods handles all the radioButton events used in all pages (1 and 3) in the
     * wizard. It separates between the two pages. 
     *
     * @param ie an ItemEvent which represents the action on a radioButton
     */
    public void itemStateChanged(ItemEvent ie){
	if(ie.getStateChange() == ie.SELECTED){
	    JRadioButton button = (JRadioButton)ie.getSource();
	    if(dialogCount == 0){//first page
		//fill the covers array with the appropriate Cover(s).
		if(button == radioButtons[0]){
		    covers = new Cover[1];
		    covers[0] = new Cover(CDFRONT);
		}
		else if(button == radioButtons[1]){
		    covers = new Cover[1];
		    covers[0] = new Cover(CDBOOKLET);
		}
		else if(button == radioButtons[2]){
		    covers = new Cover[1];
		    covers[0] = new Cover(CDBACK_SIDE);
		}
		else if(button == radioButtons[3]){
		    covers = new Cover[2];
		    covers[0] = new Cover(CDFRONT);
		    covers[1] = new Cover(CDBACK_SIDE);
		}
		else if(button == radioButtons[4]){
		    covers = new Cover[2];
		    covers[0] = new Cover(CDBOOKLET);
		    covers[1] = new Cover(CDBACK_SIDE);
		}
		//reset the users hand inserted tracks to null.
		chosenData=null;
	    }
	    else if(dialogCount == 2){//third page
		//if the yes radiobutton is pressed, set the preview images with text,
		//else with no text.
		setTextImages((button.getText().equals(lang.get("yes")))?
			      true:false);
	    }
	}
    }
        /**
     * Listens for the buttons in the buttonPanel and does the actions. If Next
     * dialogCount is +1, if prev DialogCount is -1.
     *
     * @param ae the ActionEvent which represents the click of a mouse-button.
     */
    public void actionPerformed(ActionEvent ae){
	String name=((JButton)ae.getSource()).getText();
	if(name.equals(lang.get("prevbutton"))){
	    dialogCount--;
	    updateMainPanel();
	}
	else if(name.equals(lang.get("nextbutton"))){
	    dialogCount++;
	    updateMainPanel();
	}
	else if(name.equals(lang.get("cancelbutton"))){
	    cFrame.wizardOn(false, null);
	    hide();
	    dispose();    
	}
	else if(name.equals(lang.get("wizardoffbutton"))){
	    Preferences prefs=Preferences.userNodeForPackage(CDoxFrame.class);
	    prefs.putBoolean("wizard", false);
	    cFrame.wizardOn(false, null);
	    hide();
	    dispose();
	    cFrame.showNewWizard();
	}
	else if(name.equals(lang.get("wizardready"))){
	    if(cFrame.getEditPane().hasComponents()){//check to see if there were Components in the EditPane
		cFrame.getEditPane().removeCoverTemplates();//remove them
	    }
	    if(cFrame.getTextEnabled()){
		cFrame.updateTextBar(null);
	    }
	    cFrame.resetSaveFile();
	    cFrame.setData(chosenData);
	    cFrame.addCovers(covers,lang.get("untitled"));
	    //checks to see if the project should be set to unsaved or not.
	    for(int i=0;i<covers.length;i++){
		if(covers[i].hasBackgroundImage() ||
		   covers[i].hasElements()){
		    cFrame.setSave(true);
		    break;
		}
	    }
	    cFrame.wizardOn(false, null);
	    hide();
	    dispose();
	}
    }
}
