package cdox.gui.wizard;

import javax.swing.*;
import java.awt.*;
import javax.imageio.*;
import java.awt.image.*;
import java.io.*;
import java.awt.geom.*;
import cdox.gui.*;

/**
 * The QuestionPanel is a Cow image on the left, which says a given String, in a
 * comic-balloon kind of thing. It is used in the CDoxWizard.
 *
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 10th 2002
 * @see CdoxWizard
 */
public class QuestionPanel extends JPanel{
    private BufferedImage cdox;
    private String quest=null;
    private Rectangle2D stringBounds = null;
    private int[] polyX={23,40,40,} ;
    private int[] polyY={20,15, 25};
    private Dimension size = new Dimension(0,0);
    /**
     * Creates a new <code>QuestionPanel</code> instance. It reads the cow image from
     * disk.
     *
     */
    public QuestionPanel(int width, Font f){
	super();
	try{
	    cdox = ImageIO.read(getClass().getResource("/cdox/images/cdoxquest.gif"));
	}
	catch(IOException ioe){ CDoxFrame.handleError(ioe, true);}
	setPreferredSize(new Dimension(width+cdox.getWidth(), cdox.getHeight()));
	setMinimumSize(new Dimension(width+cdox.getWidth(), cdox.getHeight()));
	setMaximumSize(new Dimension(width+cdox.getWidth(), cdox.getHeight()));
	setFont(f.deriveFont(Font.ITALIC));
    }

    
    /**
     * <code>getImageWidth</code> returns the widht of the question image.
     *
     * @return the width of the image
     */
    public int getImageWidth(){
	return cdox.getWidth();
    }

    /**
     * Draws the given String in the comic-balloon on the Panel. It automatically sets the
     * size of this panel to fit the String.
     *
     * @param q The String to be drawn
     * @param g A graphics object, used to calculate the String size with.
     */
    public void setQuestion(String q, Graphics g){
	quest = q;
	stringBounds = (g.getFont()).getStringBounds
	    (quest, ((Graphics2D)g).getFontRenderContext());
	
	//the new size of the Panel.
 	size =new Dimension((int)stringBounds.getWidth() + cdox.getWidth()+15,
 				     cdox.getHeight());
	updateUI();
    }

    /**
     * This paint-method first draws the cow-image, than Draws the comic-balloon, and it's
     * bounds. After that the String is drawn ontop of it all.
     *
     * @param g the Graphics object to draw on.
     */
    public void paint(Graphics g){
	Graphics2D g2d = (Graphics2D)g;
	g2d.drawRenderedImage(cdox, null);
	g2d.setPaint(Color.WHITE);
	g2d.fillRoundRect(cdox.getWidth()+7, 
			  getSize().height/2 - (int)stringBounds.getHeight()/2-2,
			  (int)stringBounds.getWidth()+10,
			  (int)stringBounds.getHeight()+5,
			  5,
			  5);

	g2d.setPaint(Color.BLACK);
	g2d.drawRoundRect(cdox.getWidth()+7, 
			  getSize().height/2 - (int)stringBounds.getHeight()/2-2,
			  (int)stringBounds.getWidth()+10,
			  (int)stringBounds.getHeight()+5,
			  5,
			  5);
	g2d.setPaint(Color.WHITE);
	g2d.fillPolygon(polyX, polyY, 3);
	g2d.setPaint(Color.BLACK);
	g2d.drawLine(23, 20, 40, 15);
	g2d.drawLine(23, 20, 40, 25);
	g2d.drawString(quest, cdox.getWidth()+10, getSize().height/2+(int) stringBounds.getHeight()/2);
    }
}
	
	
