package cdox.gui.wizard;

import javax.swing.*;
import java.awt.*;
import cdox.gui.*;
import cdox.edit.*;
import cdox.*;
import cdox.util.*;
import java.io.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;

/**
 * WizardPreview takes a Cover and draws it on a JPanel. It optionally can have a button
 * which opens a filechooser, to insert an Image (as BackgroundImage) to the Cover.
 *
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 10th 2002
 * @see QuestionPanel
 */
public class WizardPreview extends JPanel implements ActionListener, CDCoverStandards{
    private JButton openButton;
    private CDoxFrame cFrame;
    private String key = null;
    private Cover cover;
    private Dimension drawSize;
    private BufferedImage coverImage = null;
    private AffineTransform scaled;
    private AffineTransformOp scaledTransform;
    private float scaleFactor;
    private GridBagConstraints gb = new GridBagConstraints();
    private int buttonWidth=0;

    /**
     * The constructor creates a preview-of the given cover and draws it as an
     * scaledImage. The scaling is done to 100. 
     *
     * @param cF the CDoxFrame
     * @param bText the String to draw on a Button, if null no button is added to the
     * WizardPreview.
     * @param cov the Cover to make a preview from.
     */
    public WizardPreview(CDoxFrame cF, String bText, Cover cov){
	super(new GridBagLayout());
	cFrame = cF;
	cover = cov;
	//if a button text is given, add a button to this panel
	if(bText != null){
	    openButton = new JButton(bText);
	    openButton.setPreferredSize(new Dimension((int)openButton.getPreferredSize().getWidth(),
						      23));
	    openButton.addActionListener(this);
	    buttonWidth=openButton.getPreferredSize().width;
	    gb.weightx=2000;
	    gb.anchor=gb.EAST;
	    add(openButton, gb);
	}

	//the Size of the given cover.
	int[] size = cover.getSize();

	cover.setBackground(Color.WHITE);

	//scale the image of the cover
	scaleFactor=(float)100/(float)size[1];
	scaled = AffineTransform.getScaleInstance(scaleFactor,scaleFactor);
	scaledTransform = new AffineTransformOp
	    (scaled,cover.getRenderingHints());

	coverImage = scaledTransform.filter(cover.getBufferedImage(),null);
	drawSize = new Dimension(coverImage.getWidth(), coverImage.getHeight());

	//set the size of this cover
	setPreferredSize(new Dimension(drawSize.width+buttonWidth+5,drawSize.height));
	setMaximumSize(new Dimension(drawSize.width+buttonWidth+5, drawSize.height));
	setMinimumSize(new Dimension(drawSize.width+buttonWidth+5, drawSize.height));	
	
	cover.setStrokeSize(3f);	
	//reload the image for better drawing quality, no scaling here.
	coverImage = cover.getBufferedImage();
	cover.setStrokeSize(0.25f);
	updateUI();
    }

    /**
     * If a button is added to the panel, this is its ActionListener. It opens a
     * filechooser, and adds the chosen image as a background image to the Cover. A new
     * CoverImage is created.
     *
     * @param ae an <code>ActionEvent</code> value
     */
    public void actionPerformed(ActionEvent ae){
	int[] filters = {JPG, GIF, PNG, ALL};
	File file = cFrame.getFileDialog().showDialog(cFrame, CDox.getLocalizer().get("filetitlecdoxopen"), 
						      CDox.getLocalizer().get("fileapproveopenbutton"), 
						      null, filters);
	if(file !=null){
	    try{
		key = TempFiles.add(file);
		cover.setBackground(key, true);
	    }
	    catch(IOException ioe){ CDoxFrame.handleError(ioe, true);}
	    cover.setStrokeSize(3f);
	    coverImage = cover.getBufferedImage();
	    cover.setStrokeSize(0.25f);
	}
	getParent().paintComponents(getParent().getGraphics());
	updateUI();
	repaint();
    }
    
    /**
     * Draws the Cover image on the given Graphics object.
     *
     * @param g the Graphics object to draw on.
     */
    public void paint(Graphics g){
	Graphics2D g2d = (Graphics2D)g;
	g2d.setPaint(getBackground());
	g2d.fillRect(drawSize.width, 0, 
		     getPreferredSize().width, 
		     getPreferredSize().height);
	g2d.drawImage(coverImage,scaled,this);
	paintComponents(g2d);
    }
	
} 
	
	
