package cdox.print;

import cdox.*;
import cdox.gui.*;
import cdox.edit.*;
import cdox.util.*;
import java.util.*;
import java.awt.print.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;

/**
 * This class does all the printing, including all dialogs that retrieve the information
 * of the user.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 12th 2002
 */
public class PrintCover extends Thread implements CDCoverStandards{

    private Cover[] cover;
    private Localizer lang=CDox.getLocalizer();
    private CDoxFrame cFrame;
    private PrintService printer=null;

    /**
     * This constructs a new runnable instance. In case you start() this class, some
     * dialogs may pop up that ask the user some question about the printer that should be
     * used etc. After that, the covers will probably be printed.
     * @param cf the <code>CDoxFrame</code>.
     * @param cover the covers that shall be printed. Please note that the covers have to
     * fit onto one single page.
     * @exception IllegalArgumentException if the covers cannot be printed onto one single
     * page.
     */
    public PrintCover( CDoxFrame cf,Cover[] cover ){
	cFrame = cf;
	if(cover.length==0) throw new IllegalArgumentException(lang.get("nocover"));
	if(cover.length>2)
	    throw new IllegalArgumentException(lang.get("toomuchcovers"));
	this.cover=cover;
    }

    public void run(){
	//the printdialog and the service it returns
	// the next one will contain the user's choices in the dialog (ie, paper size)
	HashPrintRequestAttributeSet attributes=new HashPrintRequestAttributeSet();
	attributes.add(new JobName(lang.get("cdoxdoc"), Locale.getDefault()));
	if(cFrame.getAllPrinters()!=null){
	    try{
		printer=ServiceUI.printDialog
		    (null,200,200,
		     cFrame.getAllPrinters(),
		     cFrame.getDefaultPrinter(),   
		     DocFlavor.SERVICE_FORMATTED.PRINTABLE,attributes);
	    }
	    catch(Exception e){
		CDoxFrame.handleError(new Throwable(lang.get("noprinters")),false);
		cFrame.setWorking(false);
		return;
	    }
	}
	else {
	    CDoxFrame.handleError(new Throwable(lang.get("noprinters")),false);
	    cFrame.setWorking(false);
	    return;
	}
	if(printer == null){//the cancel button is pressed.
	    cFrame.setWorking(false);
	    return;
	}
	PrintServiceAttribute psa=(PrintServiceAttribute)printer.getAttributes().get
	    (ColorSupported.class);//checkk if the printer is a color printer
	if(cover.length>1)
	    for(int i=0;i<cover.length;i++){
		cover[i].setPrintCompanion(cover);
	    }
	if(psa!=null){//the printer supports questioning
	    if(psa.equals(ColorSupported.SUPPORTED)){//a color printer
		for(int i=0;i<cover.length;i++) cover[i].setGrayscale(false);
	    }
	    //a grayscale printer
	    else for(int i=0;i<cover.length;i++) cover[i].setGrayscale(true);
	}
	//the printer doesn't support questioning we assume it is a grayscale printer
	else for(int i=0;i<cover.length;i++) cover[i].setGrayscale(true);
	DocPrintJob d=printer.createPrintJob();//the new printjob
	SimpleDoc doc=new SimpleDoc(cover[0],DocFlavor.SERVICE_FORMATTED.PRINTABLE,null);

	try{
	    d.print(doc,attributes);
	}catch(PrintException pe){
	    cFrame.setWorking(false);
	    CDoxFrame.handleError(pe,true);
	}
	System.gc();
	Cover activeCover = cFrame.getEditPane().getDrawingPane().getCover();
	for(int i = 0; i<cover.length;i++){
	    if(cover[i] == activeCover)
		activeCover.setGrayscale(false);
// 	    else
// 		cover[i].setGrayscale(true);
	}
	cFrame.paintComponents(cFrame.getGraphics());
	cFrame.setWorking(false);
    }

    public String toString(){
	return "PrintCover";
    }

}
