package cdox.util;

import java.awt.event.*;
import javax.swing.*;

/**
 * This class simply clicks the button if the escape key is hit. This is useful for
 * dialogs where the escape key should do the same as clicking the cancel button. A simple
 * usage example would be: <code>new EscapeListener(button,getRootPane());</code>, where
 * button would be the cancel button and the line would be placed in the constructor of a
 * JDialog, for example.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class EscapeListener extends AbstractAction{

    private AbstractButton button;
    private JRootPane panel;

    /**
     * Constructs one.
     *@param button the button that should be pressed when the escape key is typed.
     *@param panel where to register the key stroke.
     */
    public EscapeListener( AbstractButton button, JRootPane panel ){
	this.panel=panel;
	this.button=button;
 	KeyStroke ks=KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0);
 	putValue(ACCELERATOR_KEY,ks);
 	panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).
 	    put(ks,"escape");
 	panel.getActionMap().put("escape",this);
    }

    /**
     * Unregisters the keystroke binding from the input and action map.
     */
    public void invalidateListener(){
 	KeyStroke ks=KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0);
	putValue(ACCELERATOR_KEY,null);
	panel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).remove(ks);
	panel.getActionMap().remove("escape");
    }

    /**
     * Clicks the button.
     * @param evt an <code>ActionEvent</code> value.
     */
    public void actionPerformed( ActionEvent evt ){
	button.doClick();
    }

}
