package cdox.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * This is an extended JTextField that writes uses the key events to write the keystrokes
 * as it value. One can get the key and modifier values pressed as int values.
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */

public class JKeyStrokeField extends JTextField{

    private int keyValue, modValue;
    private static JKeyStrokeField oldField=null;
    private JKeyStrokeField itsme=this;

    /**
     * Constructor whom you must provide the initial value.
     *@param key the key value.
     *@param mod the modifiers value
     */
    public JKeyStrokeField( int key, int mod ){
	setValue(key,mod);
	addKeyListener(new KeyAdapter(){
		public void keyPressed( KeyEvent evt ){
		    setValue(evt.getKeyCode(),evt.getModifiers());
		}
	    });
	addMouseListener(new MouseAdapter(){
		public void mousePressed( MouseEvent evt ){
		    if(oldField!=null) oldField.setBackground(getBackground());
		    oldField=itsme;
		    setBackground(Color.WHITE);
		}
	    });
	setEditable(false);
    }

    /**
     * Returns the actual values.
     *@return the values, in [0] is the key value and in [1] the modifier value.
     */
    public int[] getValue(){
	return new int[]{keyValue,modValue};
    }

    /**
     * Sets the actual value.
     *@param key the key value.
     *@param mod the modifiers value
     */
    public void setValue( int key, int mod ){
	setText(KeyEvent.getKeyModifiersText(mod)+" "+KeyEvent.getKeyText(key));
	keyValue=key;
	modValue=mod;
    }

}
