package cdox.util;

import java.util.*;
import org.xml.sax.*;

/**
 * This class can be used as general error handler if one wants to know about the errors
 * but not to take an action if any error occurs. The complete errors can be asked about
 * after parsing (possibly also while parsing).
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class XMLErrorHandler implements org.xml.sax.ErrorHandler{

    private ArrayList errors=new ArrayList(),
	fatalerrors=new ArrayList(),
	warnings=new ArrayList();

    public void error( SAXParseException e ){
	errors.add(e.getMessage()+" in line "+e.getLineNumber()+" column "+e.getColumnNumber());
    }

    public void fatalError( SAXParseException e ){
	fatalerrors.add(e.getMessage()+" in line "+e.getLineNumber()+" column "+e.getColumnNumber());
    }

    public void warning( SAXParseException e ){
	warnings.add(e.getMessage()+" in line "+e.getLineNumber()+" column "+e.getColumnNumber());
    }

    /**
     * Returns the number of errors.
     *@return the number.
     */
    public int getNumberOfErrors(){
	return errors.size();
    }

    /**
     * Returns the number of fatal errors.
     *@return the number.
     */
    public int getNumberOfFatalErrors(){
	return fatalerrors.size();
    }

    /**
     * Returns the number of warnings.
     *@return the number.
     */
    public int getNumberOfWarnings(){
	return warnings.size();
    }

    /**
     * Returns true, if no errors, fatal errors and warnings occured.
     *@return false otherwise.
     */
    public boolean validated(){
	return ((errors.size()==0)&&
		(fatalerrors.size()==0)&&
		(warnings.size()==0));
    }

    /**
     * Returns the errors as string objects in an iterator.
     * @return an <code>Iterator</code> value.
     */
    public Iterator getErrors(){
	return errors.iterator();
    }

    /**
     * Returns the fatal errors as string objects in an iterator.
     * @return an <code>Iterator</code> value.
     */
    public Iterator getFatalErrors(){
	return fatalerrors.iterator();
    }

    /**
     * Returns the warnings as string objects in an iterator.
     * @return an <code>Iterator</code> value.
     */
    public Iterator getWarnings(){
	return warnings.iterator();
    }

}
