package cdox.util.conf;

import cdox.*;
import cdox.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.prefs.*;
import javax.swing.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *<p>
 * This dialog constructs itself from a xml configuration file where all configurable
 * options are stored with their default values. It sets (in case of pressing the ok
 * button) the prefs automatically. You can automatically configure objects by using the
 * configure() method of this class.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 *@see Configurable
 */
public class ConfigurationDialog extends JDialog{

    private Localizer lang=null;
    private HashSet options=new HashSet();
    private HashMap configurationValues=new HashMap();
    private boolean okPressed=false;
    private HashSet keystrokes=new HashSet();
    private JDialog itsme=this;
    private boolean mustRestart=false;

    /**
     * Constructs one.
     *@param parentFrame the parent frame.
     *@param language the desired language.
     *@param in the input stream from which the xml configuration file containing the
     * options should be read.
     */
    public ConfigurationDialog( JFrame parentFrame, String language, InputStream in ){
	// misc
	super(parentFrame,true);
	lang=Localizer.getLocalizer(language);
	// init the screen size that we have and calculate a good size for it
	int maximumX=Toolkit.getDefaultToolkit().getScreenSize().height-
	    Toolkit.getDefaultToolkit().getScreenInsets
	    (GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().
	     getDefaultConfiguration()).bottom-
	    Toolkit.getDefaultToolkit().getScreenInsets
	    (GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().
	     getDefaultConfiguration()).top-100;
	// to init the double keystroke check correctly
	KeyStrokeOption.resetValidityCheck();
	// parsing and creating of some panels follows, creating the GUI
 	JTabbedPane tabbed=new JTabbedPane(); // will wrap them at top (defaults)
	try{
	    Document doc=DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(in);
	    setTitle(getAttributeByLanguage(doc,"description","title"));
	    NodeList pages=doc.getElementsByTagName("page");
	    for(int i=0;i<pages.getLength();i++){ // iterates over all <page> elements
		// each page element has some <separator> elements, which result in
		// several panels. These panels end up in the following list:
		ArrayList separatorPanels=new ArrayList();
		NodeList separators=((Element)pages.item(i)).getElementsByTagName("separator");
		for(int j=0;j<separators.getLength();j++){
		    JPanel panel=new JPanel(); // create a new titled panel
		    panel.setLayout(new GridBagLayout());
		    panel.setBorder(BorderFactory.createTitledBorder
				    (BorderFactory.createEtchedBorder(),
				     AbstractOption.getTextByLanguage(separators.item(j),"text",lang)));
		    NodeList childs=separators.item(j).getChildNodes();
		    GridBagConstraints gb=new GridBagConstraints();
		    gb.anchor=gb.WEST;
		    gb.weightx=1;
		    gb.gridy=0;
		    gb.gridx=0;
		    gb.insets=new Insets(1,2,1,2);
		    // the next one extracts and adds the options to the panel
		    for(int k=0;k<childs.getLength();k++){
			Node n=childs.item(k);
			if(n.getNodeName().equals("keystroke"))
			    addOption(panel,gb,new KeyStrokeOption(n,lang));
			if(n.getNodeName().equals("language"))
			    addOption(panel,gb,new LanguageOption(n,lang));
			if(n.getNodeName().equals("yesno")){
			    AbstractOption o=new YesNoOption(n,lang);
			    gb.gridx=0;
			    gb.gridwidth=2;
			    gb.weightx=1;
			    panel.add(o.getComponent(),gb);
			    gb.weightx=0;
			    gb.gridx=1;
			    gb.gridy++;
			    options.add(o);
			    gb.gridwidth=1;
			}
		    }
		    // add it
		    separatorPanels.add(panel);
		}
		// finally adding the new tabs, check how much pages we need etc.
		Iterator it=separatorPanels.iterator();
		JPanel root=new JPanel();
		root.setLayout(new GridBagLayout());
		GridBagConstraints gb=new GridBagConstraints();
		String title=AbstractOption.getTextByLanguage(pages.item(i),"title",lang);
		int count=1;
		int size=0;
		gb.gridx=0;
		gb.insets=new Insets(1,2,1,2);
		gb.gridy=0;
		gb.weightx=1;
		gb.anchor=gb.NORTH;
		gb.fill=gb.BOTH;
		while(it.hasNext()){
		    JPanel panel=(JPanel)it.next();
		    if((size+panel.getPreferredSize().height)>maximumX){
			gb.weighty=1;
			root.add(new JLabel(),gb);
			gb.gridy++;
			tabbed.addTab(title+" "+count,root);
			size=0;
			root=new JPanel();
			root.setLayout(new GridBagLayout());
			count++;
			gb.gridx=0;
			gb.gridy=0;
			gb.weighty=0;
		    }
		    size+=panel.getPreferredSize().height;
		    root.add(panel,gb);
		    gb.gridy++;
		}
		gb.weighty=1;
		root.add(new JLabel(),gb);
		gb.gridy++;
		if(count==1) tabbed.addTab(title,root);
		else tabbed.addTab(title+" "+count,root);
	    }
	}catch(ParserConfigurationException pce){
	    cdox.util.ErrorHandler.log(pce,true);
	}catch(SAXException saxe){
	    cdox.util.ErrorHandler.log(saxe,true);
	}catch(IOException ioe){
	    cdox.util.ErrorHandler.log(ioe,true);
	}
 	getContentPane().setLayout(new GridBagLayout());
 	GridBagConstraints gb=new GridBagConstraints();
 	gb.gridwidth=3;
 	gb.gridx=0;
 	gb.gridy=0;
 	gb.insets=new Insets(2,2,2,2);
 	getContentPane().add(tabbed,gb);
	gb.gridy=1;
	gb.gridwidth=1;
	gb.weightx=1;
	JButton button=new JButton(lang.get("okbutton"));
	getRootPane().setDefaultButton(button);
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    KeyStrokeOption[] inv=KeyStrokeOption.checkValidity();
		    if(inv!=null){
			JOptionPane.showMessageDialog
			    (itsme,lang.get("doublekeys",new String[]{inv[0].getHint(),inv[1].getHint()}),
			     lang.get("error"),JOptionPane.ERROR_MESSAGE);
			return;
		    }
		    Iterator i=options.iterator();
		    while(i.hasNext()){
			AbstractOption o=(AbstractOption)i.next();
			if(o.commitChanges()) mustRestart=true;
			o.setMapping(configurationValues);
			okPressed=true;
		    }
		    Preferences prefs=Preferences.userRoot();
		    try{
			prefs.flush();
		    }catch(BackingStoreException bse){ // again!
			cdox.util.ErrorHandler.log(bse,true);
		    }
		    setVisible(false);
		    dispose();
		}
	    });
	getContentPane().add(button,gb);
	gb.gridx=1;
	button=new JButton(lang.get("cancelbutton"));
	new EscapeListener(button,getRootPane());
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    setVisible(false);
		    dispose();
		    okPressed=false;
		}
	    });
	getContentPane().add(button,gb);
	gb.gridx=2;
	button=new JButton(lang.get("defaultbutton"));
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    Iterator i=options.iterator();
		    while(i.hasNext()) ((AbstractOption)i.next()).setToDefault();
		    okPressed=false;
		}
	    });
	getContentPane().add(button,gb);
	center();
	setResizable(false);
    }

    // adds an option to this dialog by adding it to the overall options hashset and
    // adding the label and the component to the panel using the gridbagconstraints
    private void addOption( JPanel panel, GridBagConstraints gb, AbstractOption o ){
	gb.gridx=0;
	gb.weightx=1;
	panel.add(o.getText(),gb);
	gb.weightx=0;
	gb.gridx=1;
	panel.add(o.getComponent(),gb);
	gb.gridy++;
	options.add(o);
    }

    // resets to the actual values
    public void show(){
	mustRestart=false;
	Iterator i=options.iterator();
	while(i.hasNext()){
	    AbstractOption o=(AbstractOption)i.next();
	    o.reset();
	}
	pack();
	super.show();
    }

    // returns the value of the attribute whose element in the nodelist matches the
    // current language of the localizer, if nothing matches, returns null
    private String getAttributeByLanguage( Document doc, String element, String attribute ){
	NodeList nl=doc.getElementsByTagName(element);
	for(int i=0;i<nl.getLength();i++){
	    String v=nl.item(i).getAttributes().getNamedItem("xml:lang").getNodeValue();
	    if(lang.getLanguageCode().startsWith(v)||
	       v.startsWith(lang.getLanguageCode()))
		return nl.item(i).getAttributes().getNamedItem(attribute).getNodeValue();
	}
	return null;
    }

    // centers the dialog
    private void center(){
	pack();
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension fs = getSize();
        setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
    }

    /**
     * Configures the objects mapped in this class.
     *@param map a map mapping the names used in the XML configuration file to the
     * objects that implement the Configurable interface.
     *@see Configurable
     */
    public void configure( HashMap map ){
	// first initialize the map with good values
	configurationValues.clear();
	Iterator i=options.iterator();
	while(i.hasNext()){
	    AbstractOption o=(AbstractOption)i.next();
	    o.setMapping(configurationValues);
	}
	i=map.keySet().iterator();
	while(i.hasNext()){
	    String s=i.next().toString();
	    ((Configurable)map.get(s)).configure(configurationValues.get(s));
	}
    }

    /**
     * Returns whether the last pressed button has been the ok button.
     *@return false, if no button had been pressed.
     */
    public boolean lastOKPressed(){
	return okPressed;
    }

    /**
     * Returns whether the app must be restarted to take the new config values into
     * effect.
     *@return true, if restart is needed.
     */
    public boolean mustRestart(){
	return mustRestart;
    }

}
