package cdox.util.conf;

import cdox.util.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import org.w3c.dom.*;

/**
 * This class represents a YesNo option. Below is an example of an appropriate
 * section in the XML file:
 *<pre>
 * &lt;yesno package="cdox.gui.CDoxFrame" key="playsounds"&gt;
 *   &lt;default value="yes"/&gt;
 *   &lt;hint xml:lang="de"&gt;Sounds abspielen&lt;/hint&gt;
 *   &lt;hint xml:lang="en"&gt;Play sounds&lt;/hint&gt;
 * &lt;/yesno&gt;
 *</pre>
 * The value of the default attribute value should either equal "yes" or "no".
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version May 22nd 2002
 */
public class YesNoOption extends AbstractOption{

    private Localizer lang;
    private JCheckBox value;
    private boolean defaultValue=true;

    /**
     * Constructs one.
     *@param n the DOM node where this option is described.
     *@param lang a localizer.
     */
    public YesNoOption( Node n, Localizer lang ){
	super(n.getAttributes().getNamedItem("package").getNodeValue(),
	      n.getAttributes().getNamedItem("key").getNodeValue(),n);
	this.lang=lang;
	defaultValue=node.getElementsByTagName("default").item(0).getAttributes().
	    getNamedItem("value").getNodeValue().equals("yes");
    }

    public JLabel getText(){
	return new JLabel();
    }

    public JComponent getComponent(){
	value=new JCheckBox(getTextByLanguage(node,"hint",lang));
	reset();
	return value;
    }

    public boolean commitChanges(){
	prefs.putBoolean(key,value.isSelected());
	return false;
    }

    public void setToDefault(){
	value.setSelected(defaultValue);
    }

    public void reset(){
	value.setSelected(prefs.getBoolean(key,defaultValue));
    }

}
