package cdox.util.doc;

import cdox.util.*;
import cdox.util.conf.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 *<p>
 * This class provides an easy way to generate some static HTML pages for use at your
 * webpage out of an existing documentation.xml file and/or a configuration.xml file as
 * used by the classes in the conf package.
 *</p>
 * @author <a href="mailto:cdox@gmx.net">Rutger Bezema, Andreas Schmitz</a>
 * @version June 12th 2002
 */
public class WebPageHelper extends JFrame{

    private JTextField docFile=new JTextField("/cdox/cdox.xml");
    private JTextField confFile=new JTextField("/cdox/util/conf/example.xml");
    private JTextField targetDir=new JTextField("documentation");
    private JButton docButton=new JButton("Browse");
    private JButton confButton=new JButton("Browse");
    private JFileChooser chooser=new JFileChooser();
    private JFrame itsme=this;
    private JPanel docPanel=new JPanel();
    private JPanel confPanel=new JPanel();
    private JCheckBox docJar=new JCheckBox("Use as .jar path");
    private JCheckBox confJar=new JCheckBox("Use as .jar path");
    private JTextField homepage=new JTextField("../Show");

    /**
     * Constructs one.
     */
    public WebPageHelper(){
	setTitle("Webpage Helper");
	setDefaultCloseOperation(EXIT_ON_CLOSE);
	getContentPane().setLayout(new GridBagLayout());
	GridBagConstraints gb=new GridBagConstraints();
	// documentation section
	docPanel.setBorder(BorderFactory.createTitledBorder
			(BorderFactory.createEtchedBorder(),
			 "Documentation"));
	docPanel.setLayout(new GridBagLayout());
	gb.gridx=0;
	gb.gridy=0;
	gb.gridwidth=2;
	JCheckBox box=new JCheckBox("Process documentation");
	box.setSelected(true);
	box.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    boolean b=((JCheckBox)evt.getSource()).isSelected();
		    docPanel.setEnabled(b);
		    Component[] c=docPanel.getComponents();
		    for(int i=0;i<c.length;i++) c[i].setEnabled(b);
		    if(b) docButton.setEnabled(!docJar.isSelected());
		}
	    });
	getContentPane().add(box,gb);
	gb.gridy=1;
	gb.insets=new Insets(2,2,2,2);
	getContentPane().add(docPanel,gb);
	docPanel.add(new JLabel("Documentation XML:"));
	gb.gridy=0;
	gb.gridwidth=1;
	gb.gridx=1;
	docPanel.add(docFile,gb);
	gb.gridx=2;
	docPanel.add(docButton,gb);
	gb.gridx=0;
	gb.gridy=2;
	docJar.setSelected(true);
	docButton.setEnabled(false);
	docJar.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    docButton.setEnabled(!((JCheckBox)evt.getSource()).isSelected());
		}
	    });
	docPanel.add(docJar,gb);
	docButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    chooser.setFileSelectionMode(chooser.FILES_ONLY);
		    if(chooser.showOpenDialog(itsme)==chooser.APPROVE_OPTION){
			docFile.setText(chooser.getSelectedFile().toString());
		    }
		}
	    });
	gb.gridx++;
	JButton button=new JButton("Check validity");
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    validateDocumentation();
		}
	    });
	docPanel.add(button,gb);
	// configuration section
	gb.gridy=0;
	gb.gridx=2;
	gb.gridwidth=4;
	box=new JCheckBox("Process configuration");
	box.setSelected(true);
	box.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    boolean b=((JCheckBox)evt.getSource()).isSelected();
		    confPanel.setEnabled(b);
		    Component[] c=confPanel.getComponents();
		    for(int i=0;i<c.length;i++) c[i].setEnabled(b);
		    if(b) confButton.setEnabled(!confJar.isSelected());
		}
	    });
	getContentPane().add(box,gb);
	gb.gridy=1;
	confPanel.setBorder(BorderFactory.createTitledBorder
			(BorderFactory.createEtchedBorder(),
			 "Configuration"));
	confPanel.setLayout(new GridBagLayout());
	getContentPane().add(confPanel,gb);
	confPanel.add(new JLabel("Configuration XML:"));
	gb.gridy=0;
	gb.gridx=1;
	gb.gridwidth=1;
	confPanel.add(confFile,gb);
	gb.gridx=2;
	confPanel.add(confButton,gb);
	gb.gridx=0;
	gb.gridy=1;
	confJar.setSelected(true);
	confJar.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    confButton.setEnabled(!((JCheckBox)evt.getSource()).isSelected());
		}
	    });
	confPanel.add(confJar,gb);
	confButton.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    chooser.setFileSelectionMode(chooser.FILES_ONLY);
		    if(chooser.showOpenDialog(itsme)==chooser.APPROVE_OPTION){
			confFile.setText(chooser.getSelectedFile().toString());
		    }
		}
	    });
	gb.gridx++;
	button=new JButton("Check validity");
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    validateConfiguration();
		}
	    });
	confPanel.add(button,gb);
	// general components
	gb.gridx=0;
	gb.gridy=2;
	button=new JButton("Generate it!");
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    generate();
		}
	    });
	getContentPane().add(button,gb);
	gb.gridx=1;
	gb.fill=gb.BOTH;
	getContentPane().add(targetDir,gb);
	gb.gridx=2;
	gb.fill=gb.NONE;
	button=new JButton("Browse");
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    chooser.setFileSelectionMode(chooser.DIRECTORIES_ONLY);
		    if(chooser.showOpenDialog(itsme)==chooser.APPROVE_OPTION){
			targetDir.setText(chooser.getSelectedFile().toString());
		    }
		}
	    });
	getContentPane().add(button,gb);
	gb.gridx=3;
	button=new JButton("Exit");
	button.addActionListener(new ActionListener(){
		public void actionPerformed( ActionEvent evt ){
		    System.exit(0);
		}
	    });
	getContentPane().add(button,gb);
	gb.gridx=4;
	getContentPane().add(new JLabel("Homepage:"),gb);
	gb.gridx=5;
	getContentPane().add(homepage,gb);
	center();
	setResizable(false);
    }

    // checks whether the selected options are valid
    private boolean checkValidity(){
	if((!confPanel.isEnabled())&&(!docPanel.isEnabled())){
	    error("You have not selected anything to process.");
	    return false;
	}
	File f=new File(targetDir.getText());
	if(f.exists()&&(!f.isDirectory())){
	    error("You must select a directory as target.");
	    return false;
	}
	if(!f.exists())
	    if(JOptionPane.showConfirmDialog
	       (this,"Do you want me to create the target directory\n"+
		f.toString()+"?")!=JOptionPane.YES_OPTION) return false;
	if(docPanel.isEnabled()){
	    if(!docJar.isSelected()){
		f=new File(docFile.getText());
		if((f.exists()&&(!f.isFile()))||(!f.exists())){
		    error("The specified file is no xml file:\n"+f);
		    return false;
		}
	    }
	}
	if(confPanel.isEnabled()){
	    if(!confJar.isSelected()){
		f=new File(confFile.getText());
		if((f.exists()&&(!f.isFile()))||(!f.exists())){
		    error("The specified file is no xml file:\n"+f);
		    return false;
		}
	    }
	}
	return true;
    }

    // does all the work
    private void generate(){
	if(!checkValidity()) return;
	try{
	    new File(targetDir.getText()).mkdirs();
	    if(docPanel.isEnabled()) transformDocumentation();
	    if(confPanel.isEnabled()) transformConfiguration();
	}catch(IOException ioe){
	    ioe.printStackTrace();
	}catch(TransformerConfigurationException tce){
	    tce.printStackTrace();
	}catch(TransformerException te){
	    te.printStackTrace();
	}catch(SAXException saxe){
	    saxe.printStackTrace();
	}catch(ParserConfigurationException pce){
	    pce.printStackTrace();
	}
    }

    private void error( String message ){
	JOptionPane.showMessageDialog(this,message,"Error",JOptionPane.ERROR_MESSAGE);
    }

    // well, does IT
    private void transformDocumentation() throws IOException,
	TransformerConfigurationException,TransformerException{
	Documentation documentation;
	if(docJar.isSelected())
	    documentation=new Documentation(this,WebPageHelper.class.
				  getResourceAsStream(docFile.getText()));
	else documentation=new Documentation(this,new File(docFile.getText()));
	Document doc=documentation.getDocument(); // the original document
	// creating a new DOM source
	DOMSource source=new DOMSource(doc);
	// one transformer is sufficient
	Transformer t=TransformerFactory.newInstance().newTransformer
	    (new StreamSource(WebPageHelper.class.getResourceAsStream
			      ("/cdox/util/doc/xsl/static.xsl")));
	t.setParameter("homepage",homepage.getText());
	NodeList nl=doc.getElementsByTagNameNS
	    ("http://sammael.tabu.stw-bonn.de/schema/documentation","language");
	for(int i=0;i<nl.getLength();i++){ // iterate over languages
	    String language=nl.item(i).getAttributes().getNamedItemNS
		("http://www.w3.org/XML/1998/namespace","lang").getNodeValue();
	    // the following updates the links to match the files
	    NodeList nl3=doc.getElementsByTagName("a");
	    for(int k=0;k<nl3.getLength();k++){
		Node n=nl3.item(k).getAttributes().getNamedItem("href");
		String s=n.getNodeValue();
		if(s.indexOf(".")!=-1) s=s.substring(0,s.indexOf("."));
		n.setNodeValue(s+"."+language+".html");
	    }
	    t.setParameter("language",language);
	    if(confPanel.isEnabled()){
		t.setParameter("configuration","configuration."+language+".html");
		t.setParameter("configurationname","Configuration");
	    }
	    NodeList nl2=doc.getElementsByTagNameNS
		("http://sammael.tabu.stw-bonn.de/schema/documentation","topic");
	    for(int j=0;j<nl2.getLength();j++){ // iterate over topics
		String topic=nl2.item(j).getAttributes().getNamedItem("id").getNodeValue();
		t.setParameter("topic",topic);
		t.transform // finally...
		    (source,new StreamResult
		     (new FileOutputStream
		      (targetDir.getText()+File.separator+topic+"."+language+".html")));
	    }
	}
    }

    private void transformConfiguration() throws IOException,
	TransformerConfigurationException,TransformerException,
	ParserConfigurationException,SAXException{
	Document doc;
	DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	dbf.setNamespaceAware(true);
	DocumentBuilder db=dbf.newDocumentBuilder();
	if(confJar.isSelected())
	    doc=db.parse(WebPageHelper.class.getResourceAsStream(confFile.getText()));
	else doc=db.parse(new File(confFile.getText()));
	// make transformer
	DOMSource source=new DOMSource(doc);
	Transformer t=TransformerFactory.newInstance().newTransformer
	    (new StreamSource(WebPageHelper.class.getResourceAsStream
			      ("/cdox/util/conf/conf2html.xsl")));
	t.setParameter("homepage",homepage.getText());
	NodeList nl=doc.getElementsByTagNameNS
	    ("http://cdox.sf.net/schema/configuration","description");
	for(int i=0;i<nl.getLength();i++){ // iterate over languages
	    String language=nl.item(i).getAttributes().getNamedItemNS
		("http://www.w3.org/XML/1998/namespace","lang").getNodeValue();
	    t.setParameter("language",language);
	    t.transform(source,new StreamResult
			(targetDir.getText()+File.separator+"configuration."+language+".html"));
	}
    }

    // validates and checks the documentation (topic occurance in all languages etc.)
    private void validateDocumentation(){
	try{
	    InputStream in;
	    if(docJar.isSelected()) in=WebPageHelper.class.getResourceAsStream(docFile.getText());
	    else in=new FileInputStream(docFile.getText());
	    DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	    dbf.setNamespaceAware(true);
	    dbf.setValidating(true);
	    dbf.setAttribute("http://apache.org/xml/features/validation/schema",Boolean.TRUE);
	    DocumentBuilder db=dbf.newDocumentBuilder();
	    XMLErrorHandler errors=new XMLErrorHandler();
	    db.setErrorHandler(errors);
	    Document doc=db.parse(in);
	    StringBuffer errorBuffer=new StringBuffer();
	    if(!errors.validated()){ // if there are errors, they must be logged.
		Iterator i=errors.getFatalErrors();
		if(errors.getNumberOfFatalErrors()!=0){
		    errorBuffer.append("Fatal errors: "+errors.getNumberOfFatalErrors()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
		i=errors.getErrors();
		if(errors.getNumberOfErrors()!=0){
		    errorBuffer.append("Errors: "+errors.getNumberOfErrors()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
		i=errors.getWarnings();
		if(errors.getNumberOfWarnings()!=0){
		    errorBuffer.append("Warnings: "+errors.getNumberOfWarnings()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
	    }
	    HashSet topics=new HashSet();
	    NodeList langs=doc.getElementsByTagNameNS
		("http://sammael.tabu.stw-bonn.de/schema/documentation","language");
	    for(int i=0;i<langs.getLength();i++){
		NodeList nl=((Element)langs.item(i)).getElementsByTagName("topic");
		for(int j=0;j<nl.getLength();j++){
		    String s=nl.item(j).getAttributes().getNamedItem("id").getNodeValue();
		    if(!topics.contains(s)){
			topics.add(s);
		    }
		}
	    }
	    langs=doc.getElementsByTagNameNS
		("http://sammael.tabu.stw-bonn.de/schema/documentation","language");
	    for(int i=0;i<langs.getLength();i++){
		String language=langs.item(i).getAttributes().getNamedItem("xml:lang").getNodeValue();
		NodeList nl=((Element)langs.item(i)).getElementsByTagName("topic");
		HashSet myTopics=new HashSet();
		for(int j=0;j<nl.getLength();j++){
		    String s=nl.item(j).getAttributes().getNamedItem("id").getNodeValue();
		    if(!myTopics.contains(s)){
			myTopics.add(s);
		    }
		}
		Iterator it=topics.iterator();
		while(it.hasNext()){
		    String s=it.next().toString();
		    if(!myTopics.contains(s)){
			errorBuffer.append("Language "+language+" does not contain the topic with id "+s+".\n");
		    }
		}
	    }
	    if(errorBuffer.length()==0) errorBuffer.append("Congratulations, no errors!\n");
	    new TextWindow(errorBuffer.toString()).show();
	}catch(Exception e){
	    e.printStackTrace();
	}
    }

    // validates it
    private void validateConfiguration(){
	try{
	    InputStream in;
	    if(confJar.isSelected()) in=WebPageHelper.class.getResourceAsStream(confFile.getText());
	    else in=new FileInputStream(confFile.getText());
	    DocumentBuilderFactory dbf=DocumentBuilderFactory.newInstance();
	    dbf.setNamespaceAware(true);
	    dbf.setValidating(true);
	    dbf.setAttribute("http://apache.org/xml/features/validation/schema",Boolean.TRUE);
	    DocumentBuilder db=dbf.newDocumentBuilder();
	    XMLErrorHandler errors=new XMLErrorHandler();
	    db.setErrorHandler(errors);
	    Document doc=db.parse(in);
	    StringBuffer errorBuffer=new StringBuffer();
	    if(!errors.validated()){ // if there are errors, they must be logged.
		Iterator i=errors.getFatalErrors();
		if(errors.getNumberOfFatalErrors()!=0){
		    errorBuffer.append("Fatal errors: "+errors.getNumberOfFatalErrors()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
		i=errors.getErrors();
		if(errors.getNumberOfErrors()!=0){
		    errorBuffer.append("Errors: "+errors.getNumberOfErrors()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
		i=errors.getWarnings();
		if(errors.getNumberOfWarnings()!=0){
		    errorBuffer.append("Warnings: "+errors.getNumberOfWarnings()+"\n");
		    while(i.hasNext()) errorBuffer.append(i.next()+"\n");
		}
	    }
	    in.close();
	    if(confJar.isSelected()) in=WebPageHelper.class.getResourceAsStream(confFile.getText());
	    else in=new FileInputStream(confFile.getText());
	    KeyStrokeOption.resetValidityCheck();
	    ConfigurationDialog dlg=new ConfigurationDialog(null,"eng",in);
	    KeyStrokeOption[] o=KeyStrokeOption.checkValidity();
	    if(o!=null) errorBuffer.append("The options \""+o[0]+"\" and \""+o[1]+"\" have the same keystroke.\n");
	    if(errorBuffer.length()==0) errorBuffer.append("Congratulations, no errors!\n");
	    new TextWindow(errorBuffer.toString()).show();
	}catch(Exception e){
	    e.printStackTrace();
	}
    }

    // centers it
    private void center(){
	pack();
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension fs = getSize();
        setLocation((ss.width-fs.width)/2,(ss.height-fs.height)/2);
    }

    /**
     * Run me.
     * @param args a <code>String[]</code> value.
     */
    public static void main( String[] args ){
	new WebPageHelper().show();
    }

    /**
     * A class that displays a text as JDialog.
     */
    class TextWindow extends JDialog{
	/**
	 * Constructs one.
	 *@param s the string to be displayed.
	 */
	public TextWindow( String s ){
	    setTitle("Validation results:");
	    JTextArea text=new JTextArea(s);
	    JScrollPane panel=new JScrollPane(text);
	    getContentPane().add(panel);
	    pack();
	    setLocationRelativeTo(null);
	}
    }

}
